

local M = {}

local logPile = {}
logPile[1] = {}
logPile[2] = {}

DEBUG_LOG_COUNT = 100

local logVariables = {}
logVariables.currentPile = 1
logVariables.logIndex = 1


local serverManager = require(cc.PACKAGE_NAME..".managers.serverManager")

function M.getLogPile()
	return logPile
end

function M.getLogVariables()
	return logVariables
end

function M.myformat( fmt, ... )
	local buf = {}
	for i = 1, select( '#', ... ) do
		local a = select( i, ... )
		if type( a ) ~= 'string' and type( a ) ~= 'number' then
			a = tostring( a )
		end
		buf[i] = a
	end
	return string.format( fmt, unpack( buf ) )
end

function M.logBacktrace()

	local function tableToString(pile)
		local result = ''
		local pileSize = #pile
		for i = 1, pileSize do
			local item = pile[i]
         if type(item) ~= 'string' and type(item) ~= 'number' then
				item = tostring(item)
         end
			result = result..item..'\n'
		end
		return result
	end

	local pile1, pile2 = tableToString(logPile[1]), tableToString(logPile[2])

	return pile1..pile2
end

function logBacktrace()
	return M.logBacktrace()
end

function M.logSaveData()
    if persistenceInternal then
        return json.encode(persistenceInternal)
    else
        return ""
    end
end

function logSaveData()
	return M.logSaveData()
end

function M.sendLogToServer(errorData)
    echoInfo("Sending error report to the giant goat in the sky...")
    echoInfo("running LuaHacks version:%s", HACKS_VERSION or "0")

    local errorMessage
    local traceback

    if errorData then
        errorMessage = errorData[1] ~= nil and "LUA ERROR: "..errorData[1] or ""
        traceback = errorData[2] ~= nil and errorData[2] or ""
    else
        errorMessage = "LUA ERROR: goat"
        traceback = "No actual error data provided, so no traceback for you"
    end

    local params = {}
    if errorData then
        params["code"] = tonumber(errorData[1])
    else
        params["code"] = -1
    end
    params["version"] = device.getVersionAsNumber()
    params["build"] = runaway.Native:getVersion()
    params["android"] = (device.platform == "android")
    params["model"] = device.getDeviceModel()
    params["msg"] = errorMessage
    params["trace"] = traceback
    params["userdata"] = logSaveData()
    params["console"] = logBacktrace(false)
    params["hacksversion"] = HACKS_VERSION

    local function onResult(result, data)
        echoInfo(result)
    end
    
    serverManager.performRequest("Log", params, nil, onResult, true)
end

function sendLogToServer(errorData)
	M.sendLogToServer(errorData)
end

function M.storeLog(log)

   if logVariables.logIndex > DEBUG_LOG_COUNT then
      logVariables.logIndex = 1
      if logVariables.currentPile == 1 then 
         logVariables.currentPile = 2 
      else 
         logVariables.currentPile = 1 
      end
      logPile[logVariables.currentPile] = {}
   end

   logVariables.logIndex = logVariables.logIndex + 1
   local currentTable = logPile[logVariables.currentPile]
   currentTable[logVariables.logIndex] = log
end

function storeLog(log)
	M.storeLog(log)
end

function M.echo(log)
	storeLog(log)
	release_print(log)
end

function echo(log)
	M.echo(log)
end

local inspect
if DEBUG > 0 then
    inspect = require(cc.PACKAGE_NAME..".libraries.inspect")
end

function M.print(fmt, ...)
	if DEBUG_LOG_LEVEL and DEBUG_LOG_LEVEL > 0 then

		-- NO SAVE TO SERVER
		if DEBUG_LOG_LEVEL == 1 then release_print(fmt, ...) return end

		-- JUST FIRST BIT
		if DEBUG_LOG_LEVEL == 2 then echoLog("PRINT", fmt) return end

		-- FULL MESSY LOGS
		if DEBUG_LOG_LEVEL == 3 then
			echoLog("PRINT", fmt)
			local count = select('#', ...)
			for i = 1, count do
				local item = select(i, ...)
				echoLog("---->", item)
			end
			return
		end

		-- FULL CLEAN LOGS
		if DEBUG_LOG_LEVEL == 4 then

		   local numArgs = select( '#', ... )

		   local temp, numArgsNeeded = string.gsub(fmt, "%%", "%%")

		   local newArgs = {}
		   for i = 1, numArgsNeeded do
		      if i <= numArgs then
		         newArgs[i] = select(i, ...)
		      else
		         newArgs[i] = "!"
		      end
		   end
		   local extras = " "
		   for i = numArgsNeeded + 1, numArgs do
		      local extra = select(i, ... )
		      if type(extra) ~= 'string' and type(extra) ~= 'number' then
		         extra = tostring(extra)
		      end
		      extras = extras.." "..extra
		   end
		   echoLog("PRINT", fmt..extras, unpack( newArgs ) )
		end

	end 
end

function print(fmt, ...)
	M.print(fmt, ...) 
end


function M.iprint(thing)
    if DEBUG > 0 then
        release_print(inspect.inspect(thing))
    end
end

function iprint(thing)
	M.iprint(thing)
end

function M.echoError(fmt, ...)
	echoLog("ERR", fmt, ...)
	echoCallStack()
end

function echoError(fmt, ...)
	M.echoError(fmt, ...)
end

function M.echoCallStack()
    release_print(debug.traceback("", 2))
end

function echoCallStack()
	M.echoCallStack()
end

function M.echoInfo(fmt, ...)
	echoLog("INFO", fmt, ...)
end

function echoInfo(fmt, ...)
	M.echoInfo(fmt, ...)
end

function M.echoLog(tag, fmt, ...)
	local t = { "[", string.upper(tostring(tag)), "] ", M.myformat(tostring(fmt), ...) }
	release_print(table.concat(t))
	local msg = string.format("[%s] %s", string.upper(tostring(tag)), string.format(tostring(fmt), ...))
	storeLog(msg)
	if crashManager ~= nil then
		crashManager.leaveBreadcrumb(msg)
	end
end

function echoLog(tag, fmt, ...)
	M.echoLog(tag, fmt, ...)
end

return M