
require "lfs"

local device = {}

local sharedApplication = cc.Application:getInstance()
local target = sharedApplication:getTargetPlatform()


-- we want to move away from the code below....
device.platform    = "unknown"
device.model       = "unknown"
device.isAmazon    = false

if target == cc.PLATFORM_OS_WINDOWS then
    device.platform = "windows"
elseif target == cc.PLATFORM_OS_MAC then
    device.platform = "mac"
elseif target == cc.PLATFORM_OS_ANDROID then
    device.platform = "android"
elseif target == cc.PLATFORM_OS_IPHONE or target == cc.PLATFORM_OS_IPAD then
    device.platform = "ios"
    if target == cc.PLATFORM_OS_IPHONE then
        device.model = "iphone"
    else
        device.model = "ipad"
    end
end

-- and replace it with this
function device.initNativeBindings()
    if sharedApplication:getTargetPlatform() == cc.PLATFORM_OS_ANDROID then
        luaj = require("cocos.cocos2d.luaj")  
    else
        luaoc = require("cocos.cocos2d.luaoc")
    end
end

device.initNativeBindings()
device.client_platform = "unknown"
device.model = "unknown"

if target == cc.PLATFORM_OS_ANDROID then
    local ok,ret = luaj.callStaticMethod("com/runawayplay/Platform", "isAmazon", nil, "()Z")
    if ok and ret then
        device.client_platform = "amazon"
        device.model = "amazon"
    else
        device.client_platform = "android"
        device.model = "android"
    end
elseif target == cc.PLATFORM_OS_IPHONE then
    device.client_platform = "ios"
    device.model = "iphone"
elseif target == cc.PLATFORM_OS_IPAD then
    device.client_platform = "ios"
    device.model = "ipad"
end

local language_ = sharedApplication:getCurrentLanguage()
if language_ == cc.LANGUAGE_CHINESE then
    language_ = "cn"
elseif language_ == cc.LANGUAGE_FRENCH then
    language_ = "fr"
elseif language_ == cc.LANGUAGE_ITALIAN then
    language_ = "it"
elseif language_ == cc.LANGUAGE_GERMAN then
    language_ = "gr"
elseif language_ == cc.LANGUAGE_SPANISH then
    language_ = "sp"
elseif language_ == cc.LANGUAGE_RUSSIAN then
    language_ = "ru"
elseif (language_ == cc.LANGUAGE_KOREAN) or (language_ == 8) then
    language_ = "ko"
elseif (language_ == cc.LANGUAGE_JAPANESE) or (language_ == 9) then
    language_ = "ja"
elseif (language_ == cc.LANGUAGE_PORTUGUESE) or (language_ == 11) then
    language_ = "pt"
else
    language_ = "en"
end

device.language = language_
device.writablePath = cc.FileUtils:getInstance():getWritablePath()
device.directorySeparator = "/"
device.pathSeparator = ":"
if device.platform == "windows" then
    device.directorySeparator = "\\"
    device.pathSeparator = ";"
end

echoInfo("# device.platform              = " .. device.platform)
echoInfo("# device.model                 = " .. device.model)
echoInfo("# device.language              = " .. device.language)
echoInfo("# device.writablePath          = " .. device.writablePath)
echoInfo("# device.directorySeparator    = " .. device.directorySeparator)
echoInfo("# device.pathSeparator         = " .. device.pathSeparator)
echoInfo("#")


function device.checkIsAmazon()
    -- deprecated. Check device.client_platform instead
    if device.platform == "android" then
        local ok,ret = luaj.callStaticMethod("com/runawayplay/Platform", "isAmazon", nil, "()Z")
        if ok then
            device.isAmazon = ret
            echoInfo("isAmazon")
            return
        end
    end

    echoInfo("isAmazon false")

    return false
end
device.checkIsAmazon()

function device.registerForRemoteNotifications()
    runaway.Native:registerForRemoteNotifications()
end

function device.sendHTMLEmail(subject, body, recipient)
    local actualRecipient = recipient
    if actualRecipient == nil then
        actualRecipient = ""
    end
    runaway.Native:sendHTMLEmail(subject, body, actualRecipient)
end

function device.sendSMS(body)
    runaway.Native:sendSMS(body)
end

function device.canSendSMS()
    return runaway.Native:canDisplaySMS()
end

function device.canDisplayWebView()
    return runaway.Native:canDisplayWebView()
end

function device.canOpenURL(url)
    return runaway.Native:canOpenURL(url)
end

function device.openURL(url)
    runaway.Native:openURL(url)
end

function device.openURLWithFallbacks(urlTable)
    for i = 1,#urlTable do
        local url = urlTable[i]
        if device.canOpenURL(url) then

            print('Opening URL:', url)
            device.openURL(url)
            return
        else
            print('OS cannot open URL scheme:', url)
        end
    end
    print('OS could not open any of the URL schemes in the table.')
end

function device.isLauncherAppInstalled()
    if device.client_platform == 'android' or device.client_platform == "amazon" then
        return runaway.Native:canOpenURL(URL_RUNAWAY_LAUNCHER_APP_ANDROID)
    else
        return runaway.Native:canOpenURL(URL_RUNAWAY_LAUNCHER_APP_IOS)
    end
end

function device.getDeviceManufacturer()
    return runaway.Native:getDeviceManufacturer()
end

function device.getDeviceProduct()
    return runaway.Native:getDeviceProduct()
end

function device.getDeviceModel()
    return runaway.Native:getDeviceModel()
end

function device.getDeviceIdentifier()
    return runaway.Native:getDeviceIdentifier()
end

function device.getVendorIdentifier()
    return runaway.Native:getVendorIdentifier()
end

function device.getAdvertiserIdentifier()
    return runaway.Native:getAdvertiserIdentifier()
end

function device.getVersionInformationString()
    return runaway.Native:getVersionInformationString()
end

function device.getVersionString()
    return runaway.Native:getVersionString()
end

function device.getVersionAsNumber()
    local fullVersionString = runaway.Native:getVersionString()
    local words = {}
    for word in fullVersionString:gmatch("%w+") do table.insert(words, word) end
    local versionString = words[1]
    local versionLump = versionString:gsub("%.", "")
    local versionNumber = tonumber(versionLump)
    return versionNumber
end

function device.getSystemVersion()
    return runaway.Native:getSystemVersion()
end

function device.systemVersionGreaterThan(versionString, isEqual)
    return runaway.Native:systemVersionGreaterThan(versionString, isEqual)
end

function device.getRealWidth()
    return runaway.Native:getRealWidth()
end

function device.getRealHeight()
    return runaway.Native:getRealHeight()
end

--buttonLabels={{type=-1,label="yeh"}, {type=-2,label="nah"}, {type=-3,label="maybe"}}
-- -1 = positive
-- -2 = negative
-- -3 = neutral
--Must have a negative
function device.showAlert(title, message, buttonLabels, listener)

    local alertview = runaway.AlertView:getInstance()

    local function getCancelLabel()
        for i = 1,#buttonLabels do
            local labelObj = buttonLabels[i]
            if labelObj["type"] == -2 then
                return i
            end
        end
        echoInfo("No cancel label")
        return nil
    end

    if #buttonLabels > 0 then
        cancelLabelIndex = getCancelLabel()
        cancelLabel = buttonLabels[cancelLabelIndex]["label"]
        table.remove(buttonLabels, cancelLabelIndex)
    end

    alertview:createAlert(title, message, cancelLabel)
    for i = 1,#buttonLabels do
        local label = buttonLabels[i]
        alertview:addAlertButton(label["type"], label["label"])
    end

    if type(listener) ~= "function" then
        listener = function() end
    end


    alertview:registerEventHandler(function(resultStr, dataStr)
        alertview:unregisterEventHandler()
        listener(result, dataStr)
    end)

    alertview:showAlert(listener)
end

function device.cancelAlert()
    runaway.AlertView:getInstance():cancelAlert()
end

function device.showInputBox(title, message, defaultValue)
    title = title or "INPUT TEXT"
    message = message or "INPUT TEXT, CLICK OK BUTTON"
    defaultValue = defaultValue or ""
    --TODO
    -- return CCNative:getInputText(title, message, defaultValue)
end

function device.pathExists(path)
    local mode = lfs.attributes(path, 'mode')
    if mode then
        return true
    else
        return false
    end
end


function device.ensureFolderExists(path)
    -- echoInfo("Ensuring path exists:%s", path)
    local mode = lfs.attributes(path, "mode")
    if mode then
        echoInfo(path .. " exists and is a " .. mode .. ".")
        return true;
    else
        echoInfo(path .. " does not exist.  Creating it.")
        return lfs.mkdir(path)
    end
end

function device.removePath(path)
    --  works for both files and folders
    local mode = lfs.attributes(path, "mode")
    if mode then
        if mode == "directory" then

            for file in lfs.dir(path) do
                if (file ~= ".") and (file ~= "..") then
                    device.removePath(path.."/"..file)
                end
            end

            --  this is a directory, so delete all of the subfiles/folders
            echoInfo("Deleting %s : %s", mode, path)
            local success, result = lfs.rmdir(path)
            if (success) then 
                return true;
            else
                echoInfo("Could not remove %s\n  Reason => %s", path, result or "")
            end
        else
            --   this is a file, just delete it.
            echoInfo("Deleting %s : %s", mode, path)
            os.remove(path)
        end
    end

end

function device.copyFile(sourcePath, destPath)
    local mode = lfs.attributes(sourcePath, "mode")
    if not mode then
        echoInfo("Could not find file: %s", sourcePath)
        return false
    end
    if mode == "directory" then
        echoInfo("Could not copy: %s: is a directory", sourcePath)
        return false
    end

    local sourceFile = io.open(sourcePath, "rb")
    local destFile = io.open(destPath, "wb")

    local data = sourceFile:read("*a")
    if not (data) then
        echoInfo("Read error!")
        return false
    else
        if not (destFile:write(data)) then
            echoInfo("Write error!")
            return false
        end
    end

    sourceFile:close()
    destFile:close()

    return true
end

function device.getFilesInPath(path, fullPath)
    local files = {}
    for file in lfs.dir(path) do
        if lfs.attributes(path..file,"mode") == "file" then
            table.insert(files, fullPath and path..file or file)
        end
    end
    return files
end

function device.getIsTablet()
	-- the only tablets we recognise are ipads
        return device.model == "ipad" 
end

function device.convertNumberToCurrency(currency, locale, amount)
    return runaway.Native:convertNumberToCurrency(currency, locale, amount)
end
------------------------------

function device.saveImageToPhotos(imagePath, callback)
    if device.client_platform == "ios" then
        luaoc.callStaticMethod("AppController", "savePhotoToLibrary", {imagePath=imagePath})
        
        -- ios always succeeds
        if callback ~= nil then
            callback(runaway.PluginResult.kSucceed, imagePath, nil)
        end
    else
        local listener

        -- Callback for when the photo is saved to device
        local function onSavePhotoToLibraryResult(event)
            -- Event is a userData object - extract result table from it
            local resultString = event and event:getUserDataString() or nil
            local outerJson = resultString and json.decode(resultString) or nil
            local resultTable = outerJson and json.decode(outerJson.result) or {}

            -- Extract fields from json or use defaults if not present in json
            local code = resultTable.code or runaway.PluginResult.kFailed
            local retImagePath = resultTable.imagePath or ""
            local resultUrl = resultTable.resultUrl or ""

            -- Check if this is the request we sent
            if retImagePath == imagePath then
                -- Stop listening for calback
                callbackManager.stopListeningForCallback(listener)
                -- Fire local callback
                if callback ~= nil then
                    callback(code, retImagePath, resultUrl)
                end
            end
        end

        -- Listen for save photo callback
        listener = callbackManager.listenForCallback(EVENT_SAVE_PHOTO_TO_LIBRARY_RESULT, onSavePhotoToLibraryResult)

        local args = {imagePath}
        local ok,ret = luaj.callStaticMethod("com/runawayplay/Platform", "savePhotoToLibrary", args, "(Ljava/lang/String;)Z")
    end
end
------------------------------

function device.nativeShareImage(imagePath, message)
    if device.client_platform == "ios" then
        luaoc.callStaticMethod("AppController", "shareImage", {imagePath=imagePath, message=message})
    else
        local args = {imagePath, message}
        local ok,ret = luaj.callStaticMethod("com/runawayplay/Platform", "shareImage", args, "(Ljava/lang/String;Ljava/lang/String;)Z")
    end
end
------------------------------

return device
