local display    = {}

local director = cc.Director:getInstance()
local spriteFrameCache = cc.SpriteFrameCache:getInstance()
local view = director:getOpenGLView()
local textureCache = cc.Director:getInstance():getTextureCache()
display.isSetup = false;

function display.init()
    print("display.init()")

    display.registerMemoryWarningEvent()
    display.setup()
end

-- auto scale
function display.checkResolution(r)
    r.width = checknumber(r.width)
    r.height = checknumber(r.height)
    r.autoscale = string.upper(r.autoscale)
    assert(r.width > 0 and r.height > 0,
        string.format("display - invalid design resolution size %d, %d", r.width, r.height))
end

function display.setDesignResolution(_r, framesize)
    local r = clone(_r)
    if display.orientation ~= CONFIG_SCREEN_DEFAULT then
        r.width = _r.height
        r.height = _r.width
    end

    if r.autoscale == "FILL_ALL" then
        view:setDesignResolutionSize(framesize.width, framesize.height, cc.ResolutionPolicy.FILL_ALL)
    else
        local scaleX, scaleY = framesize.width / r.width, framesize.height / r.height
        local width, height = framesize.width, framesize.height
        if r.autoscale == "FIXED_WIDTH" then
            width = framesize.width / scaleX
            height = framesize.height / scaleX
            view:setDesignResolutionSize(width, height, cc.ResolutionPolicy.NO_BORDER)
        elseif r.autoscale == "FIXED_HEIGHT" then
            width = framesize.width / scaleY
            height = framesize.height / scaleY
            view:setDesignResolutionSize(width, height, cc.ResolutionPolicy.NO_BORDER)
        elseif r.autoscale == "EXACT_FIT" then
            view:setDesignResolutionSize(r.width, r.height, cc.ResolutionPolicy.EXACT_FIT)
        elseif r.autoscale == "NO_BORDER" then
            view:setDesignResolutionSize(r.width, r.height, cc.ResolutionPolicy.NO_BORDER)
        elseif r.autoscale == "SHOW_ALL" then
            view:setDesignResolutionSize(r.width, r.height, cc.ResolutionPolicy.SHOW_ALL)
        else
            echoError(string.format("display - invalid r.autoscale \"%s\"", r.autoscale))
        end
    end

    echoInfo(string.format("# design resolution size       = {width = %0.2f, height = %0.2f}", r.width, r.height))
    echoInfo(string.format("# design resolution autoscale  = %s", r.autoscale))
end

function display.setConstants()
    local viewsize = director:getWinSize()
    display.contentScaleFactor = director:getContentScaleFactor()
    display.size               = {width = viewsize.width, height = viewsize.height}
    display.width              = display.size.width
    display.height             = display.size.height
    display.cx                 = display.width / 2
    display.cy                 = display.height / 2
    display.c_left             = -display.width / 2
    display.c_right            = display.width / 2
    display.c_top              = display.height / 2
    display.c_bottom           = -display.height / 2
    display.left               = 0
    display.right              = display.width
    display.top                = display.height
    display.bottom             = 0
    display.center             = cc.p(display.cx, display.cy)
    display.left_top           = cc.p(display.left, display.top)
    display.left_bottom        = cc.p(display.left, display.bottom)
    display.left_center        = cc.p(display.left, display.cy)
    display.right_top          = cc.p(display.right, display.top)
    display.right_bottom       = cc.p(display.right, display.bottom)
    display.right_center       = cc.p(display.right, display.cy)
    display.top_center         = cc.p(display.cx, display.top)
    display.top_bottom         = cc.p(display.cx, display.bottom)       -- Should be bottom_center?
    display.widthInPixels      = display.sizeInPixels.width
    display.heightInPixels     = display.sizeInPixels.height

    echoInfo(string.format("# display.sizeInPixels         = {width = %0.2f, height = %0.2f}", display.sizeInPixels.width, display.sizeInPixels.height))
    echoInfo(string.format("# display.size                 = {width = %0.2f, height = %0.2f}", display.size.width, display.size.height))
    echoInfo(string.format("# display.contentScaleFactor   = %0.2f", display.contentScaleFactor))
    echoInfo(string.format("# display.width                = %0.2f", display.width))
    echoInfo(string.format("# display.height               = %0.2f", display.height))
    echoInfo(string.format("# display.cx                   = %0.2f", display.cx))
    echoInfo(string.format("# display.cy                   = %0.2f", display.cy))
    echoInfo(string.format("# display.left                 = %0.2f", display.left))
    echoInfo(string.format("# display.right                = %0.2f", display.right))
    echoInfo(string.format("# display.top                  = %0.2f", display.top))
    echoInfo(string.format("# display.bottom               = %0.2f", display.bottom))
    echoInfo(string.format("# display.c_left               = %0.2f", display.c_left))
    echoInfo(string.format("# display.c_right              = %0.2f", display.c_right))
    echoInfo(string.format("# display.c_top                = %0.2f", display.c_top))
    echoInfo(string.format("# display.c_bottom             = %0.2f", display.c_bottom))
    echoInfo(string.format("# display.center               = {x = %0.2f, y = %0.2f}", display.center.x, display.center.y))
    echoInfo(string.format("# display.left_top             = {x = %0.2f, y = %0.2f}", display.left_top.x, display.left_top.y))
    echoInfo(string.format("# display.left_bottom          = {x = %0.2f, y = %0.2f}", display.left_bottom.x, display.left_bottom.y))
    echoInfo(string.format("# display.left_center          = {x = %0.2f, y = %0.2f}", display.left_center.x, display.left_center.y))
    echoInfo(string.format("# display.right_top            = {x = %0.2f, y = %0.2f}", display.right_top.x, display.right_top.y))
    echoInfo(string.format("# display.right_bottom         = {x = %0.2f, y = %0.2f}", display.right_bottom.x, display.right_bottom.y))
    echoInfo(string.format("# display.right_center         = {x = %0.2f, y = %0.2f}", display.right_center.x, display.right_center.y))
    echoInfo(string.format("# display.top_center           = {x = %0.2f, y = %0.2f}", display.top_center.x, display.top_center.y))
    echoInfo(string.format("# display.top_bottom           = {x = %0.2f, y = %0.2f}", display.top_bottom.x, display.top_bottom.y))
    echoInfo(string.format("# display.orientation          = %s",    display.orientation))

    echoInfo("#")

    ---------------------
    ----- SAFE AREA -----
    ---------------------

    -- A rectangular region defined relative to the bounds of the game display. The safeArea 
    -- is guaranteed to be unobscured by any system status bars or controls. UI elements are 
    -- positioned relative to this region.
    display.safeArea                    = {}

    -- The amount of space between each of the edges of the safeArea and the edge of the 
    -- game display.
    display.safeArea.leftInset          = CONFIG_SAFE_AREA_INSETS and CONFIG_SAFE_AREA_INSETS.left or 0
    display.safeArea.rightInset         = CONFIG_SAFE_AREA_INSETS and CONFIG_SAFE_AREA_INSETS.right or 0
    display.safeArea.bottomInset        = CONFIG_SAFE_AREA_INSETS and CONFIG_SAFE_AREA_INSETS.bottom or 0
    display.safeArea.topInset           = CONFIG_SAFE_AREA_INSETS and CONFIG_SAFE_AREA_INSETS.top or 0

    -- SafeArea sizes
    display.safeArea.width              = (viewsize.width - display.safeArea.leftInset) - display.safeArea.rightInset
    display.safeArea.height             = (viewsize.height - display.safeArea.topInset) - display.safeArea.bottomInset
    display.safeArea.size               = { width = display.safeArea.width, height = display.safeArea.height }

    -- Significant points and distances of safeArea
    display.safeArea.left               = display.safeArea.leftInset
    display.safeArea.right              = display.width - display.safeArea.rightInset
    display.safeArea.top                = display.height - display.safeArea.topInset
    display.safeArea.bottom             = display.safeArea.bottomInset
    display.safeArea.cx                 = (display.safeArea.left + display.safeArea.right) / 2
    display.safeArea.cy                 = (display.safeArea.bottom + display.safeArea.top) / 2
    display.safeArea.cxToAbsoluteLeft   = (display.safeArea.width / 2) + display.safeArea.leftInset
    display.safeArea.cxToAbsoluteRight  = (display.safeArea.width / 2) + display.safeArea.rightInset
    display.safeArea.cyToAbsoluteTop    = (display.safeArea.height / 2) + display.safeArea.topInset
    display.safeArea.cyToAbsoluteBottom = (display.safeArea.height / 2) + display.safeArea.bottomInset
    display.safeArea.cxToAbsoluteCenter = display.cx - display.safeArea.cx
    display.safeArea.cyToAbsoluteCenter = display.cy - display.safeArea.cy
    display.safeArea.center             = cc.p(display.safeArea.cx, display.safeArea.cy)
    display.safeArea.left_top           = cc.p(display.safeArea.left, display.safeArea.top)
    display.safeArea.left_bottom        = cc.p(display.safeArea.left, display.safeArea.bottom)
    display.safeArea.left_center        = cc.p(display.safeArea.left, display.safeArea.cy)
    display.safeArea.right_top          = cc.p(display.safeArea.right, display.safeArea.top)
    display.safeArea.right_bottom       = cc.p(display.safeArea.right, display.safeArea.bottom)
    display.safeArea.right_center       = cc.p(display.safeArea.right, display.safeArea.cy)
    display.safeArea.top_center         = cc.p(display.safeArea.cx, display.safeArea.top)
    display.safeArea.bottom_center      = cc.p(display.safeArea.cx, display.safeArea.bottom)

    -- Pixel measurements
    local safeLeftPercent                   = display.safeArea.left / display.width
    local safeRightPercent                  = display.safeArea.right / display.width
    local safeTopPercent                    = display.safeArea.top / display.height
    local safeBottomPercent                 = display.safeArea.bottom / display.height

    display.safeArea.leftInPixels       = math.floor(display.sizeInPixels.width * safeLeftPercent)
    display.safeArea.rightInPixels      = math.floor(display.sizeInPixels.width * safeRightPercent)
    display.safeArea.topInPixels        = math.floor(display.sizeInPixels.height * safeTopPercent)
    display.safeArea.bottomInPixels     = math.floor(display.sizeInPixels.height * safeBottomPercent)
    display.safeArea.widthInPixels      = math.floor(display.sizeInPixels.width * (safeRightPercent - safeLeftPercent))
    display.safeArea.heightInPixels     = math.floor(display.sizeInPixels.height * (safeTopPercent - safeBottomPercent))
    display.safeArea.cxInPixels         = math.floor(display.safeArea.leftInPixels + (display.safeArea.widthInPixels / 2))
    display.safeArea.cyInPixels         = math.floor(display.safeArea.bottomInPixels + (display.safeArea.heightInPixels / 2))
    display.safeArea.sizeInPixels       = { width = display.safeArea.widthInPixels, height = display.safeArea.heightInPixels }

    echoInfo(string.format("# display.safeArea.size                 = {width = %0.2f, height = %0.2f}", display.safeArea.size.width, display.safeArea.size.height))
    echoInfo(string.format("# display.safeArea.width                = %0.2f", display.safeArea.width))
    echoInfo(string.format("# display.safeArea.height               = %0.2f", display.safeArea.height))
    echoInfo(string.format("# display.safeArea.cx                   = %0.2f", display.safeArea.cx))
    echoInfo(string.format("# display.safeArea.cy                   = %0.2f", display.safeArea.cy))
    echoInfo(string.format("# display.safeArea.left                 = %0.2f", display.safeArea.left))
    echoInfo(string.format("# display.safeArea.right                = %0.2f", display.safeArea.right))
    echoInfo(string.format("# display.safeArea.top                  = %0.2f", display.safeArea.top))
    echoInfo(string.format("# display.safeArea.bottom               = %0.2f", display.safeArea.bottom))
    echoInfo(string.format("# display.safeArea.cxToAbsoluteLeft     = %0.2f", display.safeArea.cxToAbsoluteLeft))
    echoInfo(string.format("# display.safeArea.cxToAbsoluteRight    = %0.2f", display.safeArea.cxToAbsoluteRight))
    echoInfo(string.format("# display.safeArea.cyToAbsoluteTop      = %0.2f", display.safeArea.cyToAbsoluteTop))
    echoInfo(string.format("# display.safeArea.cyToAbsoluteBottom   = %0.2f", display.safeArea.cyToAbsoluteBottom))
    echoInfo(string.format("# display.safeArea.cxToAbsoluteCenter   = %0.2f", display.safeArea.cxToAbsoluteCenter))
    echoInfo(string.format("# display.safeArea.cyToAbsoluteCenter   = %0.2f", display.safeArea.cyToAbsoluteCenter))
    echoInfo(string.format("# display.safeArea.center               = {x = %0.2f, y = %0.2f}", display.safeArea.center.x, display.safeArea.center.y))
    echoInfo(string.format("# display.safeArea.left_top             = {x = %0.2f, y = %0.2f}", display.safeArea.left_top.x, display.safeArea.left_top.y))
    echoInfo(string.format("# display.safeArea.left_bottom          = {x = %0.2f, y = %0.2f}", display.safeArea.left_bottom.x, display.safeArea.left_bottom.y))
    echoInfo(string.format("# display.safeArea.left_center          = {x = %0.2f, y = %0.2f}", display.safeArea.left_center.x, display.safeArea.left_center.y))
    echoInfo(string.format("# display.safeArea.right_top            = {x = %0.2f, y = %0.2f}", display.safeArea.right_top.x, display.safeArea.right_top.y))
    echoInfo(string.format("# display.safeArea.right_bottom         = {x = %0.2f, y = %0.2f}", display.safeArea.right_bottom.x, display.safeArea.right_bottom.y))
    echoInfo(string.format("# display.safeArea.right_center         = {x = %0.2f, y = %0.2f}", display.safeArea.right_center.x, display.safeArea.right_center.y))
    echoInfo(string.format("# display.safeArea.top_center           = {x = %0.2f, y = %0.2f}", display.safeArea.top_center.x, display.safeArea.top_center.y))
    echoInfo(string.format("# display.safeArea.bottom_center        = {x = %0.2f, y = %0.2f}", display.safeArea.bottom_center.x, display.safeArea.bottom_center.y))
    echoInfo(string.format("# display.safeArea.leftInPixels         = %0.2f", display.safeArea.leftInPixels))
    echoInfo(string.format("# display.safeArea.rightInPixels        = %0.2f", display.safeArea.rightInPixels))
    echoInfo(string.format("# display.safeArea.topInPixels          = %0.2f", display.safeArea.topInPixels))
    echoInfo(string.format("# display.safeArea.bottomInPixels       = %0.2f", display.safeArea.bottomInPixels))
    echoInfo(string.format("# display.safeArea.widthInPixels        = %0.2f", display.safeArea.widthInPixels))
    echoInfo(string.format("# display.safeArea.heightInPixels       = %0.2f", display.safeArea.heightInPixels))
    echoInfo(string.format("# display.safeArea.cxInPixels           = %0.2f", display.safeArea.cxInPixels))
    echoInfo(string.format("# display.safeArea.cyInPixels           = %0.2f", display.safeArea.cyInPixels))

    echoInfo("#")
end

function display.setScale(dlcSize)
    local configs = CC_DESIGN_SCALE
    if type(configs) ~= "table" then return end

    local scale = 1
    if configs.autoscale == "FIXED_WIDTH" then
        scale = display.sizeInPixels.width / configs.width
    elseif configs.autoscale == "FIXED_HEIGHT" then
        scale = display.sizeInPixels.height / configs.height
    elseif configs.autoscale == "EXACT_FIT" then
        echoError(string.format("display - invalid r.autoscale \"%s\"", configs.autoscale))
    elseif configs.autoscale == "NO_BORDER" then
        echoError(string.format("display - invalid r.autoscale \"%s\"", configs.autoscale))
    elseif configs.autoscale == "SHOW_ALL" then
        echoError(string.format("display - invalid r.autoscale \"%s\"", configs.autoscale))
    else
        echoError(string.format("display - invalid r.autoscale \"%s\"", configs.autoscale))
    end

    local lastSize
    local selectedSize = CONFIG_DEVICE_EXCEPTIONS[runaway.Native:getDeviceModel()] -- assign assets if device needs shittier assets
    if selectedSize == nil then -- otherwise work out assets based on size
        for i = 1,#CONFIG_RESOURCE_SIZE do
            local size = CONFIG_RESOURCE_SIZE[i]
            local maxContentScale = size.scale or 99999
            if scale < maxContentScale then
                selectedSize = size
                break
            end
            lastSize = size
        end
        
        if lastSize then 
            selectedSize = lastSize
        end
    end

    -- if dlc scale doesn't match what was loaded from res, then force res to use the dlc scale
    if dlcSize and dlcSize ~= selectedSize.scale then
        local newPaths = {}
        local oldPaths = cc.FileUtils:getInstance():getOriginalSearchPaths()
        for i, searchPath in ipairs(oldPaths) do
            if searchPath ~= "res/4x/"
            and searchPath ~= "res/2x/"
            and searchPath ~= "res/1x/" then

                table.insert(newPaths, clone(oldPaths[i]))
            end
        end
        echoInfo("res asset scale doesn't match dlc asset scale, setting res asset scale to dlc asset scale")
        table.insert(newPaths, string.format("res/%dx/", dlcSize))
        cc.FileUtils:getInstance():setSearchPaths(newPaths)
        director:setContentScaleFactor(dlcSize)
    else
        cc.FileUtils:getInstance():addSearchPath(selectedSize.path)
        director:setContentScaleFactor(selectedSize.scale)
    end
end


function display.setupOrientation(framesize)
    if framesize.width > framesize.height then
        display.orientation = ORI_LANDSCAPE
    else
        display.orientation = ORI_PORTRAIT
    end
    ORIENTATION = display.orientation
end

function display.setup(dlcSize)


    if dlcSize then
        echoInfo("Checking res asset scale against dlc asset scale.")
        display.isSetup = false
    end

    local framesize = view:getFrameSize()

    local configs = CC_DESIGN_RESOLUTION

    if type(configs) ~= "table" then return end

    display.checkResolution(configs)
    if type(configs.callback) == "function" then
        local c = configs.callback(framesize)
        for k, v in pairs(c or {}) do
            configs[k] = v
        end
        display.checkResolution(configs)
    end

    display.setupOrientation(framesize)

    display.setDesignResolution(configs, framesize)

    local sizeInPixels = view:getFrameSize()
    display.sizeInPixels = {width = sizeInPixels.width, height = sizeInPixels.height}

    if not display.isSetup then
        display.setScale(dlcSize)
        if PVR_PREMULTIPLIED_ALPHA ~= nil then
            cc.Image:setPVRImagesHavePremultipliedAlpha(PVR_PREMULTIPLIED_ALPHA)
        end
        display.isSetup = true
    end
    display.setConstants()

end

display.ANCHOR_POINTS = {
    cc.p(0.5, 0.5),  -- CENTER
    cc.p(0, 1),      -- TOP_LEFT
    cc.p(0.5, 1),    -- TOP_CENTER
    cc.p(1, 1),      -- TOP_RIGHT
    cc.p(0, 0.5),    -- CENTER_LEFT
    cc.p(1, 0.5),    -- CENTER_RIGHT
    cc.p(0, 0),      -- BOTTOM_LEFT
    cc.p(1, 0),      -- BOTTOM_RIGHT
    cc.p(0.5, 0),    -- BOTTOM_CENTER
}

function display.wrapSceneWithTransition(scene, transitionType, time, more)
    local key = string.upper(tostring(transitionType))
    if string.sub(key, 1, 12) == "CCTRANSITION" then
        key = string.sub(key, 13)
    end

    if key == "RANDOM" then
        local keys = table.keys(display.SCENE_TRANSITIONS)
        key = keys[math.random(1, #keys)]
    end

    if display.SCENE_TRANSITIONS[key] then
        local cls, count, default = unpack(display.SCENE_TRANSITIONS[key])
        time = time or 0.2

        if count == 3 then
            scene = cls:create(time, scene, more or default)
        else
            scene = cls:create(time, scene)
        end
    else
        echoError("display.wrapSceneWithTransition() - invalid transitionType %s", tostring(transitionType))
    end
    return scene
end

function display.replaceScene(newScene, transitionType, time, more)
    callbackManager.sendCallback(EVENT_WILL_CHANGE_SCENE, {newScene = newScene})
    display.runScene(newScene, transitionType, time, more)
end

function display.pause()
    director:pause()
end

function display.resume()
    director:resume()
end

function display.newLayerColor(colorC4B)
    colorC4B["a"] = colorC4B["a"]/255
    return display.newLayer(colorC4B)
end

function display.newScale9Sprite(source, x, y, size)
    local params = {}
    params["scale9"] = true
    return display.newSprite(source, x, y, params)
end

function display.newBatchNode(image, capacity)
    return cc.SpriteBatchNode:create(image, capacity or 100)
end

function display.memoryWarningRecieved()
    echoInfo(EVENT_APPLICATION_MEMORY_WARNING)
end

function display.memoryWarningElapsed()
    echoInfo(EVENT_APPLICATION_MEMORY_WARNING_ELAPSED)
end

function display.registerMemoryWarningEvent()
    local eventDispatcher = director:getEventDispatcher()
    local listener1 = cc.EventListenerCustom:create(EVENT_APPLICATION_MEMORY_WARNING,display.memoryWarningRecieved)
    eventDispatcher:addEventListenerWithFixedPriority(listener1, 1)

    local listener2 = cc.EventListenerCustom:create(EVENT_APPLICATION_MEMORY_WARNING_ELAPSED,display.memoryWarningElapsed)
    eventDispatcher:addEventListenerWithFixedPriority(listener2, 1)
end

display.PROGRESS_TIMER_BAR = cc.PROGRESS_TIMER_TYPE_BAR
display.PROGRESS_TIMER_RADIAL = cc.PROGRESS_TIMER_TYPE_RADIAL

function display.newProgressTimer(image, progresssType)
    if type(image) == "string" then
        image = display.newSprite(image)
    end

    local progress = cc.ProgressTimer:create(image)
    progress:setType(progresssType)
    return progress
end


-- Below is from cocos/framework/display
display.COLOR_WHITE = cc.c3b(255, 255, 255)
display.COLOR_BLACK = cc.c3b(0, 0, 0)
display.COLOR_RED   = cc.c3b(255, 0, 0)
display.COLOR_GREEN = cc.c3b(0, 255, 0)
display.COLOR_BLUE  = cc.c3b(0, 0, 255)

display.AUTO_SIZE      = 0
display.FIXED_SIZE     = 1
display.LEFT_TO_RIGHT  = 0
display.RIGHT_TO_LEFT  = 1
display.TOP_TO_BOTTOM  = 2
display.BOTTOM_TO_TOP  = 3

display.CENTER        = cc.p(0.5, 0.5)
display.LEFT_TOP      = cc.p(0, 1)
display.LEFT_BOTTOM   = cc.p(0, 0)
display.LEFT_CENTER   = cc.p(0, 0.5)
display.RIGHT_TOP     = cc.p(1, 1)
display.RIGHT_BOTTOM  = cc.p(1, 0)
display.RIGHT_CENTER  = cc.p(1, 0.5)
display.CENTER_TOP    = cc.p(0.5, 1)
display.CENTER_BOTTOM = cc.p(0.5, 0)

display.SCENE_TRANSITIONS = {
    CROSSFADE       = cc.TransitionCrossFade,
    FADE            = {cc.TransitionFade, cc.c3b(0, 0, 0)},
    FADEBL          = cc.TransitionFadeBL,
    FADEDOWN        = cc.TransitionFadeDown,
    FADETR          = cc.TransitionFadeTR,
    FADEUP          = cc.TransitionFadeUp,
    FLIPANGULAR     = {cc.TransitionFlipAngular, cc.TRANSITION_ORIENTATION_LEFT_OVER},
    FLIPX           = {cc.TransitionFlipX, cc.TRANSITION_ORIENTATION_LEFT_OVER},
    FLIPY           = {cc.TransitionFlipY, cc.TRANSITION_ORIENTATION_UP_OVER},
    JUMPZOOM        = cc.TransitionJumpZoom,
    MOVEINB         = cc.TransitionMoveInB,
    MOVEINL         = cc.TransitionMoveInL,
    MOVEINR         = cc.TransitionMoveInR,
    MOVEINT         = cc.TransitionMoveInT,
    PAGETURN        = {cc.TransitionPageTurn, false},
    ROTOZOOM        = cc.TransitionRotoZoom,
    SHRINKGROW      = cc.TransitionShrinkGrow,
    SLIDEINB        = cc.TransitionSlideInB,
    SLIDEINL        = cc.TransitionSlideInL,
    SLIDEINR        = cc.TransitionSlideInR,
    SLIDEINT        = cc.TransitionSlideInT,
    SPLITCOLS       = cc.TransitionSplitCols,
    SPLITROWS       = cc.TransitionSplitRows,
    TURNOFFTILES    = cc.TransitionTurnOffTiles,
    ZOOMFLIPANGULAR = cc.TransitionZoomFlipAngular,
    ZOOMFLIPX       = {cc.TransitionZoomFlipX, cc.TRANSITION_ORIENTATION_LEFT_OVER},
    ZOOMFLIPY       = {cc.TransitionZoomFlipY, cc.TRANSITION_ORIENTATION_UP_OVER},
}

display.TEXTURES_PIXEL_FORMAT = {}

display.DEFAULT_TTF_FONT        = "Arial"
display.DEFAULT_TTF_FONT_SIZE   = 32


local PARAMS_EMPTY = {}
local RECT_ZERO = cc.rect(0, 0, 0, 0)

local sceneIndex = 0
function display.newScene(name, params)
    params = params or PARAMS_EMPTY
    sceneIndex = sceneIndex + 1
    local scene
    if not params.physics then
        scene = cc.Scene:create()
    else
        scene = cc.Scene:createWithPhysics()
    end
    scene.name_ = string.format("%s:%d", name or "<unknown-scene>", sceneIndex)

    if params.transition then
        scene = display.wrapSceneWithTransition(scene, params.transition, params.time, params.more)
    end

    return scene
end

function display.wrapScene(scene, transition, time, more)
    local key = string.upper(tostring(transition))

    if key == "RANDOM" then
        local keys = table.keys(display.SCENE_TRANSITIONS)
        key = keys[math.random(1, #keys)]
    end

    if display.SCENE_TRANSITIONS[key] then
        local t = display.SCENE_TRANSITIONS[key]
        time = time or 0.2
        more = more or t[2]
        if type(t) == "table" then
            scene = t[1]:create(time, scene, more)
        else
            scene = t:create(time, scene)
        end
    else
        error(string.format("display.wrapScene() - invalid transition %s", tostring(transition)))
    end
    return scene
end

function display.runScene(newScene, transition, time, more)
    if director:getRunningScene() then
        if transition then
            newScene = display.wrapScene(newScene, transition, time, more)
        end
        director:replaceScene(newScene)
    else
        director:runWithScene(newScene)
    end
end

function display.getRunningScene()
    return director:getRunningScene()
end

function display.newNode()
    return cc.Node:create()
end

function display.newLayer(...)
    local params = {...}
    local c = #params
    local layer
    if c == 0 then
        -- /** creates a fullscreen black layer */
        -- static Layer *create();
        layer = cc.Layer:create()
    elseif c == 1 then
        -- /** creates a Layer with color. Width and height are the window size. */
        -- static LayerColor * create(const Color4B& color);
        layer = cc.LayerColor:create(cc.convertColor(params[1], "4b"))
    elseif c == 2 then
        -- /** creates a Layer with color, width and height in Points */
        -- static LayerColor * create(const Color4B& color, const Size& size);
        --
        -- /** Creates a full-screen Layer with a gradient between start and end. */
        -- static LayerGradient* create(const Color4B& start, const Color4B& end);
        local color1 = cc.convertColor(params[1], "4b")
        local p2 = params[2]
        assert(type(p2) == "table" and (p2.width or p2.r), "display.newLayer() - invalid paramerter 2")
        if p2.r then
            layer = cc.LayerGradient:create(color1, cc.convertColor(p2, "4b"))
        else
            layer = cc.LayerColor:create(color1, p2.width, p2.height)
        end
    elseif c == 3 then
        -- /** creates a Layer with color, width and height in Points */
        -- static LayerColor * create(const Color4B& color, GLfloat width, GLfloat height);
        --
        -- /** Creates a full-screen Layer with a gradient between start and end in the direction of v. */
        -- static LayerGradient* create(const Color4B& start, const Color4B& end, const Vec2& v);
        local color1 = cc.convertColor(params[1], "4b")
        local p2 = params[2]
        local p2type = type(p2)
        if p2type == "table" then
            layer = cc.LayerGradient:create(color1, cc.convertColor(p2, "4b"), params[3])
        else
            layer = cc.LayerColor:create(color1, p2, params[3])
        end
    end
    return layer
end

function display.newSprite(source, x, y, params)
    local spriteClass = cc.Sprite
    local scale9 = false

    if type(x) == "table" and not x.x then
        -- x is params
        params = x
        x = nil
        y = nil
    end

    local params = params or PARAMS_EMPTY
    if params.scale9 or params.capInsets then
        spriteClass = ccui.Scale9Sprite
        scale9 = true
        params.capInsets = params.capInsets or RECT_ZERO
        params.rect = params.rect or RECT_ZERO
    end

    local sprite
    while true do
        -- create sprite
        if not source then
            sprite = spriteClass:create()
            break
        end

        local sourceType = type(source)
        if sourceType == "string" then
            if string.byte(source) == 35 then -- first char is #
                -- create sprite from spriteFrame
                local spriteFrameName = string.sub(source, 2)
                if not scale9 then
                    if textureManager and textureManager.SMART_TEXTURE_LOADING then
                        -- Get spriteFrame from textureManager (will get from spriteFrameCache, loading texture from catalogue first if needed)
                        local spriteFrame = textureManager.getOrLoadSpriteFrame(spriteFrameName)
                        if spriteFrame then sprite = spriteClass:createWithSpriteFrame(spriteFrame) end
                    else
                        sprite = spriteClass:createWithSpriteFrameName(spriteFrameName)
                    end
                else
                    sprite = spriteClass:createWithSpriteFrameName(spriteFrameName, params.capInsets)
                end
                break
            end

            -- create sprite from image file
            if display.TEXTURES_PIXEL_FORMAT[source] then
                cc.Texture2D:setDefaultAlphaPixelFormat(display.TEXTURES_PIXEL_FORMAT[source])
            end
            if not scale9 then
                sprite = spriteClass:create(source, params.rect)
            else
                sprite = spriteClass:create(source, params.rect, params.capInsets)
            end
            if display.TEXTURES_PIXEL_FORMAT[source] then
                cc.Texture2D:setDefaultAlphaPixelFormat(cc.TEXTURE2_D_PIXEL_FORMAT_BGR_A8888)
            end
            break
        elseif sourceType ~= "userdata" then
            error(string.format("display.newSprite() - invalid source type \"%s\"", sourceType), 0)
        else
            sourceType = tolua.type(source)
            if sourceType == "cc.SpriteFrame" then
                if not scale9 then
                    sprite = spriteClass:createWithSpriteFrame(source)
                else
                    sprite = spriteClass:createWithSpriteFrame(source, params.capInsets)
                end
            elseif sourceType == "cc.Texture2D" then
                sprite = spriteClass:createWithTexture(source)
            else
                error(string.format("display.newSprite() - invalid source type \"%s\"", sourceType), 0)
            end
        end
        break
    end

    if sprite then
        if x and y then sprite:setPosition(x, y) end
        if params.size then sprite:setContentSize(params.size) end
    else
        error(string.format("display.newSprite() - create sprite failure, source \"%s\"", tostring(source)), 0)
    end

    return sprite
end

function display.newSpriteFrame(source, ...)
    local frame
    if type(source) == "string" then
        if string.byte(source) == 35 then -- first char is #
            source = string.sub(source, 2)
        end
        frame = spriteFrameCache:getSpriteFrame(source)
        if not frame then
            error(string.format("display.newSpriteFrame() - invalid frame name \"%s\"", tostring(source)), 0)
        end
    elseif tolua.type(source) == "cc.Texture2D" then
        frame = cc.SpriteFrame:createWithTexture(source, ...)
    else
        error("display.newSpriteFrame() - invalid parameters", 0)
    end
    return frame
end

function display.newFrames(pattern, begin, length, isReversed)
    local frames = {}
    local step = 1
    local last = begin + length - 1
    if isReversed then
        last, begin = begin, last
        step = -1
    end

    for index = begin, last, step do
        local frameName = string.format(pattern, index)
        local frame = spriteFrameCache:getSpriteFrame(frameName)
        if not frame then
            error(string.format("display.newFrames() - invalid frame name %s", tostring(frameName)), 0)
        end
        frames[#frames + 1] = frame
    end
    return frames
end

local function newAnimation(frames, time)
    local count = #frames
    assert(count > 0, "display.newAnimation() - invalid frames")
    time = time or 1.0 / count
    return cc.Animation:createWithSpriteFrames(frames, time),
           cc.Sprite:createWithSpriteFrame(frames[1])
end

function display.newAnimation(...)
    local params = {...}
    local c = #params
    if c == 2 then
        -- frames, time
        return newAnimation(params[1], params[2])
    elseif c == 4 then
        -- pattern, begin, length, time
        local frames = display.newFrames(params[1], params[2], params[3])
        return newAnimation(frames, params[4])
    elseif c == 5 then
        -- pattern, begin, length, isReversed, time
        local frames = display.newFrames(params[1], params[2], params[3], params[4])
        return newAnimation(frames, params[5])
    else
        error("display.newAnimation() - invalid parameters")
    end
end

function display.loadImage(imageFilename, callback)
    if not callback then
        return textureCache:addImage(imageFilename)
    else
        textureCache:addImageAsync(imageFilename, callback)
    end
end

local fileUtils = cc.FileUtils:getInstance()
function display.getImage(imageFilename)
    local fullpath = fileUtils:fullPathForFilename(imageFilename)
    return textureCache:getTextureForKey(fullpath)
end

function display.removeImage(imageFilename)
    textureCache:removeTextureForKey(imageFilename)
end

function display.loadSpriteFrames(dataFilename, imageFilename, callback)
    if display.TEXTURES_PIXEL_FORMAT[imageFilename] then
        cc.Texture2D:setDefaultAlphaPixelFormat(display.TEXTURES_PIXEL_FORMAT[imageFilename])
    end
    if not callback then
        spriteFrameCache:addSpriteFrames(dataFilename, imageFilename)
    else
        spriteFrameCache:addSpriteFramesAsync(dataFilename, imageFilename, callback)
    end
    if display.TEXTURES_PIXEL_FORMAT[imageFilename] then
        cc.Texture2D:setDefaultAlphaPixelFormat(cc.TEXTURE2_D_PIXEL_FORMAT_BGR_A8888)
    end
end

function display.removeSpriteFrames(dataFilename, imageFilename)
    spriteFrameCache:removeSpriteFramesFromFile(dataFilename)
    if imageFilename then
        display.removeImage(imageFilename)
    end
end

function display.removeSpriteFrame(imageFilename)
    spriteFrameCache:removeSpriteFrameByName(imageFilename)
end

function display.setTexturePixelFormat(imageFilename, format)
    display.TEXTURES_PIXEL_FORMAT[imageFilename] = format
end

function display.setAnimationCache(name, animation)
    animationCache:addAnimation(animation, name)
end

function display.getAnimationCache(name)
    return animationCache:getAnimation(name)
end

function display.removeAnimationCache(name)
    animationCache:removeAnimation(name)
end

function display.removeUnusedSpriteFrames()
    spriteFrameCache:removeUnusedSpriteFrames()
    textureCache:removeUnusedTextures()
end

return display
