require "cocos.init"
require ("configServer")
require ("framework.constantsBoot")
require ("lfs")
local network = require("framework.network")
local json = require("framework.json")
local socketUrl = require("socket.socket.url")
local director = cc.Director:getInstance()
local sharedScheduler = director:getScheduler()
local backgroundColor = cc.c4b(0, 0, 0, 255)
local loadingFont = FONT_ARIAL_ITALIC
local loadingFontColor = cc.c3b(255, 255, 255)
local imageShown = true

local firstLoader = {}

local GET_SETTINGS_URL = string.format("%s/settings",SERVER_URL)


function firstLoader.init(performLuaBoot)
	firstLoader.performLuaBoot = performLuaBoot
	if performLuaBoot then
		network.setTimeoutForRead(15)       --  default timeout is 1 minute, reduce to 15 secs
		firstLoader.platform = firstLoader.getPlatform()
		firstLoader.dlcZipPath = runaway.FileHelpers:getDlcPath()
		firstLoader.ensureFolderExists(firstLoader.dlcZipPath)
		firstLoader.dlcLuaPath = firstLoader.dlcZipPath .. "luaboot/"
	end
end


function firstLoader.runFirstLoad(onCompleteCallback)
	print("firstLoader.runFirstLoad()")
	if firstLoader.performLuaBoot then
		firstLoader.buildUI()
		firstLoader.onCompleteCallback = onCompleteCallback
		firstLoader.startLoading()
	else
		onCompleteCallback()
	end
end

function firstLoader.startLoading()
	firstLoader.canRestart = false
	print("getting settings from ".. GET_SETTINGS_URL)
	if runaway.Native:isInternetAvailable() then
		firstLoader.PerformRequest(HTTPREQUEST_GET, GET_SETTINGS_URL, nil, nil, firstLoader.onGetVersions)
	else
		firstLoader.onFailed()
	end
end

function firstLoader.onGetVersions(result, values)
	dump(result, "firstLoader.onGetVersions.result")
	dump(values, "firstLoader.onGetVersions.values")
	if result.code == 0 and values ~= nil then -- kSucceed
		local thisBuild = runaway.Native:getVersion()
		local dlcVersions = {}
		local requiredDlcVersion = 0

		-- grab the relevant values out of the settings blob
		if values.settings then
			firstLoader.settingsList = values
			if firstLoader.platform == "amazon" then				
				dlcVersions = values.settings.dlcVersionsAmazon or {}
			elseif firstLoader.platform == "android" then
				dlcVersions = values.settings.dlcVersionsAndroid or {}
			else
				dlcVersions = values.settings.dlcVersionsiOS or {}
			end
		else
			dump(values, "firstLoader missing settings (values)")
		end

		dump(dlcVersions, "firstLoader -> dlcVersions")
			--  find the requried DLC version
		--  sort the dlcVersions table in ascending order of "maxBuild" property
		table.sort(dlcVersions, function(lhs,rhs) return (lhs.maxBuild or 0) < (rhs.maxBuild or 0) end)
		for i = 1,#dlcVersions do
			local version = dlcVersions[i]
			if version.maxBuild <= thisBuild then
				requiredDlcVersion = version.dlcVersion
			end
		end
		print ("found requiredDlcVersion ".. (requiredDlcVersion or "NULL"))
		if values.dlc_url and requiredDlcVersion then

			return firstLoader.getLuaDlc(values.dlc_url, requiredDlcVersion)
		else
			print ("firstLoader didn't find DLC to download for build #".. thisBuild)
			firstLoader.onCompleted()
		end
	else
		firstLoader.onFailed()
	end
end


function firstLoader.getLuaDlc(dlcBaseUrl, version)
	firstLoader.loadingLabel:setString("Loading.")
	print("firstLoader.getLuaDlc")
    local zipEtag = firstLoader.getLastEtag()

	local dlcUrl = dlcBaseUrl .. version .. "/luaboot.zip"
	-- first, delete the luaboot path, and then re-create it before downloading
	firstLoader.zipDestPath = firstLoader.dlcZipPath .. "luaboot.zip"

	print("loading from ".. dlcUrl)
	print("saving to ".. firstLoader.zipDestPath)
	-- TODO add ETAG support.
	firstLoader.downloadFile(dlcUrl, firstLoader.zipDestPath, "luaboot.zip", zipEtag, firstLoader.onGetLuaZip)
end

function firstLoader.resetLuaPath()
	-- clear out the old lua folder
	firstLoader.removePath(firstLoader.dlcLuaPath);
	firstLoader.ensureFolderExists(firstLoader.dlcLuaPath)
end

function firstLoader.onGetLuaZip(result, data)
	firstLoader.loadingLabel:setString("Loading..")
	dump(result, "firstLoader.onGetLuaZip result")
	dump(data, "firstLoader.onGetLuaZip data")
	if result.code == 0 then
		print("firstLoader - download Zip completed")
		if firstLoader.unzipFile(firstLoader.zipDestPath) then
			-- if we succeeded, then add the lua unzipped folder to the search paths
				print("adding lua search path"..firstLoader.dlcLuaPath)
	        cc.FileUtils:getInstance():addSearchPath(firstLoader.dlcLuaPath, true)

	        local etag = nil
	        if result.headers then
	        	if result.headers["Etag"] ~= nil then
		            etag = result.headers["Etag"]
		        else
		            etag = result.headers["ETag"]
		        end
		        firstLoader.storeEtag(etag)
		    end
		    firstLoader.onCompleted()
		else
			print("firstLoader unable to extract luaboot.zip")
			firstLoader.onFailed()			
		end
	
	elseif result.code == HTTPREQUEST_SUCCESS_NOT_MODIFIED then
		--	it's already been installed, so just add the search path
		print("loader Zip already installed.  skipping")
        cc.FileUtils:getInstance():addSearchPath(firstLoader.dlcLuaPath, true)
	    firstLoader.onCompleted()
	elseif result.code == HTTPREQUEST_ERROR_FORBIDDEN or result.code == HTTPREQUEST_ERROR_NOT_FOUND then 
		--	S3 returns http:403 instead of 404 if a file doesn't exist
		print("firstLoader didn't find a luaboot.zip  - removing any old files")
		firstLoader.resetLuaPath()
	    firstLoader.onCompleted()
	else
		firstLoader.onFailed()
	end
end

function firstLoader.onFailed()
	print("firstLoader failed.  Allowing restart")
--	if network.isInternetConnectionAvailable() then
		firstLoader.loadingLabel:setString("Unable to check for updates.\nTap to retry.")
	-- else
	-- 	firstLoader.loadingLabel:setString("Unable to connect to the Internet.\nPlease connect and tap to retry.")
	-- end
	firstLoader.canRestart = true
end

function firstLoader.onRestartClick()
	print("tap")
	if firstLoader.canRestart then
		firstLoader.startLoading()
	end
end


function firstLoader.onCompleted()
	firstLoader.loadingLabel:setString("Loading...")
	print("=========================")
	print("firstLoader.onCompleted()")
	print("=========================")
	firstLoader.performWithDelayGlobal(firstLoader.onCompleteCallback, 0)
end


------------------------------------
---------- UI Methods ----------
------------------------------------
-- function ui.newTTFLabel(params)
--     assert(type(params) == "table",
--            "[framework.ui] newTTFLabel() invalid params")

--     local text       = tostring(params.text)
--     local font       = params.font or ui.DEFAULT_TTF_FONT
--     local size       = params.size or ui.DEFAULT_TTF_FONT_SIZE
--     local color      = params.color or display.COLOR_WHITE
--     local textAlign  = params.align or ui.TEXT_ALIGN_LEFT
--     local textValign = params.valign or ui.TEXT_VALIGN_CENTER
--     local x, y       = params.x, params.y
--     local dimensions = params.dimensions

--     assert(type(size) == "number",
--            "[framework.ui] newTTFLabel() invalid params.size")

--     local label
--     if dimensions then
--         label = cc.LabelTTF:create(text, font, size, dimensions, textAlign, textValign)
--     else
--         label = cc.LabelTTF:create(text, font, size)
--     end

--     if label then
--         -- CCNodeExtend.extend(label)
--         label:setColor(color)

--         function label:realign(x, y)
--             if textAlign == ui.TEXT_ALIGN_LEFT then
--                 label:setPosition(math.round(x + label:getContentSize().width / 2), y)
--             elseif textAlign == ui.TEXT_ALIGN_RIGHT then
--                 label:setPosition(x - math.round(label:getContentSize().width / 2), y)
--             else
--                 label:setPosition(x, y)
--             end
--         end

--         if x and y then label:realign(x, y) end
--     end

--     return label
-- end

function firstLoader.buildUI()
	if firstLoader.sceneNode == nil then
		local view = director:getOpenGLView()
		local framesize = view:getFrameSize()
		firstLoader.sceneNode = cc.Scene:create()
		if director:getRunningScene() then
			print ("firstloader replaceScene")
			director:replaceScene(firstLoader.sceneNode)
		else
			print ("firstloader runWithScene")
			director:runWithScene(firstLoader.sceneNode)
		end

		--	white background
		firstLoader.touchLayer = ccui.Layout:create()
		firstLoader.touchLayer:setBackGroundColorType(ccui.LayoutBackGroundColorType.solid)
		firstLoader.touchLayer:setBackGroundColor(backgroundColor)
		firstLoader.touchLayer:setPosition(0,0)
		firstLoader.touchLayer:setAnchorPoint(cc.p(0,0))
		firstLoader.touchLayer:setContentSize(framesize)
		firstLoader.touchLayer:setTouchEnabled(true)
		firstLoader.touchLayer:addTouchEventListener(firstLoader.onRestartClick)
		firstLoader.sceneNode:addChild(firstLoader.touchLayer)

		--	runaway logo
		-- local fileUtils = cc.FileUtils:getInstance()
	 --    local fullpath = fileUtils:fullPathForFilename("runaway_logo_small.png")
		-- local logoImage = cc.Sprite:create(fullpath)
	 --    logoImage:setPosition(framesize.width/2, framesize.height/2)
	 --    firstLoader.sceneNode:addChild(logoImage)

	 	local fontSize = 30
	 	if framesize.width > 1000 then
	 		fontSize = 50
	 	end
	    -- message text
		firstLoader.loadingLabel = cc.LabelTTF:create("Loading", FONT_ARIAL_ITALIC, fontSize)
		firstLoader.loadingLabel:setColor(loadingFontColor) 
		firstLoader.loadingLabel:setPosition(framesize.width/2, framesize.height/2)
		firstLoader.sceneNode:addChild(firstLoader.loadingLabel)

		-- handle removing load screen on android!!
		firstLoader.performWithDelayGlobal(function()
			runaway.Native:hideLoadingImage()
			imageShown = false
		end, 0)			
	end
end




------------------------------------
---------- Helper methods ----------
------------------------------------
function firstLoader.getPlatform()
	local sharedApplication = cc.Application:getInstance()
	local target = sharedApplication:getTargetPlatform()
	local platform = ""
	if target == cc.PLATFORM_OS_ANDROID then
	    platform = "android"
    	luaj = require("cocos.cocos2d.luaj")  
        local ok,ret = luaj.callStaticMethod("com/runawayplay/Platform", "isAmazon", nil, "()Z")
        if ok then
            platform = "amazon"
        end
	elseif target == cc.PLATFORM_OS_IPHONE or target == cc.PLATFORM_OS_IPAD then
	    platform = "ios"
   		luaoc = require("cocos.cocos2d.luaoc")
	end
	return platform
end


function firstLoader.getStateFilename()
    return string.gsub(cc.FileUtils:getInstance():getWritablePath(), "[\\\\/]+$", "") .. "/firstLoader"
end

function firstLoader.getLastEtag()
	local filename = firstLoader.getStateFilename()
	local etag = nil
	if io.exists(filename) then
		etag = io.readfile(filename)
	end
	print ("loading last etag = ".. (etag or "BLANK"))
	return etag
end

function firstLoader.storeEtag(etag)
	print ("storing etag = ".. (etag or "BLANK"))
	local filename = firstLoader.getStateFilename()
	io.writefile(filename, etag)
end

function firstLoader.removePath(path)
    --  works for both files and folders
    print("removing path:" .. path)
    local mode = lfs.attributes(path, "mode")
    if mode then
        if mode == "directory" then

            for file in lfs.dir(path) do
                if (file ~= ".") and (file ~= "..") then
                    firstLoader.removePath(path.."/"..file)
                end
            end

            --  this is a directory, so delete all of the subfiles/folders
            print(string.format("Deleting %s : %s", mode, path))
            local success, result = lfs.rmdir(path)
            if (success) then 
                return true;
            else
                print(string.format("Could not remove %s\n  Reason => %s", path, result or ""))
            end
        else
            --   this is a file, just delete it.
            print(string.format("Deleting %s : %s", mode, path))
            os.remove(path)
        end
    end

end


function firstLoader.ensureFolderExists(path)
	local mode = lfs.attributes(path, "mode")
	if mode then
		print(path .. " exists and is a " .. mode .. ".")
		return true;
	else
		print(path .. " does not exist.  Creating it.")
		return lfs.mkdir(path)
	end
end


function firstLoader.unzipFile(filePath)
	print ("Unzipping file "..filePath);
	--local dlcPath = device.writablePath;

	-- delete the folder to ensure that we get a clean install
	firstLoader.resetLuaPath()

	--  if this is a resource file, then extract it to the appropriate 1x, 2x, 3x, 4x folder
	-- NOTE: Not doing this anymore
	-- if (file.res) then
	--     dlcPath = dlcPath .. M.getFileScaleFactor(file, true);
	-- end
	print ("extracting file:" .. filePath .. "\n to path:"..firstLoader.dlcLuaPath)
	local results = runaway.FileHelpers:extractZipFile(filePath, firstLoader.dlcLuaPath);
	local resultData = json.decode(results);
	if (resultData.success) then
		print ("Unzipping file "..filePath..":\n successful");
	else
		print ("Unzipping file "..filePath..":\n failed");
	end
	print ("  into folder:\n   ".. firstLoader.dlcLuaPath)
	return resultData.success
end

function firstLoader.downloadFile(url, destPath, fileId, etag, callback)
	local retry = nil
	local attempts = 0
	local function onFileProgress(result, downloaded, total)
		-- firstLoader.onDownloadProgressUpdate(result, downloaded, )
	end
	local function onFileComplete(result, req)
		if result.code == HTTPREQUEST_SUCCESS or result.code == HTTPREQUEST_SUCCESS_NOT_MODIFIED then
			print("Download complete:\n".. url);
			-- self:onDownloadProgressCompleteFile(result, req, file)
			dump(req, "onFileComplete.req")
			callback(result)
		elseif result.code == HTTPREQUEST_ERROR_FORBIDDEN 
			or result.code == HTTPREQUEST_ERROR_NOT_FOUND then
			-- HTTPREQUEST_ERROR_FORBIDDEN is returned if the file doesn't exist in S3
			callback(result)
		else

			print ("Download Failed with error:")
			print (result.code)
			if attempts <= 5 then
				retry()
			else
				callback(result)
			end
		end
	end

	retry = function()
		attempts = attempts + 1
		local req = network.createHTTPFileRequest(onFileProgress, onFileComplete, url, destPath, fileId)
		if etag then
			local etagQuery = [[If-None-Match: "]]..etag..[["]]
			req:setHeaders({etagQuery})
		end
		network.getFile(req)
	end
	retry()
end

function firstLoader.PerformRequest(method, url, arguments, headers, callback)	
	local retry = nil
	local attempts = 0

	if not callback then
		error("PerformRequest needs a callback")
	end

	function checkIsJson(headers)
		if type(headers) == "table" then
			for k, header in pairs(headers) do
				if string.find(header, "application/json") then
					return true
				end
			end
		end
		return false
	end

	local function getHttpValues(headers, response_body)
		local values = {}
		if response_body ~= nil then
			if(checkIsJson(headers)) then
				values = json.decode(response_body)
			else
				for key, value in string.gmatch(response_body, "([^&=]+)=([^&=]*)&?" ) do
					--print( ("key=%s, value=%s"):format(key, value) )
					values[key] = socketUrl.unescape(value)
				end
			end
		end
		return values
	end

	local function onHttpResponse(result, txt, request, response)
		print("onHttpResponse")
		if result.code == HTTPREQUEST_SUCCESS then
			print("firstLoader.PerformRequest success from " .. (url or "UNKNOWN"))
			local values = getHttpValues(result.headers, txt)
			-- dump(values, "values")
			callback(result, values)
		else
			if (result.httpCode == 404) then -- don't retry on 404 error
				callback(result)
			else
				print("firstLoader.PerformRequest failure from " .. (url or "UNKNOWN"))
				dump(result, "onHttpResponse.result")
				dump(response, "response")
				print("attempts=".. attempts or "??")
				if attempts <= 3 then
					retry()
				else
					callback(result)
				end
			end
		end
	end

	local headers = {}
    headers["Accept"] = "application/json;"
	-- local reqHeaders = {}
	-- for k, header in pairs(headers) do
	--  	table.insert(reqHeaders, tostring(k)..":"..tostring(header))
	-- end    
    -- reqHeaders["Content-Type"] = "application/json; charset=utf8"
    retry = function()
		attempts = attempts + 1
		print(method)
		-- dump(reqHeaders, "reqHeaders")
	    local req = network.createHTTPRequest(onHttpResponse, url, method, "firstLoader", nil, headers)
	    print("sending request")
		network.send(req)
	end

	retry()
end

function firstLoader.performWithDelayGlobal(callback, time)
   local newCallback
   newCallback = sharedScheduler:scheduleScriptFunc(function()
      sharedScheduler:unscheduleScriptEntry(newCallback)
      callback() 
   end, time, false)
   return newCallback
end

return firstLoader
