if type(DEBUG) ~= "number" then DEBUG = 1 end
local CURRENT_MODULE_NAME = ...


cc = cc or {}
cc.PACKAGE_NAME = string.sub(CURRENT_MODULE_NAME, 1, -6)

require "cocos.init"
require(cc.PACKAGE_NAME .. ".constants")

require(cc.PACKAGE_NAME .. ".debug")
require(cc.PACKAGE_NAME .. ".functions_runaway")

require("cocos.framework.extends.NodeEx")
require("cocos.framework.extends.SpriteEx")
require("cocos.framework.extends.LayerEx")
require("cocos.framework.extends.MenuEx")

echoInfo("")
echoInfo("# DEBUG                        = "..DEBUG)
echoInfo("#")

if DEBUG_COVERAGE ~= nil and DEBUG_COVERAGE then
	require(cc.PACKAGE_NAME .. ".libraries.coverage")
end

--Global variable check
if DEBUG_GLOBALS ~= nil and DEBUG_GLOBALS then
	local declaredNames = {}
	function declare (name, initval)
		rawset(_G, name, initval)
		declaredNames[name] = true
	end
	setmetatable(_G, {
		__newindex = function (t, n, v)
			if not declaredNames[n] then
				print("attempt to write to undeclared var. "..n)
				print(debug.traceback("", 1))
				declare(n, v)
			else
				rawset(t, n, v)   -- do the actual set
			end
		end,
		__index = function (_, n)
			if not declaredNames[n] then
				print("attempt to read undeclared var. "..n)
			else
				return nil
			end
		end,
	})

	function getDeclaredNames()
		return declaredNames
	end
end

device     			= require(cc.PACKAGE_NAME .. ".device")
display    			= require(cc.PACKAGE_NAME .. ".display")
display.init()
network    			= require(cc.PACKAGE_NAME .. ".network")
ui         			= require(cc.PACKAGE_NAME .. ".ui")
crypto     			= require(cc.PACKAGE_NAME .. ".crypto")
json       			= require(cc.PACKAGE_NAME .. ".json")
xml        			= require(cc.PACKAGE_NAME .. ".xml")
Localize   			= require(cc.PACKAGE_NAME .. ".libraries.Localize")
vector2d   			= require(cc.PACKAGE_NAME .. ".libraries.vector2d")
mailboxManager 		= require(cc.PACKAGE_NAME .. ".managers.mailboxManager")
scheduler           = require(cc.PACKAGE_NAME .. ".scheduler")
errorManager 		= require(cc.PACKAGE_NAME .. ".managers.errorManager")
callbackManager 	= require(cc.PACKAGE_NAME .. ".managers.callbackManager")
backButtonManager	= require(cc.PACKAGE_NAME .. ".managers.backButtonManager")
dialogManager		= require(cc.PACKAGE_NAME .. ".managers.dialogManager")

if not NO_EXTENSIONS then
    require(cc.PACKAGE_NAME .. ".cc.init")
end

local sharedTextureCache = cc.Director:getInstance():getTextureCache()
local sharedDirector = cc.Director:getInstance()
local function showMemoryUsage()
    echoInfo(string.format("LUA VM MEMORY USED: %0.2f KB", collectgarbage("count")))
end

if DEBUG_FPS then
    sharedDirector:setDisplayStats(true)
end

if DEBUG_MEM then
    sharedDirector:getScheduler():scheduleScriptFunc(showMemoryUsage, 10.0, false)
end

echoInfo("done")
