-- THE MIGHTY DIFFPATCHATRON, CHILD OF LUADIFFPATCHER
-- DIFFPATCHATRON HAS SURPASSED ITS ANCESTORS AND COMPLETED THEIR UNFINISHED TASK
local stringDiffModule = require(cc.PACKAGE_NAME..'.libraries.diff_match_patch')

local DIFF_INSERT = "_DIFF_INSERT"
local DIFF_DELETE = "_DIFF_DELETE"
local DIFF_MODIFY = "_DIFF_MODIFY"
local DIFF_MOVE = "_DIFF_MOVE"
local DIFF_COMBINED = "_DIFF_COMBINED"
local DIFF_STRING = "_DIFF_STRING"

local stringDiffThreshold = 64 -- the minimum length of string that will be diff'd

local diffPatchatron = {}

local idCounter = 0

-- Memoization for areTheSame
local similarityPile = {}

local function getSimilarityPile(key1, key2)
    local ref1 = similarityPile[key1]
    local ref2 = similarityPile[key2]
    if ref1 then
        local val1 = ref1[key2]
        if val1 ~= nil then
            return val1
        end
    elseif ref2 then
        local val2 = ref2[key1]
        if val2 ~= nil then
            return val2
        end
    end
end

-- Memoization for tableLCS
local lcsPile = {}

local function getLCSPile(sequenceID, leftLength, rightLength)
    local sequencePile = lcsPile[sequenceID]
    if sequencePile then
        local llPile = sequencePile[leftLength]
        if llPile then
            return llPile[rightLength]
        end
    end
end

local function addToLCSPile(sequenceID, i, j, thing)
    if lcsPile[sequenceID] == nil then
        lcsPile[sequenceID] = {}
    end
    if lcsPile[sequenceID][i] == nil then
        lcsPile[sequenceID][i] = {}
    end
    lcsPile[sequenceID][i][j] = thing
end

-- Table and comparion functions start here

local function hammingString(str1,str2)
    -- counts nonexistant characters (where one string is longer) as a difference
    local distance = 0
    local strlenmin
    local strlenmax
    if #str1 > #str2 then
        strlenmin = #str2
        strlenmax = #str1
    else
        strlenmin = #str1
        strlenmax = #str2
    end
    local offset = strlenmax - strlenmin
    distance = distance + offset
    for i = 1, strlenmin do
        if str1:sub(i,i) ~= str2:sub(i,i) then
            distance = distance + 1
        end
    end
    distance = distance / strlenmax
    return distance
end

local function nDist(x, y)
    local min, max
    if x == 0 or y == 0 then return 1 end
    if x > y then
        min = y
        max = x
    else
        min = x
        max = y
    end
    local diff = max - min
    local ndiff = diff/max
    return ndiff
end


-- local function deepcopy(orig)
--     local orig_type = type(orig)
--     local copy
--     if orig_type == 'table' then
--         copy = {}
--         for orig_key, orig_value in next, orig, nil do
--             copy[deepcopy(orig_key)] = deepcopy(orig_value)
--         end
--         setmetatable(copy, deepcopy(getmetatable(orig)))
--     else -- number, string, boolean, etc
--         copy = orig
--     end
--     return copy
-- end

local function mergeTables(a, b)
    local result = table.deepcopy(a)
    for k,v in pairs(b) do
        result[k] = v
    end
    return result
end

local function tinsertvalues(t, ...)
    local pos, values
    if select('#', ...) == 1 then
        pos,values = #t+1, ...
    else
        pos,values = ...
    end
    if values == nil or type(values) ~= 'table' then return end
    if #values > 0 then
        for i=#t,pos,-1 do
            t[i+#values] = t[i]
        end
        local offset = 1 - pos
        for i=pos,pos+#values-1 do
            t[i] = values[i + offset]
        end
    end
end

local function findMatchingIndexPair(i, t)
    for k,v in pairs(t) do
        if i[1] == v[1] and i[2] == v[2] then
            return k
        end
    end
    return nil
end

local function getIndices(t)
    local indices = {}
    for k,v in pairs(t) do indices[k] = k end
    return indices
end

-- local function isEmpty(table)
--     if next(table) ~= nil then
--         return false
--     end
--     return true
-- end

function diffPatchatron.areTheSame(a, b, memo) -- this is close enough to in-place
    --If it is a different type, then there is no way it can be the same
    if type(a) ~= type(b) then return false end
    if a == b then return true end

    -- Only tables, userdata and functions are compared by reference
    if type(a) == 'table' then
        memo = memo or false
        local memoResult
        if memo then
            memoResult = getSimilarityPile(a, b)
            if memoResult ~= nil then
                return memoResult
            end
        end
        local indices = {}
        local same = true
        for i,v in pairs(a) do
            same = same and diffPatchatron.areTheSame(a[i], b[i], memo)
            if not same then break end
            indices[i] = i
        end
        if same then
            for i,v in pairs(b) do
                if indices[i] then
                    same = same and diffPatchatron.areTheSame(a[i], b[i], memo)
                    if not same then break end
                else
                    same = false
                    break
                end
            end
        end
        if memo and memoResult == nil then
            if similarityPile[a] == nil then
                similarityPile[a] = {}
            end
            similarityPile[a][b] = same
        end
        return same
    end
    return false
end

local function haveMatchingStructure(a, b)
    if type(a) == type(b) then
        if type(a) == 'table' then
            local keys = mergeTables(getIndices(a), getIndices(b))
            local same = true
            for k,v in pairs(keys) do
                same = same and haveMatchingStructure(a[v], b[v])
            end
            return same
        else
            return true -- same structure, but not tables and maybe not equal
        end
    else
        return false -- different types, different structure
    end
end

function diffPatchatron.similarityBetween(a, b, remainingDepth)
    if a == nil then return 0 end
    if b == nil then return 0 end
    remainingDepth = remainingDepth or 1
    if type(a) ~= type(b) then return 0 end
    local similarity = 0
    local count = 0
    if type(a) == 'table' then
        if remainingDepth == 0 then
            similarity = 1
            count = count + 1
        else
            for k,v in pairs(b) do
                similarity = similarity + diffPatchatron.similarityBetween(diffPatchatron.findClosestMatch(a, v, k), v, remainingDepth - 1)
                count = count + 1
            end
        end
    else
        if type(a) == 'number' then
            --similarity = 1 - nDist(a, b) -- too unpredictable
            if a == b then return 1 else return 0.1 end -- give a little bit of similarity, they are the same type!
        elseif type(a) == 'string' then
            similarity = 1 - hammingString(a, b)
        else
            -- dunno what to do with other types
            similarity = 0
        end
        count = count + 1
    end
    similarity = similarity / count -- normalized so that more elements doesn't make it more similar
    return similarity
end

function diffPatchatron.findClosestMatch(t, b, indexB, threshold)
    threshold = threshold or 0.5
    local maxSimilarity = -1
    local maxSimilarityRef = nil
    local maxSimilarityIndex = nil
    if indexB == nil then return nil, nil end
    if type(indexB) == 'string' then
        maxSimilarityRef = t[indexB]
        maxSimilarityIndex = indexB
    else
        for k,v in pairs(t) do
            if type(indexB) == type(k) then
                local similarityCandidate = diffPatchatron.similarityBetween(b, v, 1000) -- Your table shouldn't be deeper than 1000 really
                if similarityCandidate > maxSimilarity and similarityCandidate > 0.5 then -- this threshold could probably be a bit shifty too
                    maxSimilarity = similarityCandidate
                    maxSimilarityRef = v
                    maxSimilarityIndex = k
                end
            end
        end
    end
    return maxSimilarityRef, maxSimilarityIndex
end

local function getValuesWithMatchingIndices(table, indices)
    local result = {}
    for k,v in pairs(indices) do
        result[k] = table[k]
    end
    return result
end

local function buildMatchTable(a, b)
    local result = {}
    local remainingB = getIndices(b)
    for k,v in pairs(a) do
        local toCheck = getValuesWithMatchingIndices(b, remainingB)
        local indexB = nil
        if b[k] ~= nil then
            indexB = k
        end
        local correspondingVInB, valueToDelete = diffPatchatron.findClosestMatch(toCheck, v, indexB)
        if valueToDelete ~= nil then
            remainingB[valueToDelete] = nil
        end
        table.insert(result, {k, valueToDelete})
    end
    for k,v in pairs(remainingB) do
        table.insert(result, {nil, v})
    end
    return result
end

local function getLastEntry(t)
    if type(t) == 'table' then
        return t[#t], #t
    end
    return nil
end

local function tableLCS(a, b, lenA, lenB, sequenceID)
    local aType = type(a)
    if aType ~= type(b) then return {}, {} end
    if aType ~= 'table' then return {}, {} end
    if #a == 0 or #b == 0 or lenA == 0 or lenB == 0 then return {}, {} end
    local result = {}
    local resultIndices = {}
    local suffixA = a[lenA]
    local suffixB = b[lenB]
    if diffPatchatron.areTheSame(suffixA, suffixB, true) then
        result = {suffixB}
        resultIndices = {{lenA, lenB}}
        local lcs, prefixIndices
        local tempLCS = getLCSPile(sequenceID, lenA - 1, lenB - 1) -- Time to memoize
        if tempLCS then
            lcs, prefixIndices = tempLCS[1], tempLCS[2]
        else
            lcs, prefixIndices = tableLCS(a, b, lenA - 1, lenB - 1, sequenceID)
            addToLCSPile(sequenceID, lenA - 1, lenB - 1, {lcs, prefixIndices})
        end
        tinsertvalues(result, 1, lcs)
        tinsertvalues(resultIndices, 1, prefixIndices)
    else
        local LCSA, indicesA, LCSB, indicesB
        local tempLCSA = getLCSPile(sequenceID, lenA - 1, lenB)
        if tempLCSA then
            LCSA, indicesA = tempLCSA[1], tempLCSA[2]
        else
            LCSA, indicesA = tableLCS(a, b, lenA - 1, lenB, sequenceID)
            addToLCSPile(sequenceID, lenA - 1, lenB, {LCSA, indicesA})
        end
        local tempLCSB = getLCSPile(sequenceID, lenA, lenB - 1)
        if tempLCSB then
            LCSB, indicesB = tempLCSB[1], tempLCSB[2]
        else
            LCSB, indicesB = tableLCS(a, b, lenA, lenB -1, sequenceID)
            addToLCSPile(sequenceID, lenA, lenB - 1, {LCSB, indicesB})
        end
        if #LCSA > #LCSB then
            result = LCSA
            resultIndices = indicesA
        else
            result = LCSB
            resultIndices = indicesB
        end
    end
    return result, resultIndices
end

-- Diff functions start here

local function formatInsertOperation(newValue)
    return {newValue}, DIFF_INSERT
end

local function formatDeleteOperation(previousValue)
    return {previousValue, 0, 0}, DIFF_DELETE -- 0 is the delete code
end

local function formatModifyOperation(previousValue, newValue)
    return {previousValue, newValue}, DIFF_MODIFY
end

local function formatMoveOperation(value, oldIndex, newIndex)
    --oldIndex is unused
    return {"", newIndex - 1, 3}, DIFF_MOVE -- 3 is the move code
end

local function formatStringOperation(stringDiff)
    return {stringDiff, 0, 2}, DIFF_STRING -- 2 is the string code
end

local function getStringDiff(left, right)
    local stringDiff = stringDiffModule.patch_make(left, right)
    return stringDiffModule.patch_toText(stringDiff)
end

local function computeDiff(left, right, leftParent, rightParent, partOfLCS)
    if left == nil and right == nil then return nil end
    local result = {}
    local resultCode
    if left == nil then
        result, resultCode = formatInsertOperation(right) -- DIFF_INSERT
    elseif right == nil then
        result, resultCode = formatDeleteOperation(left) -- DIFF_DELETE
    elseif type(left) == type(right) then
        if leftParent ~= rightParent then
            if not partOfLCS then
                if diffPatchatron.areTheSame(left, right) then
                    return formatMoveOperation(right, leftParent, rightParent), DIFF_MOVE -- DIFF_MOVE
                else
                    local delOp, delCode = formatDeleteOperation(left)
                    local insOp, insCode = formatInsertOperation(right)
                    return {delOp, insOp}, DIFF_COMBINED
                end
            end
        end
        if type(left) == 'table' then
            local lcs, lcsIndices = tableLCS(left, right, #left, #right, idCounter) -- LCS
            idCounter = idCounter + 1
            local matchTable = buildMatchTable(left, right)
            for k,v in pairs(matchTable) do
                local leftIndex, rightIndex = v[1], v[2]
                local tempPartOfLCS = true
                if lcsIndices ~= nil then
                    if findMatchingIndexPair(v, lcsIndices) == nil then
                        tempPartOfLCS = false
                    end
                end

                local tempDiffs, tempResultCode = computeDiff(left[leftIndex], right[rightIndex], leftIndex, rightIndex, tempPartOfLCS)

                if tempResultCode == DIFF_COMBINED then -- Delete and insert to simplify moves
                    local parentOld = '_'..tostring(leftIndex - 1)
                    local parentNew = tostring(rightIndex - 1)
                    result[parentOld] = tempDiffs[1]
                    result[parentNew] = tempDiffs[2]
                 else -- Just one operation
                    local parentIndex
                    if rightIndex == nil then -- this is so we can give things the correct index
                        parentIndex = leftIndex
                    else
                        parentIndex = rightIndex
                    end
                    if type(parentIndex) == 'number' then
                        parentIndex = parentIndex - 1 -- JSON uses 0-based arrays, Lua uses 1--based arrays
                        if not table.isEmpty(result) and result['_t'] == nil then
                            result['_t'] = 'a'
                        end
                        if tempResultCode == DIFF_DELETE then
                            parentIndex = '_'..tostring(parentIndex)
                        elseif tempResultCode == DIFF_MOVE then
                            parentIndex = '_'..tostring(leftIndex - 1) -- Old index for moves?
                        end
                    end

                    if type(tempDiffs) == 'table' and not table.isEmpty(tempDiffs) then
                        result[parentIndex] = tempDiffs -- no point adding empty diffs
                    end
                end
            end
        elseif type(left) == 'string' then
            if #left < stringDiffThreshold or #right < stringDiffThreshold then -- Don't diff short strings
                result, resultCode = computeDiff(left[leftIndex], right[rightIndex], leftIndex, rightIndex, tempPartOfLCS) -- DIFF_MODIFY on short strings
            else
                local stringDiff = getStringDiff(left, right)
                result, resultCode = formatStringOperation(stringDiff) -- DIFF_STRING
            end
        else
            if left == right then
                return nil
            else
                result, resultCode = formatModifyOperation(left, right) -- DIFF_MODIFY
            end
        end
    else
        result, resultCode = formatModifyOperation(left, right) -- DIFF_MODIFY
    end
    return result, resultCode
end

-- The actual function you call to diff stuff
function diffPatchatron.diff(left, right)
    diffs = computeDiff(left, right)
    return diffs
end

return diffPatchatron
