--tinyxml by Aleksis Kavalieris
--based heavily on SLAXML and AKLOM - https://github.com/Phrogz/SLAXML

local tinyxml = {}

function tinyxml.unescape(inString)
	-- cache function reference for maximum speed
	local gsub = string.gsub
	inString = gsub(inString, '&lt;', '<')
	inString = gsub(inString, '&gt;', '>')
	inString = gsub(inString, '&quot;', '"')
	inString = gsub(inString, '&apos;', "'")
	return gsub(inString, '&amp;', '&')
end

function tinyxml.parse(xmlString)
	local sub, gsub, find, push, pop, unescape = string.sub, string.gsub, string.find, table.insert, table.remove, tinyxml.unescape
	
	-- clear whitespace
	-- xmlString = gsub(xmlString, '>%s+', '>')
	-- xmlString = gsub(xmlString, '%s+<', '<')

	result = {}
	local currentElement = result
	local stack = {n=0}
	local currentPosition = 1
	local lastEndPosition = 1
	local theStart, theEnd, theClose, theName, theAttr, theEmpty

	while true do
		-- cache function reference for maximum speed
		theStart, theEnd, theClose, theName, theAttr, theEmpty = find(xmlString, '<(%/?)(%a%w*)(.-)(%/?)>', currentPosition)
		if not theStart then
			if currentPosition == #xmlString then
				break
			else
				push(result, sub(xmlString, lastEndPosition))
				break
			end
		end

		-- print(sub(xmlString, theStart, theEnd))

		currentPosition = theEnd + 1
		isEmpty = (theEmpty and theEmpty ~= '')

		-- add text outside tags
		local outsideBlock = unescape(sub(xmlString, lastEndPosition, theStart - 1))
		if outsideBlock and outsideBlock ~= '' then
			push(currentElement, outsideBlock)
		end

		-- is it a closing tag?
		if theClose and theClose ~= '' then
			local success = pop(stack)
			stack.n = stack.n - 1
			if not success or success.name ~= theName then
				error("tinyxml: your tags are not nested properly")
			end
			currentElement = stack[stack.n]
			if not currentElement then
				currentElement = result
			end
		else
			local theElement = {
				name = theName,
				attr = {}
			}

			-- Parse the attribute string
			gsub(
				theAttr,
				'([%a_:][%w._:-]*)%s*=%s*([\'"])(.-)%2',
				function(inAttName, _, inAttValue)
					theElement.attr[inAttName] = unescape(inAttValue)
				end
			)

			push(currentElement, theElement)

			if not isEmpty then
				push(stack, theElement)
				stack.n = stack.n + 1
				currentElement = theElement
			end
		end

		lastEndPosition = currentPosition
	end

	if stack.n > 0 then
		error("tinyxml: element "..stack[stack.n].name.." still open")
	end
	-- iprint(result)
	return result

end

return tinyxml