local M = {}

VOLUME = {}

VOLUME_DEFAULT_VOLS = {}

M._storedMasterVolume = nil

M._audioConnector = nil

M._ambienceEvent = nil
M._musicEvent = nil
M._repeatableSounds = {}
M._paramEvent = nil
M._paramEventValue = 1

M._volumes = {}

function M.getEvent(eventName)
	return M._audioConnector:getEvent(eventName)
end

------------------------------

function M.createVolumeController(busName, defaultVolume)
	echoInfo("createVolumeController %q => %s", busName, defaultVolume)
	local v = {
		_busName = busName,
		_currentVolume = defaultVolume
	}

	function v.volumeChange(perc)

		v._currentVolume = perc
		if v._currentVolume > 1 then
			v._currentVolume = 1
		elseif v._currentVolume < 0 then
			v._currentVolume = 0
		end
		M._audioConnector:setVolume(v._busName, v._currentVolume)
	end

	function v.getVolume()
		return v._currentVolume
	end

	v.volumeChange(defaultVolume)

	function v.setMute(mute)
		M._audioConnector:setMute(v._busName, mute)
	end

	return v
end
------------------------------

function M.setMute(volumeName, mute)
	if M._volumes[volumeName] ~= nil then
		M._volumes[volumeName].setMute(mute)
		M._volumes[volumeName].mute = mute
	end
end
-------------------------------

function M.getMute(volumeName)
	if M._volumes[volumeName] ~= nil then
		return M._volumes[volumeName].mute
	end
	return nil
end
-------------------------------

function M.setVolume(volumeName, perc)
	echoInfo("Setting volume %q=>%s", volumeName, perc)
	if M._volumes[volumeName] ~= nil then
		M._volumes[volumeName].volumeChange(perc)
	end
end
------------------------------

function M.getVolume(volumeName)
	if M._volumes[volumeName] ~= nil then
		return M._volumes[volumeName].getVolume()
	end
	return 0
end
------------------------------

------------------------------
-------- INIT/CLEANUP --------
------------------------------

function M.init(localAudioData)

	print("AudioManager init")
	dump(localAudioData, "localAudioData")

	M._audioConnector = runaway.AudioManager:getInstance()

	-- Get Banks from Config
	for i = 1, #AUDIO.BANKS do
		print("Preparing to load bank", AUDIO.BANKS[i])
		if M._audioConnector:loadStudioBank(AUDIO.BANKS[i], false, i) then print(AUDIO.BANKS[i], "- Load Successful") end
	end
	-- Get Buses/Volume controlers from Config
	for _, volumeDetail in pairs(AUDIO.VOLUMES) do
		echoInfo("Preparing to load bus %q", volumeDetail.BUS)
		M._volumes[volumeDetail.NAME] = M.createVolumeController(volumeDetail.BUS, volumeDetail.DEFAULT_VOLUME)
	end

	if localAudioData ~= nil and localAudioData.volumes ~= nil then
		for volumeName, volume in pairs(localAudioData.volumes) do
			M.setVolume(volumeName, volume)
		end
	end

	--M.DumpEventsTest()

	if runaway.Native:isBackgroundAudioPlaying() then
		M.setMuteOnBackgroundSounds(true)
	end

    local eventDispatcher     = cc.Director:getInstance():getEventDispatcher()
    local onEnterBackgroundListener = cc.EventListenerCustom:create(APP_ENTER_BACKGROUND_EVENT, M.onEnterBackground)
    local onEnterForegroundListener = cc.EventListenerCustom:create(APP_ENTER_FOREGROUND_EVENT, M.onEnterForeground) 
    local onBackgroundAudioListener = cc.EventListenerCustom:create(APP_BACKGROUND_AUDIO_CHANGED, M.onBackgroundAudio) 
    eventDispatcher:addEventListenerWithFixedPriority(onEnterBackgroundListener, 3)
    eventDispatcher:addEventListenerWithFixedPriority(onEnterForegroundListener, 2)
    eventDispatcher:addEventListenerWithFixedPriority(onBackgroundAudioListener, 3)
end
------------------------------

function M.cleanup()
	M.stopMusicEvent()
	M.stopAmbienceEvent()

	if table.nums(M._repeatableSounds) > 0 then
		for i, sound in pairs(M._repeatableSounds) do
			sound:releaseEvent()
			M._repeatableSounds[i] = nil
		end
	end

	-- for volumeName, volumeDetail in pairs(AUDIO.VOLUMES) do
	-- 	M.setVolume(volumeDetail.NAME, volumeDetail.DEFAULT_VOLUME)
	-- end
end
------------------------------


------------------------------
------- AMBIENCE EVENTS ------
------------------------------

function M.playAmbienceEvent(eventName)
	if M._ambienceEvent == nil then
		M._ambienceEvent = M._audioConnector:getEvent(eventName)
		if M._ambienceEvent then M._ambienceEvent:start() end
	end
end
------------------------------

function M.stopAmbienceEvent()
	if M._ambienceEvent ~= nil then
		M._ambienceEvent:stop(false)	--	allowFadeout = false
		M._ambienceEvent:releaseEvent()
		M._ambienceEvent = nil
	end
end
------------------------------


------------------------------
-------- MUSIC EVENTS --------
------------------------------

function M.playMusicEvent(eventName)
	if M._musicEvent == nil then
		M._musicEvent = M._audioConnector:getEvent(eventName)
		if M._musicEvent ~= nil then
			M._musicEvent:start()
		end
	end
end
------------------------------

function M.stopMusicEvent()
	if M._musicEvent ~= nil then
		M._musicEvent:stop(false)	--	allowFadeout = false
		M._musicEvent:releaseEvent()
		M._musicEvent = nil
	end
end
------------------------------


------------------------------
--------- SFX EVENTS ---------
------------------------------

function M.playSingleShot(eventName)
	echoInfo("audioManager: playing single shot sound: %s", eventName or "nil")
	M._audioConnector:playSingleShotEvent(eventName)
end
------------------------------

function M.playAndRelease(eventName)
	local event = M._audioConnector:getEvent(eventName)
	event:start()
	event:releaseEvent()
end
------------------------------

function M.playRepeatable(eventName)
	-- M.stopRepeatable(eventName)
	if M._repeatableSounds[eventName] == nil then
		M._repeatableSounds[eventName] = M._audioConnector:getEvent(eventName)
	end
	if M._repeatableSounds[eventName] ~= nil then
		M._repeatableSounds[eventName]:start()
	end
end
------------------------------

function M.stopRepeatable(eventName)
	if M._repeatableSounds[eventName] ~= nil then
		M._repeatableSounds[eventName]:stop(false)
	end
end
------------------------------

function M.DumpEventsTest()
    M._audioConnector:debugLoadedEvents();
end
------------------------------

function M.MemoryUsageTest()
    M._audioConnector:debugMemoryUsage();
end


------------------------------
-------- PARAM EVENTS --------
------------------------------

function M.setParamForEvent(event, paramName, value)
	event:setParameter(paramName, value)
end


function M.setStoredMasterVolume(vol)
	M._storedMasterVolume = vol
end
------------------------------

function M.getStoredMasterVolume()
	return M._storedMasterVolume
end
------------------------------

function M.setMuteOnBackgroundSounds(mute)
	for _, volumeDetail in pairs(AUDIO.VOLUMES) do
		if (volumeDetail.IS_BACKGROUND) then
			M._volumes[volumeDetail.NAME].setMute(mute)
		end
	end
end

------------------------------
-------- PERSISTENCE ---------
------------------------------

function M.getLocalSaveData()
	local localData = {}

	localData.volumes = {}
	for volumeName, volumeDetail in pairs(AUDIO.VOLUMES) do
		localData.volumes[volumeDetail.NAME] = M.getVolume(volumeDetail.NAME)
	end

	return localData
end

------------------------------

function M.onEnterBackground()
    M._audioConnector:suspend()
end

function M.onEnterForeground()
    M._audioConnector:resume()
end

function M.onBackgroundAudio(event)
	local resultString = event:getUserDataString()
	local result = dlcManager.decryptJson(resultString)

	M.setMuteOnBackgroundSounds(result.isBackgroundPlaying)
end

return M