--backButtonManager.lua

local backButtonManager = {}

local functionHub = {}
local priorityQueue = nil

local backButtonListenerLayer

local disable = false

local function getSortedFunctionQueue()
	local tempOrder = {}
	for key, item in pairs(functionHub) do
		tempOrder[item.priority] = item.functionRef
	end
	local result = {}
    for k, v in pairs(tempOrder) do
        table.insert(result, 1, v)
    end
	return result
end

local function getPriorityQueue()
	if not priorityQueue then
		priorityQueue = getSortedFunctionQueue()
	end
	return priorityQueue
end

local function getFunctionWithPriorityRank(rank)
	if not priorityQueue then
		priorityQueue = getSortedFunctionQueue()
	end
	return priorityQueue[rank]
end

local function backKeyPress()
	if disable then
		echoInfo("back key disabled")
		return
	end
	local sortedFunctionQueue = getPriorityQueue()
	for k,v in ipairs(sortedFunctionQueue) do
		if type(v) == "function" then
			local result = v()
			if result then
				break
			end
		end
	end
end

local function onKeyPress(keyPressed)
	if keyPressed == runaway.KeyCode.KEY_BACK then
		backKeyPress()
    end
end

local function init()
	backButtonListenerLayer = cc.Layer:create()
	local listener = cc.EventListenerKeyboard:create()
    listener:registerScriptHandler(onKeyPress, 39) -- 39 is the constant for keypad event dispatcher

    local eventDispatcher = backButtonListenerLayer:getEventDispatcher()
    eventDispatcher:addEventListenerWithFixedPriority(listener, 1)
end

function backButtonManager.registerFunctionWithPriority(functionRef, priority, tag)
	functionHub[tag] = {functionRef = functionRef, priority = priority, tag = tag}
	priorityQueue = nil
end

function backButtonManager.setDisabled(b)
	disable = b
end

function backButtonManager.addSimulatedBackButton(layer, spriteframe)
	local function buttonFunction()
		backKeyPress()
	end
	local positionX, positionY = 10, 10
	local button = display.newSprite(spriteframe)
	button.disabled = false
	local boundsSize = cc.size(40,40)
	local touchLayer = ccui.Layout:create()
	touchLayer:setContentSize(boundsSize)
	touchLayer:setTouchEnabled(true)
	touchLayer:setSwallowTouches(true)
	touchLayer:addTouchEventListener(function(sender, eventType)
		if button.disabled then
			return
		else
			backKeyPress()
			button.disabled = true
			scheduler.performWithDelayGlobal(function() button.disabled = false end, 1)
		end
	end)
	button:addChild(touchLayer)
	button:setPosition(positionX, positionY)
	layer:addChild(button, 10000)
end

init()

return backButtonManager