local errorManager = {}

local eventDispatcher = cc.Director:getInstance():getEventDispatcher()
local conditionWatcher    = require(cc.PACKAGE_NAME..".libraries.conditionWatcher")
scheduler.attachLocalScheduler(errorManager)

errorManager.errorCount = 0
local dismissKey = "DISMISS"

function errorManager.cleanup()
	errorManager.unscheduleFunction(dismissKey)
end

function errorManager._displayErrorDialog(errorObject, errorData)
	local function autoDismiss()
		print("autoDismiss")
		local callback = dialogManager.removeCurrentDialog()
        if callback then callback(result) end
	end

	local function dismiss(button)
		print("dimiss")
		errorManager.isHandlingError = false
		local event = cc.EventCustom:new(ERROR_HANDLED)
		event._usedata = {needRestart = false}
		cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)
	end
	local function dismissAndRestartGame(button)
		print("dismissAndRestartGame")
		errorManager.unscheduleFunction(dismissKey)

		errorManager.isHandlingError = false
		local event = cc.EventCustom:new(ERROR_HANDLED)
		event._usedata = {needRestart = true}
		cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)
		--	CONDITION_DATA_UNSAFE will be cleared in reset functionality
		gameManager.recoverFromError()
	end

	if errorObject.needRestart then
	    conditionWatcher:setCondition(CONDITION_DATA_UNSAFE,        true)
		dialogManager.showDialog( errorObject.name, errorObject.message, true, {ui.SYSTEM_DIALOG_BUTTON_OK}, dismissAndRestartGame)
		errorManager.scheduleOnce(autoDismiss, ERROR_AUTO_DISMISS_TIME, dismissKey)
	else
		dialogManager.showDialog( errorObject.name, errorObject.message, true, {ui.SYSTEM_DIALOG_BUTTON_OK}, dismiss)
	end
end

function errorManager.handleError(event)
	local isAlreadyHandlingError = errorManager.isHandlingError
	errorManager.isHandlingError = true

	local errorName = 'Lua Error'
	local errorData = event._usedata
	local errorMessage = "No error information provided"
	if errorData and #errorData then
		errorMessage = errorData[1]
	end
	local errorCode = tonumber(errorMessage)
	local errorObject = ERROR_CODES[errorCode]

	local isRelease = runaway.GameConfig:isProduction()
	local buildNumber = runaway.Native:getVersion()
	if (isRelease and buildNumber <= gameManager.maxAnalyticsVersion) then
		errorName = ERROR_CODES[4].name
		errorMessage = ERROR_CODES[4].message
	end

	-- if we've had more than 5 errors, then flag the DLC to re-download the manifest clean
	errorManager.errorCount = errorManager.errorCount + 1
	-- This was previously set to only redownload after 5 errors
	--	however most people never get this far.

	local earlyCrash = false
	local currentTime = os.clock()/10000
    if GAMESTART_TIMESTAMP then
        local elapsedTime = currentTime - (GAMESTART_TIMESTAMP/10000)
    	-- print(currentTime, GAMESTART_TIMESTAMP/10000, elapsedTime)
        if (elapsedTime < 60) then
        	earlyCrash = true
        end
    end

	if errorManager.errorCount > 1 then
		errorManager.errorCount = 0
		if dlcManager then	--	 it's possible that the DLC manager hasn't been initialised yet, especially if an error 
			dlcManager.triggerResetManifest()
		end
	elseif earlyCrash then
		if dlcManager then
			echoInfo("Early crash, forcing dlc re-unzip")
			dlcManager.resetManifest = true
		end
	end

	if errorObject == nil then
		if DEBUG > 0 then
			errorObject = {name = errorName, message = (errorMessage or "A nondescript error occurred."), needRestart = true, showDialog = true}
		else
			errorObject = ERROR_CODES[4]
		end
	end

	echoInfo(errorObject.message, errorMessage or "no errorMessage")

	if not isAlreadyHandlingError then
		sendLogToServer(errorData)
		if errorObject.showDialog then
			errorManager._displayErrorDialog(errorObject, errorData)
		else
			errorManager.isHandlingError = false
		end
	end
end

display.loadSpriteFrames("hud/buttons.plist", "hud/buttons.pvr.ccz")
display.loadSpriteFrames("hud/menus.plist", "hud/menus.pvr.ccz")

return errorManager
