CONDITION_GAME_IS_FOREGROUND     =  "CONDITION_GAME_IS_FOREGROUND"
CONDITION_GAME_IS_RUNNING        =  "CONDITION_GAME_IS_RUNNING"
CONDITION_USER_DATA_VERIFIED     =  "CONDITION_USER_DATA_VERIFIED"
CONDITION_DATA_UNSAFE            =  "CONDITION_DATA_UNSAFE"    --  for when errors occur and we're cleaning up
CONDITION_SERVER_COMMUNICATING   =  "CONDITION_SERVER_COMMUNICATING"   

    
OBSERVER_PERSISTENCE_RUNNING     =  "OBSERVER_PERSISTENCE_RUNNING"
OBSERVER_PERSISTENCE_FORCE       =  "OBSERVER_PERSISTENCE_FORCE"
OBSERVER_SERVER_COMMUNICATING    =  "OBSERVER_SERVER_COMMUNICATING"


function setupGameManagerConditions()
    eventDispatcher:addEventListenerWithFixedPriority(cc.EventListenerCustom:create(APP_ENTER_BACKGROUND_EVENT, onGameEnterBackground), 2)  --  2= wait for game scheduled tasks to be stopped
    eventDispatcher:addEventListenerWithFixedPriority(cc.EventListenerCustom:create(APP_ENTER_FOREGROUND_EVENT, onGameEnterForeground), 1)
    setInitialConditionStates()

    --  set up a trigger so that persistence is only running it's scheduled update
    --      when the game is running, and the local data is verified against the server 
    local observer = conditionWatcher:createConditionObserver(OBSERVER_PERSISTENCE_RUNNING);
    observer:addCondition(CONDITION_GAME_IS_RUNNING, true);
    observer:addCondition(CONDITION_USER_DATA_VERIFIED, true);
    observer:addCondition(CONDITION_DATA_UNSAFE, false);
    observer:onAllConditionsAchieved(
        function() 
            echoInfo("Observer started persistence when game entered running state, and data was verified") 
            persistence.resume()
            mailboxManager.resume()
        end)
    observer:onAllConditionsAchievedBroken(
        function()
            echoInfo("Observer stopping regular persistence when game left running state") 
            persistence.stop() 
            mailboxManager.stop()
        end)

    --  set up a trigger so that a persistence update is forced when the game goes into the background
    --      only when the game is running, and the local data is verified against the server 
    local observer = conditionWatcher:createConditionObserver(OBSERVER_PERSISTENCE_FORCE);
    observer:addCondition(CONDITION_GAME_IS_RUNNING, true);
    observer:addCondition(CONDITION_USER_DATA_VERIFIED, true);
    observer:addCondition(CONDITION_GAME_IS_FOREGROUND, false);
    observer:addCondition(CONDITION_DATA_UNSAFE, false);
    observer:onAllConditionsAchieved(
        function()
            echoInfo("Observer Forcing persistence when going background") 
            conditionWatcher:setCondition(CONDITION_USER_DATA_VERIFIED,     false)  --  to stop saving upon resume until re-verified
            persistence.force(false, nil, true)   --  extra "true" flag forces save to disk before the app sleeps
        end)


    local observer = conditionWatcher:createConditionObserver(OBSERVER_SERVER_COMMUNICATING);
    -- observer:addCondition(CONDITION_GAME_IS_RUNNING, true);
    observer:addCondition(CONDITION_SERVER_COMMUNICATING, true);
    observer:onAnyConditionAchieved(
        function()  onServerCommunicating(true) end)
    observer:onAllConditionsFailed(
        function()  onServerCommunicating(false) end)
end

function setInitialConditionStates()
    echoInfo("Setting initial Condition states")
    conditionWatcher:setCondition(CONDITION_GAME_IS_FOREGROUND,     true)
    conditionWatcher:setCondition(CONDITION_GAME_IS_RUNNING,        false)
    conditionWatcher:setCondition(CONDITION_USER_DATA_VERIFIED,     false)
    conditionWatcher:setCondition(CONDITION_DATA_UNSAFE,            false)
    conditionWatcher:setCondition(CONDITION_SERVER_COMMUNICATING,   false)
end

function onGameEnterBackground()
    conditionWatcher:setCondition(CONDITION_GAME_IS_FOREGROUND,     false)
    gameManager.onEnterBackground()
end

function onGameEnterForeground()
    conditionWatcher:setCondition(CONDITION_GAME_IS_FOREGROUND,     true)
    --  if we bring the app foreground, then re-check the user data.
    if (gameManager.isSceneInGame()) then
        gameManager.isOnEnterForeground = true;
        echoInfo("Switching to foreground while isSceneInGame, so checkSession")
        gameManager.checkSessionLocal();
    end
end



--  show the "Networking" spinning wheel
function onServerCommunicating(active)
    --assumes the existance of SYSTEMLAYER 
    if active then
        if SHOW_SERVER_COMMUNICATING_SPINNER and (gameManager.waitLayer == nil) and (SYSTEMLAYER ~= nil) then
            --  create the spinner layer
            local waitLayer = display.newSprite(PLATFORM_SPRITEFRAME_SPINNER)
            waitLayer:runAction(cc.RepeatForever:create(cc.RotateBy:create(2, 360)))
            waitLayer:setPosition(display.width * 0.5, display.height * 0.22)
            waitLayer:setScale(0.35)
            waitLayer:setColor(cc.c3b(196, 255, 196))      --   light green

            SYSTEMLAYER:addChild(waitLayer)
            gameManager.waitLayer = waitLayer
        end
    else
        if (gameManager.waitLayer ~= nil) and (SYSTEMLAYER ~= nil) then
            SYSTEMLAYER:removeChild(gameManager.waitLayer)
            gameManager.waitLayer = nil
        end
    end
end
