local M = {}

M.counterTimeStamps = {}
M.timeBetweenChecks = 10

function M.init(serverManager)  
	M.serverManager = serverManager
end

function M.getCounter(counterKey, callback)
	if not M.counterTimeStamps[counterKey] then M.counterTimeStamps[counterKey] = {timeStamp = 0, count = 0} end
	local function onResult(result, data)
		if result.code == runaway.PluginResult.kSucceed then
			M.counterTimeStamps[counterKey].count = data[counterKey] and data[counterKey].counter_value
		end
		if type(callback) == "function" then
			callback(result, data, M.counterTimeStamps[counterKey].count)
		end
	end

	local timeSinceLastCheck = (os.time() - M.counterTimeStamps[counterKey].timeStamp)
	if timeSinceLastCheck > M.timeBetweenChecks then
		local params = {}
		params["counter_key"] = counterKey
	   serverManager.performRequest("PublicCounterValue", params, nil, onResult, true)
	   M.counterTimeStamps[counterKey].timeStamp = os.time()
	else
		if type(callback) == "function" then
			result = {}
			result.code = runaway.PluginResult.kSucceed
			callback(result, data, M.counterTimeStamps[counterKey].count)
		end
	end
end
------------------------------

function M.getCounters(callback)
   serverManager.performRequest("PublicCounterValue", nil, nil, callback, true)
end
------------------------------

function M.incrementCounter(counterKey, incrementValue, callback)
	local function onResult(result, data)
		local counterValue = nil
		if result.code == runaway.PluginResult.kSucceed then
			counterValue = data[counterKey] and data[counterKey].counter_value
		end
		if type(callback) == "function" then
			callback(result, data, counterValue)
		end
	end

	local params = {}
	params["counter_key"] = counterKey
	params["increment"] = incrementValue or 1
   serverManager.performRequest("PublicCounterIncrement", params, nil, onResult, true)
end
------------------------------

return M