serverManager = {}
serverManager.headers = {}
serverManager.server = nil

function serverManager.init(headers, server, endpoints)
	serverManager.headers = headers
	serverManager.server = server

	serverManager.server:setEndPoints(endpoints)

    if ENCRYPT_SERVER then
        echoInfo("ENCRYPT_SERVER")
        serverManager.server:SetEncryptionCipher('aes-128-cfb')
        serverManager.server:SetEncryptionKey(runaway.GameConfig:getAESJSONKey())
        serverManager.server:SetEncryptionIV(runaway.GameConfig:getAESJSONIV())
    end
    scheduler.attachLocalScheduler(serverManager)

end

function serverManager.performRequest(endpointName, arguments, extraHeaders, callback, enableRetry)
    echoInfo("######################")
    echoInfo("######################")
    echoInfo("serverManager.PerformRequest(%s)", endpointName)
    echoInfo(device.client_platform)
    echoInfo("######################")
    echoInfo("######################")

    local endpoint = serverManager.server:GetEndpoint(endpointName) 
	-- print("URL("..endpointName..")=>".. (endpoint and endpoint.url or "not found"))
    -- serverManager.headers['client_platform'] = device.client_platform

    local tempHeaders = {}
    tempHeaders['X-Client-Platform'] = device.client_platform
    for k, v in pairs(serverManager.headers) do
        tempHeaders[k] = v
    end
    if endpoint.method == HTTPREQUEST_POST then
        local postHeaders = {}
        postHeaders["Accept"] = "application/json;"
        postHeaders["Content-Type"] = "application/json; charset=utf8"
        table.merge(tempHeaders, postHeaders)
    end

    if extraHeaders ~= nil then
    	--dump (extraHeaders, "extraHeaders")
        table.merge(tempHeaders, extraHeaders)
    end

	if arguments == nil then
        arguments = {}        
    end

    local attempts = 0
    local retry = nil
    local function onComplete(result, data)
        conditionWatcher:setCondition(CONDITION_SERVER_COMMUNICATING,   false)
        local resultCode = result and result.code or -1
        if enableRetry 
            and (resultCode ~= runaway.PluginResult.kSucceed)
            and (resultCode ~= HTTPREQUEST_ERROR_UNAUTHORIZED) 
            and (attempts <= 3) then

            echoInfo("serverManager.PerformRequest(%s) failed.  retrying", endpointName)
            retry()
        else
            callback(result, data)
        end
    end
    retry = function()
        attempts = attempts + 1
        conditionWatcher:setCondition(CONDITION_SERVER_COMMUNICATING,   true)
        serverManager.server:PerformRequest( endpoint.method, endpoint.url, arguments, tempHeaders, onComplete)
    end


    local noNetworkRetry = nil
    local function onNetworkNotConnected()
        if type(serverManager.cleanupLocalScheduler) == "function" then
            serverManager.cleanupLocalScheduler()
        end
        dialogManager.showDialog(
                SKELETON_MESSAGES.NetworkNotConnected.Title,
                SKELETON_MESSAGES.NetworkNotConnected.Desc,
                true,
                { ui.SYSTEM_DIALOG_BUTTON_OK },
                noNetworkRetry,
                {hsUrl = SKELETON_MESSAGES.NetworkNotConnected.hsUrl}
            )
    end
    noNetworkRetry = function()
        if runaway.Native:isInternetAvailable() then
            retry()
        else
            echoInfo("isInternetAvailable = false!")
            serverManager.scheduleOnce(function() onNetworkNotConnected() end, 0.5)
        end
    end
    noNetworkRetry()
end

function serverManager.setAccessToken(accessToken)
    serverManager.server:SetAccessToken(accessToken)
end

function serverManager.hasAccessToken()
    return (serverManager.server:GetAccessToken() ~= nil)
end

function serverManager.hasErrors(data)
    return data and type(data.errors) == "table" and (not table.isEmpty(data.errors)) 
end

function serverManager.hasSpecificError(data, errorId)    
    if data and type(data.errors) == "table" and (not table.isEmpty(data.errors)) then
        for _, errorNum in pairs(data.errors) do
            echoInfo("Found error number #%d", errorNum)
            if errorNum == errorId then   --  unable to migrate account
                return true
            end
        end
    end
    return false
end

return serverManager
