--socialManager.lua

local SocialServiceBase = require(cc.PACKAGE_NAME..".social.facebookSocialService")
local FacebookSocialService = require(cc.PACKAGE_NAME..".social.facebookSocialService")
local TwitterSocialService = require(cc.PACKAGE_NAME..".social.twitterSocialService")
local GameCenterSocialService = nil
local GooglePlaySocialService = nil
local AmazonSocialService = nil

if device.client_platform == "android" then
	GooglePlaySocialService = require(cc.PACKAGE_NAME..".social.googlePlaySocialService")
elseif device.client_platform == "amazon" then
	AmazonSocialService = require(cc.PACKAGE_NAME..".social.amazonSocialService")
else
	GameCenterSocialService = require(cc.PACKAGE_NAME..".social.gameCenterSocialService")
end


local M = {}
M.linkedFriendVersion = 2	--	bump this number to force everyone to re-search for friends (can be updated via lua hack too)

function M.init(serverManager)  
	if (persistenceUserGame.socialAccounts == nil) then
		persistenceUserGame.socialAccounts = {}
	end
	M.serverManager = serverManager

	M.social2LogoutListener = cc.EventListenerCustom:create(SOCIAL_2_NOTIFY_LOGOUT, function(event) M.logoutBasicSocial(event._usedata) end)
    eventDispatcher:addEventListenerWithFixedPriority(M.social2LogoutListener, 1)

	--	all supported services
	M.AllSocialServices = {}
	M.AllSocialServices[SOCIAL_GOOGLEPLAY]	= {
		name = 'Google Play',	
		icon = 'SocialGoogleplay',
		connectionName = CONNECTION_GOOGLEPLAY,
		socialType = SOCIAL_GOOGLEPLAY, 
		serviceClass = GooglePlaySocialService
	}
	M.AllSocialServices[SOCIAL_FACEBOOK] 	= {
		name = 'Facebook',
		icon = 'SocialFacebook',
		connectionName = CONNECTION_FACEBOOK,
		socialType = SOCIAL_FACEBOOK,
		serviceClass = FacebookSocialService
	}
	M.AllSocialServices[SOCIAL_GAMECENTER] 	= {
		name = 'Game Center',	
		icon = 'SocialGamecenter',	
		connectionName = CONNECTION_GAMECENTER,
		socialType = SOCIAL_GAMECENTER,
		serviceClass = GameCenterSocialService
	}
	M.AllSocialServices[SOCIAL_AMAZON] 	= {
		name = 'Amazon',	
		icon = 'SocialAmazon',	
		connectionName = CONNECTION_AMAZON,
		socialType = SOCIAL_AMAZON,
		serviceClass = AmazonSocialService
	}
	M.AllSocialServices[SOCIAL_TWITTER] = {
		name = "Twitter",
		icon = "SocialTwitter",
		connectionName = CONNECTION_TWITTER,
		socialType = SOCIAL_TWITTER,
		serviceClass = TwitterSocialService
	}

	-- now just get the active ones for this platform
	M.activeServiceDescriptions = {}
	M.activeServiceDescriptions[1]	= M.AllSocialServices[SOCIAL_FACEBOOK]
	if device.client_platform == "android" then
		M.activeServiceDescriptions[2]	= M.AllSocialServices[SOCIAL_GOOGLEPLAY]	
	elseif device.client_platform == "amazon" then 
		M.activeServiceDescriptions[2]	= M.AllSocialServices[SOCIAL_AMAZON]
	elseif device.client_platform == "ios" then
		M.activeServiceDescriptions[2]	= M.AllSocialServices[SOCIAL_GAMECENTER]
	end
	M.activeServiceDescriptions[3] = M.AllSocialServices[SOCIAL_TWITTER]

	-- remove disabled social services per-game
	if DISABLED_SOCIAL_NETWORKS then
		for i = #M.activeServiceDescriptions, 1, -1 do
			if DISABLED_SOCIAL_NETWORKS[M.activeServiceDescriptions[i].socialType] then
				table.remove(M.activeServiceDescriptions, i)
			end
		end
	end

	M.cleanup()

	-- initialise the active services
    if (LOCAL_MODE ~= true) then
    	for _, socialDescription in pairs(M.activeServiceDescriptions) do
    		if socialDescription.serviceClass then
	    		socialDescription.service = socialDescription.serviceClass.new()
	    		socialDescription.service:init({}, M.serverManager)
	    	end
	   	end
    end

    local function inviteSent(event)
    	if device.client_platform == "android" or device.client_platform == "amazon" then
    		echoInfo("SMS/Email invite maybe sent, maybe not")
    		return
    	end
    	local eventDataString = event:getUserDataString()
		local eventData = json.decode(eventDataString)
		if eventData.result == "sent" then
			dialogManager.showDialog(SKELETON_MESSAGES.InviteSent.Title, SKELETON_MESSAGES.InviteSent.Desc, false, {ui.SYSTEM_DIALOG_BUTTON_OK}, nil)
		elseif eventData.result == "failed" then
			dialogManager.showDialog(SKELETON_MESSAGES.InviteFailed.Title, SKELETON_MESSAGES.InviteFailed.Desc, false, {ui.SYSTEM_DIALOG_BUTTON_OK}, nil)
		else
			echoInfo("invite not sent")
		end
    end

    M.inviteListeners = {
    	smsListener = eventDispatcher:addEventListenerWithFixedPriority(cc.EventListenerCustom:create('sendSMS', inviteSent), 1),
    	emailListener = eventDispatcher:addEventListenerWithFixedPriority(cc.EventListenerCustom:create('sendHTMLEmail', inviteSent), 1),
	}

	-- M.updateSocial() -- should this really be called here?
end
------------------------------

function M.getSocialServiceDescriptionByName(serviceName)
	for _, service in pairs(M.activeServiceDescriptions) do
		if service.connectionName == serviceName then
			return service
		end
	end
	return nil
end
------------------------------

function M.getSocialServiceDescription(serviceType)
	for _, service in pairs(M.activeServiceDescriptions) do
		if service.socialType == serviceType then
			return service
		end
	end
end
------------------------------

function M.getSocialServiceName(serviceType)
	local serviceDescription = M.getSocialServiceDescription(serviceType)
	if serviceDescription then
		return serviceDescription.name
	end
end
------------------------------

function M.isAnySocialServiceActive()
	return (M.getMainSocialService() ~= nil)
end

function M.isSocialServiceActive(serviceType)
	if serviceType then
		echoInfo("serviceType: "..serviceType)
		local serviceDescription = M.getSocialServiceDescription(serviceType)
		if serviceDescription and serviceDescription.isActive then
			--	if we get here, then the service is logged in
			--	we also need to check if they haven't disabled it from social accounts
			return persistenceUserGame.public.activeAccounts[serviceDescription.connectionName]
		end
	end
	return false
end
------------------------------

function M.isSocialServiceLoggedIn(serviceType)
	if serviceType then
		local serviceDescription = M.getSocialServiceDescription(serviceType)
		if serviceDescription then
			return serviceDescription.service:isLogined()
		end
	end
	return false
end
------------------------------

function M.isSocialServiceAvailable(serviceType)
	if serviceType then
		local serviceDescription = M.getSocialServiceDescription(serviceType)
		if serviceDescription then
			return true
		end
	end
	return false
end
------------------------------

function M.getMainSocialService()
	-- these are sorted in order of priority
	for _, serviceDescription in pairs(M.activeServiceDescriptions) do
		if serviceDescription.isActive then
			if persistenceUserGame.public.activeAccounts[serviceDescription.connectionName] then
				return serviceDescription
			end
		end
	end
end
------------------------------

function M.getMainSocialServiceType()
	local serviceDescription = M.getMainSocialService()
	return serviceDescription.socialType
end
------------------------------

function M.getSocialServiceIcon(serviceType)
	local serviceDescription = M.getSocialServiceDescription(serviceType)
	if serviceDescription then
		return serviceDescription.icon
	end
end
------------------------------

------------------------------------------------

-- this is the main trigger.  it is called by gameManager once the game account is logged in. <><><>
function M.updateSocial()


	local function onUpdateSocialCompleted(result, data)
		--	acutally connect the accounts
		if (result ~= nil) and (result.code == runaway.PluginResult.kSucceed) then
			if (M.activeAccounts) then

	    		for _, account in pairs(M.activeAccounts) do
	    			if account.isActive then
		    			local accountType = account.account_type
						echoInfo("starting socialManager with accountType %d", accountType)

	    				local serviceDescription = M.getSocialServiceDescription(accountType)
						M.connectSocialService(serviceDescription, account, false, true, nil)
					end
				end				
			end
			if device.client_platform == "amazon" then -- We are not logging in with Amazon at this time so just set it is active
				M.activeServiceDescriptions[2].isActive = true
			end
		end
	end

	M.updateSocialInternal(onUpdateSocialCompleted)
end

function M.updateSocialInternal(callback)
    local function socialGetResult(result, data)
    	if (result ~= nil) and (result.code == runaway.PluginResult.kSucceed) then
    		-- echoInfo("*************************************************************************************")
    		-- dump(data, "socialGetResult.data")
    		-- echoInfo("*************************************************************************************")
    		-- dump(M.activeServiceDescriptions, "socialGetResult.activeServiceDescriptions")
    		-- echoInfo("*************************************************************************************")
    		-- dump(persistenceUserGame.public.activeAccounts, "persistenceUserGame.public.activeAccounts")
    		M.activeAccounts = data.userAccount
    		if (M.activeAccounts) then

    			-- M.activeAccounts -- this is the list of Social Accounts that have been registered against this user on the server (these are never removed)
    			-- persistenceUserGame.public.activeAccounts -- this is the list of currently social account *names* that this user has active

    			--	switch on/off the appropriate accounts
	    		for _, account in pairs(M.activeAccounts) do
	    			account.isActive = false
	    		end

	    		for socialName, isActive in pairs(persistenceUserGame.public.activeAccounts) do
	    			if isActive then
	    				echoInfo("Activating Social account type %s", socialName)
	    				local serviceDescription = M.getSocialServiceDescriptionByName(socialName)
	    				if (serviceDescription) then
	    			 		for _, account in pairs(M.activeAccounts) do
	    			 			if account.account_type == serviceDescription.socialType then
	    			 				echoInfo("activated!")
					    			account.isActive = true
					    		end
				    		end
	    				end
	    			end
	    		end
    		end
	    end
	    if type(callback) == "function" then
	    	callback(result)
	    end
    end
    M.serverManager.performRequest("SocialGet", nil, nil, socialGetResult, true)
end


------------------------------

function M.isGuest()
	echoInfo("socialManager.isGuest()")
	local isGuest = true
	for _, account in pairs(M.activeAccounts) do
		-- dump(account, "account")
		if account.account_type == SOCIAL_FACEBOOK then
			isGuest = false
			break
		end
	end

	echoInfo(isGuest)
	return isGuest
end

------------------------------

function M.connectSocialService(serviceDescription, expectedAccount, isNew, silent, callback)
	local isActive = persistenceUserGame.public.activeAccounts[serviceDescription.connectionName] 
	if isActive and serviceDescription then
		serviceDescription.isActive = true
	end

	local function onLoginCallback(result, updatedAccount, force)
		-- Sending this notification even if the connection fails,
		-- listener in migration manager is figuring out what to do based on the result:

		if (result.code == runaway.PluginResult.kSucceed) then
			--	ensure that we have logged in with the same account as previous
			if (updatedAccount ~= nil) and M.isAccountIdentical(serviceDescription.socialType, updatedAccount) then

				--	if updatedAccount is nil, then we were logged in already prior to this login attempt
				local function onServerRegistered(onServerResult, onServerData)
					echoInfo("onServerRegistered %s", serviceDescription.name)
					-- dump(onServerResult, "onServerResult", 10)

					local isUsed = false
					local isUsedCantMove = false
					if onServerData and type(onServerData.errors) == "table" and (not table.isEmpty(onServerData.errors)) then
						echoInfo("Found errors table")
						for _, errorNum in pairs(onServerData.errors) do
							echoInfo("Found error number #%d", errorNum)
							if errorNum == 9 then	--	social account already used
								isUsed = true
							elseif errorNum == 36 then 
								isUsedCantMove = true
							-- elseif errorNum == 38 then	-- ERROR_AUTH_DATA_UPDATE_REQUIRED
							-- 	--	handled in registerSocialAccountOnServer()
							end
						end
					end

					if (isUsed) and not M.silentSocialFailure then
						echoInfo("%s social Account was attached to a different game save", serviceDescription.name)

						if not force and not isUsedCantMove then
							local accountName = updatedAccount and ("'" .. updatedAccount.name .. "' ") or ""
							-- luahack to enable switching FB accounts between games.
							dialogManager.showDialog(
								string.format(SKELETON_MESSAGES.SocialAccountReassign.Title, serviceDescription.name),
								string.format(SKELETON_MESSAGES.SocialAccountReassign.Desc, serviceDescription.name, accountName),
								true,
								{ ui.SYSTEM_DIALOG_BUTTON_CANCEL, ui.SYSTEM_DIALOG_BUTTON_OK },
								function (button)
									if button == ui.SYSTEM_DIALOG_BUTTON_OK then
										onLoginCallback(result, updatedAccount, true)
									else
										M.disconnectSocialService(serviceDescription, callback)
									end
								end,
								{hsUrl = SKELETON_MESSAGES.SocialAccountReassign.hsUrl}
								)
						else
							dialogManager.showDialog(
								string.format(SKELETON_MESSAGES.SocialAccountDuplicate.Title, serviceDescription.name),
								string.format(SKELETON_MESSAGES.SocialAccountDuplicate.Desc, serviceDescription.name),
								true,
								{ ui.SYSTEM_DIALOG_BUTTON_OK },
								function ()
									M.disconnectSocialService(serviceDescription, callback)
								end,
								{hsUrl = SKELETON_MESSAGES.SocialAccountDuplicate.hsUrl}
								)
						end

					elseif (onServerResult.code == runaway.PluginResult.kSucceed) then
						-- inform the user that the social network has been connected
						echoInfo("Successfully attached %s to account", serviceDescription.name)
						local function onReloadedSocial(result, data)
							echoInfo("onReloadedSocial")
							if not silent then
								dialogManager.showDialog(
									string.format(SKELETON_MESSAGES.SocialAccountConnected.Title, serviceDescription.name),
									string.format(SKELETON_MESSAGES.SocialAccountConnected.Desc, serviceDescription.name),
									false,
									{ ui.SYSTEM_DIALOG_BUTTON_OK },
									nil,
									{hsUrl = SKELETON_MESSAGES.SocialAccountConnected.hsUrl})
							end
							-- I have moved callback to here. If this is causing issue, talk to me! - Nic
							if type(callback) == "function" then
								callback(onServerResult)
							end
							-- trigger loading the friends async for this social service
							M.loadFriends(serviceDescription, updatedAccount)
						end

						M.updateSocialInternal(onReloadedSocial)
					else
						M.disconnectSocialService(serviceDescription, callback)
					end
				end
				--------------

				local wasAccountRegisteredBefore = M.isAccountTypeRegistered(serviceDescription.socialType)
				echoInfo("M.connectSocialService(%s)\n  isNew=%s\n  wasAccountRegisteredBefore=%s",
					serviceDescription.name,
					(isNew and "true" or "false"),
					(wasAccountRegisteredBefore and "true" or "false"))

				--if isNew and not wasAccountRegisteredBefore and updatedAccount then
				if not wasAccountRegisteredBefore and updatedAccount then
					echoInfo("Registering %s Account on server", serviceDescription.name)
					M.registerSocialAccountOnServer(serviceDescription.socialType, updatedAccount.id, updatedAccount.gameSpecificId, updatedAccount.username or updatedAccount.name, onServerRegistered, force)
				else
					onServerRegistered(result)
				end

			else
				if not silent then
					dialogManager.showDialog(
								string.format(SKELETON_MESSAGES.SocialAccountInvalid.Title, serviceDescription.name),
								string.format(SKELETON_MESSAGES.SocialAccountInvalid.Desc, serviceDescription.name, serviceDescription.name),
								true,
								{ ui.SYSTEM_DIALOG_BUTTON_OK },
								function() end,
								{hsUrl = SKELETON_MESSAGES.SocialAccountInvalid.hsUrl}
								)
				end
				M.disconnectSocialService(serviceDescription, callback)
			end
		else
			if (result.code == runaway.PluginResult.kCancelled) and (serviceDescription.socialType == SOCIAL_GAMECENTER) then
				dialogManager.showDialog("Not connected", "Please try connecting to Game Center through your system settings.", false, {ui.SYSTEM_DIALOG_BUTTON_OK}, nil)
			end
			if not silent then
				dialogManager.showDialog(
							string.format(SKELETON_MESSAGES.SocialAccountFailed.Title, serviceDescription.name),
							string.format(SKELETON_MESSAGES.SocialAccountFailed.Desc, serviceDescription.name),
							true,
							{ ui.SYSTEM_DIALOG_BUTTON_OK },
							function() 
								if type(callback) == "function" then
									callback(result)
								end
							end,
							{hsUrl = SKELETON_MESSAGES.SocialAccountFailed.hsUrl}
							)
			else
				if type(callback) == "function" then
					callback(result)
				end
			end
		end
	end
	--------------
	--	special case - Facebook login but no game_specific_id available
	if (serviceDescription.socialType == 2) and expectedAccount and not expectedAccount.game_specific_id then
		expectedAccount = nil
	end
	if serviceDescription.service:isLogined() and expectedAccount then
		echoInfo("%s service is already logged in", serviceDescription.name)
		serviceDescription.isActive = true
		persistenceUserGame.public.activeAccounts[serviceDescription.connectionName] = true
		onLoginCallback({code=runaway.PluginResult.kSucceed}, expectedAccount)
	else
		echoInfo("%s service is not yet logged in", serviceDescription.name)
		M.loginService(serviceDescription, onLoginCallback)
	end
end
------------------------------

function M.isAccountIdentical(socialType, updatedAccount)
	-- dump(M.activeAccounts, "M.isAccountIdentical -> activeAccounts")
	-- dump(updatedAccount, "M.isAccountIdentical - > updatedAccount")

	-- find the appropriate account to compare to
	local expectedAccount = nil
	for _, account in pairs(M.activeAccounts) do
		if account.account_type == socialType then
			expectedAccount = account
			break
		end
	end

	local isIdentical = true
	if expectedAccount and updatedAccount then
		local expectedId = expectedAccount.account_id
		isIdentical = (expectedId == (updatedAccount.id or updatedAccount.account_id))
					or (expectedId == updatedAccount.game_specific_id) -- unsure if this version is used - leaving it in just in case
					or (expectedId == updatedAccount.gameSpecificId) -- confirmed required
	end
	-- echoInfo("isAccountIdentical = %s", isIdentical and "True" or "")
	return isIdentical
end



function M.isAccountTypeRegistered(accountType)
	echoInfo("Checking for previous connection of accountType #%s", accountType)
	for k, serverAccount in pairs(M.activeAccounts) do
		if serverAccount.account_type == accountType then
			--	special case for facebook accounts.  ensure that "game_specific_id" has a value
			if (accountType == 2) then
				return (serverAccount.game_specific_id ~= nil)
			end

			return true
		end
	end
	return false
end



function M.disconnectSocialService(serviceDescription, callback)
	echoInfo("disconnect %s", serviceDescription.name)

	local function onDisconnected(result)
		-- dump(result, "onDisconnected result")
		if result["code"] == runaway.PluginResult.kSucceed then
			echoInfo("%s logout success", serviceDescription.name)
			serviceDescription.isActive = false

			persistenceUserGame.public.activeAccounts[serviceDescription.connectionName] = false
		   M.stashFriends()	--	trigger friends list update

		   if SKELETON_MESSAGES.SocialAccountDisconnected then
			   dialogManager.showDialog( -- Show disconnect dialog if google play (required for featuring)
					string.format(SKELETON_MESSAGES.SocialAccountDisconnected.Title, serviceDescription.name),
					string.format(SKELETON_MESSAGES.SocialAccountDisconnected.Desc, serviceDescription.name),
					false,
					{ui.SYSTEM_DIALOG_BUTTON_OK},
					nil
				)
			end
		else
			echoInfo(serviceName.." logout failure")
		end
		if type(callback) == "function" then
			callback(result)
		end
	end

	if serviceDescription ~= nil then
		--	this version is nerfed.  It fully logs out the social account
		echoInfo("attempting to log out service #%s in disconnectSocialService", serviceName)
		serviceDescription.service:logout(onDisconnected)
	end
end
------------------------------

function M.loginService(serviceDescription, callback, params)
	--	called from connectSocialService
	echoInfo("attempting to login %s", serviceDescription.name)
	local socialAccessToken = nil 
	local gameSpecificId = nil
	local tokenForBusiness = nil

	local function onProfile(result, data)
		if type(callback) == "function" then
			if data then
				-- dump(data, "loginService->onProfile data")
				-- dump(socialAccessToken, "loginService->onProfile socialAccessToken")
				if (socialAccessToken == nil or socialAccessToken == "") and (serviceDescription.service ~= nil) then
					socialAccessToken = serviceDescription.service:getAccessToken()
				end

				--	this is for specific facebook handling where tokenForBusiness is shared across apps, but id is different per app
				if tokenForBusiness then
					data.id = tokenForBusiness
					data.gameSpecificId = gameSpecificId
				end

				data.socialAccessToken = socialAccessToken
			end
			callback(result, data)
		end
	end

	local function onServiceLogin(result, data)
		if result["code"] == runaway.PluginResult.kSucceed then
			echoInfo("%s social service login success", serviceDescription.name)
			serviceDescription.isActive = true

			persistenceUserGame.public.activeAccounts[serviceDescription.connectionName] = true
			-- dump(data, "onServiceLogin dataa")
			socialAccessToken = data and data.socialAccessToken

			--	this is for specific facebook handling where tokenForBusiness is shared across apps, but id is different per app
			gameSpecificId = data and data.gameSpecificId
			tokenForBusiness = data and data.tokenForBusiness

			-- now that we're logged in, grab the social profile
			serviceDescription.service:getProfile(onProfile)
		else
			echoInfo("%s social service login failure", serviceDescription.name)
			serviceDescription.isActive = false
			if type(callback) == "function" then
				callback(result)
			end
		end
	end

	if serviceDescription and serviceDescription.service then
		dump(serviceDescription.socialType,"serviceDescription.socialType")
		local params = {}
		if serviceDescription.socialType == SOCIAL_FACEBOOK then
			params = SOCIAL_FACEBOOK_PARAMS
		elseif serviceDescription.socialType == SOCIAL_GOOGLEPLAY then
			params = SOCIAL_GOOGLEPLAY_PARAMS
		elseif serviceDescription.socialType == SOCIAL_GAMECENTER then
			params = SOCIAL_GAMECENTER_PARAMS
		elseif serviceDescription.socialType == SOCIAL_AMAZON then
			params = SOCIAL_AMAZON_PARAMS
		end
		dump(params, "params")
		serviceDescription.service:login(params, onServiceLogin)
	end
end

------------------------------

function M.stashFriends(friends)
	--	if null is passed in, then we are just updating which ones are active
	friends = friends or M.allFriendsStash

	-- apply any friend updates required	
	for k = 1,#friends do
        local friend = friends[k]
    	friend.isActive = false
    	for _, serviceDescription in pairs(M.activeServiceDescriptions) do
    		if (serviceDescription.isActive) then
		    	if serviceDescription.service:applyFriendData(friend) then
		    		friend.isActive = true
		    	end
		    end
	    end
	end

	--	store full friend data
	M.allFriendsStash = friends
	M.activeFriendsStash = {}
	-- 	store friends on active social networks
	for k = 1,#friends do
        local friend = friends[k]
    	if friend.isActive then
    		table.insert(M.activeFriendsStash, friend)
    	end
    end

    echoInfo("Triggering REBUILD_FRIENDS in stashFriends")
	callbackManager.sendCallback(REBUILD_FRIENDS, {})

    -- dump(M.activeFriendsStash, "activeFriendsStash")
end
------------------------------

function M.loadFriends(serviceDescription)
	local function onFriendsLinked(result, data)
		--	now query the full list of social friends from the server
		-- dump(data, "data to link friends")
		-- echoInfo("FriendsLink result")
		M.updateFriends()
	end

	local function onGetAccountFriends(result, data) 
        if result["code"] == runaway.PluginResult.kSucceed then
	        local params = {}
	        -- dump(data, "new friends linker")
	        params.account_ids = {}
	        if data then
		        for k = 1,#data do
	        		local v = data[k]
		            table.insert(params.account_ids, v.id)
		        end
		    end
	        params.account_type = serviceDescription.socialType

	        -- only update the friends list if there are new friends added
	        local oldLinkedFriends = platformUserGameData.getDataForKey("linkedFriendIds") or {}
	        local linkedFriendVersion = platformUserGameData.getDataForKey("linkedFriendVersion")

	       	-- dump(oldLinkedFriends, "oldLinkedFriends")
	        local newLinkedFriends = clone(oldLinkedFriends)
	        local newFriendFound = false

	        if linkedFriendVersion == M.linkedFriendVersion then
				--	if our linkedFriendVersion matches up then only update with additions
		        for _, newFriend in pairs(params.account_ids) do
			        if not table.contains(oldLinkedFriends, newFriend) then
				        newFriendFound = true
	        			table.insert(newLinkedFriends, newFriend)
				    end
			    end
			else
				--	if our linkedFriendVersion doesn't match up up then update all friends
				newFriendFound = true
				newLinkedFriends = params.account_ids
			end

		    if newFriendFound then
		        -- dump(params, "friends data sent to FriendsLink")
		        --	send our updated list of friends on this social network to the server
		        echoInfo("socialManager.loadFriends() --> New friends found.  Updating list on server")
				platformUserGameData.setData("linkedFriendIds", newLinkedFriends)
				platformUserGameData.setData("linkedFriendVersion", 1)
		        M.serverManager.performRequest("FriendsLink", params, nil, onFriendsLinked, true)
		    else
		    	echoInfo("socialManager.loadFriends() --> No friends appear to have been changed.")
		    	onFriendsLinked()
		    end
        else
            echoInfo("Get friends failure")
        end
    end
   serviceDescription.service:getFriends(onGetAccountFriends)
end
------------------------------

function M.onFriendsUpdated(result, data)
	--this will be a list of ALL friends, not just the friends on a particular network
	if result["code"] == runaway.PluginResult.kSucceed then
		M.stashFriends(data.friends)
	else
		echoInfo("getLinkedFriendsResult failure")
	end
end

function M.updateFriends()
	M.serverManager.performRequest("FriendsGet", nil, nil, M.onFriendsUpdated, true)
end
------------------------------

function M.getFriendsList()
	return table.deepcopy(M.activeFriendsStash)
end

function M.findFriendWithId(friendId)
	for k, friend in pairs(M.activeFriendsStash) do
		if friend.id == friendId then
			return friend
		end
	end
end

function M.getDisplayNameForFriend(friendId)
	local friend = M.findFriendWithId(friendId)

	if friend then
		return friend.name
	else
		echoInfo("Couldn't find friend with id "..tostring(friendId))
	end
end

function M.getImageForFriend(friend, callback)
	--	get the first social service that this friend is attached to
	for k = 1,#M.activeServiceDescriptions do
        local serviceDescription = M.activeServiceDescriptions[k]
        if socialManager.isSocialServiceActive(serviceDescription.socialType) then
	    	--	applyFriendData returns true if this friend is active on this social network.
	    	if serviceDescription.service:applyFriendData(friend) then
	    		serviceDescription.service:getImageForFriend(friend, callback)
	    		return
			end
	    end
	end

	-- if the friend has a picUrl then just use the default social service
	if friend.picUrl ~= nil then
		--	fall back on the default icon rendering mechanism (which facebook happens to also use), even if facebook isn't active
		M.AllSocialServices[SOCIAL_FACEBOOK].service:getImageForFriend(friend, callback)
	end
end

------------------------------

function M.registerSocialAccountOnServer(accountType, accountId, gameSpecificId, username, callback, force)
	local params = {
			account_type = accountType,
			account_id = accountId,
			username = username,
			game_specific_id = gameSpecificId
		}
	if force then
		params.force = true
	end

	-- echoInfo("registerSocialAccountOnServer(%d)", accountType)
	-- dump(params, "sending to AccountAdd")

	local function onAccountAdded(result, data)
		if M.serverManager.hasSpecificError(data, 38)	then
			-- ERROR_AUTH_DATA_UPDATE_REQUIRED == 38
			gameManager.saveUserAuthData(accountType, accountId)
		end
		if (type(callback) == "function") then
			callback(result, data)
		end		
	end

	M.serverManager.performRequest("AccountAdd", params, nil, onAccountAdded, true)
end

------------------------------

function M.logout(serviceName, callback)
	local serviceDescription = M.getSocialServiceDescriptionByName(serviceName)
	
	if serviceDescription ~= nil then
		echoInfo("attempting to log out service #%s in logout", serviceName)
		serviceDescription.service:logout(function(result)
			if result["code"] == runaway.PluginResult.kSucceed then
				echoInfo(serviceName.." logout success")
				serviceDescription.isActive = false

				persistenceUserGame.public.activeAccounts[serviceDescription.connectionName] = false

				if callback ~= nil and type(callback) == "function" then
					callback(result)
				end
			else
				echoInfo(serviceName.." logout failure")
--				loginFlags[serviceName] = false
			end
		end)
	end
end

function M.logoutAllAccounts(callback)
	local serviceToLogout = nil
	for k = 1,#M.activeServiceDescriptions do
        local serviceDescription = M.activeServiceDescriptions[k]
		if (serviceDescription.isActive) then
			serviceDescription.service:logout(
				--	recursion
				function() 
					serviceDescription.isActive = false
					M.logoutAllAccounts(callback) 
				end
				)
			return
		end
	end

	--	no more to log out.
	if type(callback) == "function" then
		callback()
	end
end

------------------------------

--	auto loads GameCenter/Google Play 
--	unless it has been explicitly deactivated
function M.autoLoadBasicSocial()
	local serviceDescription = nil

	if device.client_platform == "android" then
		serviceDescription	= M.AllSocialServices[SOCIAL_GOOGLEPLAY]
	elseif device.client_platform =="amazon" then 
		serviceDescription	= M.AllSocialServices[SOCIAL_AMAZON]
	elseif device.client_platform == "ios" then
		serviceDescription	= M.AllSocialServices[SOCIAL_GAMECENTER]
	end		

	--	if the service is defined (it may be deactivated for this build)
	if serviceDescription then
		-- echoInfo("Autoconnecting %s social service", serviceDescription.name)
		-- M.connectSocialService(serviceDescription, nil, true, true, onSocialAutoConnected)

		--	if the service has never been logged in		
		if persistenceUserGame.public.activeAccounts[serviceDescription.connectionName] == nil then
			local function onSocialAutoConnected(result, data)
				if result.code ~= runaway.PluginResult.kSucceed then
					--	connecting the service failed, 
					--	so we'll explicitly mark it as disconnected
					--	this way we won't keep attempting it on this account in the future
					M.disconnectSocialService(serviceDescription, nil)
				end
			end

			echoInfo("Autoconnecting %s social service", serviceDescription.name)
			M.connectSocialService(serviceDescription, nil, true, true, onSocialAutoConnected)
		end
	end
end
------------------------------

function M.basicSocialIsLoggedIn()
	local serviceDescription = nil

	if device.client_platform == "android" then
		serviceDescription	= M.AllSocialServices[SOCIAL_GOOGLEPLAY]
	elseif device.client_platform == "amazon" then 
		serviceDescription	= M.AllSocialServices[SOCIAL_AMAZON]
	elseif device.client_platform == "ios" then
		serviceDescription	= M.AllSocialServices[SOCIAL_GAMECENTER]
	end

	return serviceDescription.service and serviceDescription.service:isLogined()
end
------------------------------

function M.basicSocialIsActive()
	local serviceDescription = nil

	if device.client_platform == "android" then
		serviceDescription	= M.AllSocialServices[SOCIAL_GOOGLEPLAY]
	elseif device.client_platform == "amazon" then 
		serviceDescription	= M.AllSocialServices[SOCIAL_AMAZON]
	elseif device.client_platform == "ios" then
		serviceDescription	= M.AllSocialServices[SOCIAL_GAMECENTER]
	end

	return serviceDescription.service and serviceDescription.isActive
end
------------------------------

function M.loginBasicSocial()
	local serviceDescription = nil

	if device.client_platform == "android" then
		serviceDescription	= M.AllSocialServices[SOCIAL_GOOGLEPLAY]
	elseif device.client_platform == "amazon" then
		serviceDescription	= M.AllSocialServices[SOCIAL_AMAZON]
	elseif device.client_platform == "ios" then
		serviceDescription	= M.AllSocialServices[SOCIAL_GAMECENTER]
	end		

	--	if the service is defined (it may be deactivated for this build)
	if serviceDescription then
		function onSocialForceConnected(result, data)
			
			if result.code ~= runaway.PluginResult.kSucceed then
				--	connecting the service failed, 
				--	so we'll explicitly mark it as disconnected
				--	this way we won't keep attempting it on this account in the future
				M.disconnectSocialService(serviceDescription, nil)
			end
		end
		echoInfo("Autoconnecting %s social service", serviceDescription.name)
		M.connectSocialService(serviceDescription, nil, true, true, onSocialForceConnected)
	end
end
------------------------------

function M.logoutBasicSocial(data)
	echoInfo("socialManager.logoutBasicSocial(data)")
	
	local serviceDescription = nil

	if device.client_platform == "android" then
		serviceDescription	= M.AllSocialServices[SOCIAL_GOOGLEPLAY]
	elseif device.client_platform == "amazon" then
		serviceDescription	= M.AllSocialServices[SOCIAL_AMAZON]
	elseif device.client_platform == "ios" then
		serviceDescription	= M.AllSocialServices[SOCIAL_GAMECENTER]
	end

	if serviceDescription then
		M.disconnectSocialService(serviceDescription, nil)
	end
end

function M.updateUser(newData)
	table.merge(M.currentUser, newData and newData.user or {})
	table.merge(M.currentUser, newData and newData.userGame or {})
	-- dump(M.currentUser, "socialManager.updateUser -> currentUser")
end

function M.getUser()
	local currentUser = table.deepcopy(M.currentUser)
	currentUser.accounts = M.activeAccounts
	return currentUser
end

function M.getDisplayName()
	local user = M.getUser()
	local mainAccountType = M.getMainSocialServiceType()
	local mainAccount

	local numberOfAccounts = #M.activeAccounts
	for i = 1, numberOfAccounts do
		local account = M.activeAccounts[i]
		if account.account_type == mainAccountType then
			mainAccount = account
		end
	end

	return mainAccount and mainAccount.username
end

function M.getUserIDForAccountType(accountType)
	local user = M.getUser()
	local targetAccount

	local numberOfAccounts = #user.accounts
	for i = 1, numberOfAccounts do
		local account = user.accounts[i]
		if account.account_type == accountType then
			targetAccount = account
		end
	end

	-- dump(targetAccount, "targetAccount")

	if targetAccount then
		return targetAccount.account_id
	end
end

function M.getFriendSocialAccount(friend, socialType)
	for _, account in pairs(friend.accounts) do
		if account.account_type == socialType then
			return account
		end
	end
end

function M.cleanup()
	M.allFriendsStash = {}
	M.activeFriendsStash = {}
	M.currentUser = {}
	M.activeAccounts = {}
	for _, serviceDescription in pairs(M.activeServiceDescriptions) do
		serviceDescription.isActive = false
	end
end
------------------------------

function M.achievementsActive()
	for k = 1,#M.activeServiceDescriptions do
        local serviceDescription = M.activeServiceDescriptions[k]
		if serviceDescription.isActive and serviceDescription.service:achievementsAvail() then
	    	return true
	    end
	end
end

M._goalDataCache = nil
function M.submitAchievement(achievementIdentifier, callback)
	-- echoInfo("Achievement submitted : %s", achievementIdentifier)
	--	 look up the achievement JSON file for the appropriate code
	
	local cacheIsSorted = false
	if M._goalDataCache == nil then
		local fileUtils = cc.FileUtils:getInstance()
		if fileUtils:isFileExist("json/goalData.json") then
			local goalData = fileUtils:getStringFromFile('json/goalData.json')
			if goalData then
				M._goalDataCache = dlcManager.decryptJson(goalData)
			else
				echoInfo("Error finding goalData.json")
			end
		end

		-- Flutter goalData contains orderIndex, so order table:
		if M._goalDataCache ~= nil then
			local sortedList = {}
			for i, goalItem in pairs(M._goalDataCache) do
				if goalItem.orderIndex == nil then break end
				sortedList[goalItem.orderIndex] = goalItem
			end
			if #sortedList > 0 then 
				M._goalDataCache = sortedList
				cacheIsSorted = true
			end
		end
	end

	if M._goalDataCache then
		-- find the achievementIdentifier in the goalDataCache
		
		local goal = nil
		if cacheIsSorted then
			for i = 1, #M._goalDataCache do
				local goalItem = M._goalDataCache[i]
				if (goalItem and goalItem.testID == achievementIdentifier) then
					goal = goalItem
					break
				end
			end
		else
			for goalID,goalData in pairs(M._goalDataCache) do
				if goalData.testID and 
					goalData.testID == achievementIdentifier then
					goal = goalData
				end
			end
		end

		if goal then
			-- dump(goal, "found achievement goal blob")
			for k = 1,#M.activeServiceDescriptions do
        		local serviceDescription = M.activeServiceDescriptions[k]
				if serviceDescription.isActive and serviceDescription.service:achievementsAvail() then
					local achievementInfo = serviceDescription.service:createAchievementInfo(goal)
			    	serviceDescription.service:submitAchievement(achievementInfo, callback)
			    	return 	-- there should only be one Achievement enabled social service active at a time (gamecenter/google play)	
			    end
			end
		else
			echoInfo("socialManager.submitAchievement - Achievement Identifier %s not found", achievementIdentifier or "(missing)")
			--dump(M._goalDataCache, "M._goalDataCache")
		end
	else
		echoInfo("socialManager.submitAchievement - goalData.json not found.  cannot submit achievement")			
	end
	callback({code = runaway.PluginResult.kFailed})
end

function M.showAchievements()
	for k = 1,#M.activeServiceDescriptions do
        local serviceDescription = M.activeServiceDescriptions[k]
		if serviceDescription.service:achievementsAvail() then
	    	serviceDescription.service:showAchievements()
	    end
	end
end

function M.resetAchievements()
	for k = 1,#M.activeServiceDescriptions do
        local serviceDescription = M.activeServiceDescriptions[k]
		if serviceDescription.service:achievementsAvail() then
	    	serviceDescription.service:resetAchievements()
	    end
	end
end

function M.getNewsItems(callback)
    M.serverManager.performRequest("NewsGet", nil, nil, callback, true)
end

function M.getAccessToken(socialType)
	local socialDescription = M.getSocialServiceDescription(socialType)
	if socialDescription and socialDescription.service then
		return socialDescription.service:getAccessToken()
	end
	return ""
end

function M.saveUser(socialType)
	local playerID

	if socialType == SOCIAL_MOBAGE then
		playerID = mobagePlayer.uid
	elseif socialType == SOCIAL_LCD then
		local lcdUser = lcdManager:getCurrentUser()
		playerID = lcdUser.userId
	end

	if playerID then
		gameManager.saveUserAuthData(socialType, playerID)
	else
		echoInfo("No player ID found for social type "..socialType)
	end
end

function M.setSilentSocialFailure(bool)
	M.silentSocialFailure = bool
end

return M

