local M = {}

M._timeCheated = 0
M._timeStartedThisSession = 0
M._serverTime = 0
M._serverTimeDiff = 0
M._isTimePaused = false
M._timerSchedule = nil

function M.cheatTime(secsToCheat)
	M._timeCheated = M._timeCheated + secsToCheat
end
------------------------------

function M.getTime()
	-- if not M._isTimePaused then
	-- 	M.setTime()
	-- end
	-- echoInfo("getTime %d", _serverTime)

	if not M._timerSchedule then
		M._timerSchedule = scheduler.scheduleGlobal(function()
			if not M._isTimePaused then M.setTime() end
		end, 1)
	end

	return M._serverTime
end
------------------------------

function M.setTime()
	M._serverTime = M._serverTimeDiff + os.time() + (M._timeCheated or 0)
end
------------------------------

function M.setServerTime(time)
	M.unPauseTime()
	M._serverTimeDiff = time - os.time()
end
------------------------------

function M.getSessionTime()
	return M.getTime() - M._timeStartedThisSession
end
------------------------------

function M.pauseTime()
	echoInfo("Pause Time!")
	if M._isTimePaused then return end
	M.setTime()
	M._isTimePaused = true
end
------------------------------

function M.unPauseTime()
	echoInfo("UnPause Time")
	M._isTimePaused = false
end
------------------------------

function M.unPauseTime()
	echoInfo("UnPause Time")
	M._isTimePaused = false
end
------------------------------

function M.getIsTimePaused() return M._isTimePaused end
------------------------------

function M.getCurrentDay()
	return os.date("%x", M.getTime())
end
------------------------------

function M.getDayForTime(time)
	return os.date("%x", time)
end
------------------------------

function M.getTimeForDay(day)
	local splitDate = utils.splitString(day, '/')

	local mm = tonumber(splitDate[1])
	local dd = tonumber(splitDate[2])
	local yyyy = tonumber(splitDate[3])

	local diff = M.getDifferenceToUTC()
	local time = diff + os.time{year=yyyy, month=mm, day=dd, hour=0}

	if time == nil then
		print("timeManager.getTimeForDay requires mm/dd/yyyy format!")
	end

	return time
end
------------------------------

function M.getDifferenceToUTC()
	local now = M.getTime()
	return os.difftime(now, os.time(os.date("!*t", now)))
end
------------------------------

function M.getYesterday()
	local currentTime = M.getTime()
	local yesterdayTime = currentTime - SECONDS_IN_DAY
	return M.getDayForTime(yesterdayTime)
end
------------------------------

function M.incrementSessionCount()
	echoInfo("Game-Restored, Incrementing Session Count")
	if persistenceData.userData then
		if persistenceData.userData.numSessions == nil then
			persistenceData.userData.numSessions = 0
		end
		persistenceData.userData.numSessions = persistenceData.userData.numSessions + 1
	end
end
------------------------------

function M.getRealDaysSinceStarting()
	local timeStarted = persistenceData.userData.timeStarted
	local timeStamp = M.getTime()
	local timeSince = timeStamp - timeStarted
	local numDays = math.floor(timeSince / SECONDS_IN_DAY)
	return numDays
end
------------------------------

function M.getSaveData()
	local saveData = {
		timeCheated = M._timeCheated
	}
	return saveData
end
------------------------------

function M.processSaveData(saveData)
	M._timeCheated = saveData.timeCheated
end
------------------------------

function M.init()
	M._timeCheated = M._timeCheated or 0
	M._timeStartedThisSession = M.getTime()

	local eventDispatcher     = cc.Director:getInstance():getEventDispatcher()
	M.onEnterForegroundListener = cc.EventListenerCustom:create(APP_ENTER_FOREGROUND_EVENT, M.incrementSessionCount) 
	eventDispatcher:addEventListenerWithFixedPriority(M.onEnterForegroundListener, 2)	--	lower priority, so that authentication stuff happens first
end
------------------------------

function M.cleanup()
	M._timeCheated = 0
	M._timeStartedThisSession = M.getTime()

	if (M.onEnterForegroundListener ~= nil) then
		local eventDispatcher     = cc.Director:getInstance():getEventDispatcher()
		eventDispatcher:removeEventListener(M.onEnterForegroundListener)	--	lower priority, so that authentication stuff happens first
		M.onEnterForegroundListener = nil
	end
end
------------------------------

return M