
local network = {}

--TODO Add these functions
function network.isLocalWiFiAvailable()
    return CCNetwork:isLocalWiFiAvailable()
end

function network.isInternetConnectionAvailable()
    return CCNetwork:isInternetConnectionAvailable()
end

function network.isHostNameReachable(hostname)
    if type(hostname) ~= "string" then
        echoError("network.isHostNameReachable() - invalid hostname %s", tostring(hostname))
        return false
    end
    return CCNetwork:isHostNameReachable(hostname)
end

function network.getInternetConnectionStatus()
    return CCNetwork:getInternetConnectionStatus()
end

function network.createHTTPRequest(callback, url, method, tag, reqData, headers)
    local request = runaway.HttpRequestNative:create()
    request:setUrl(url)

    if tag ~= nil then
        request:setTag(tag)
    end 

    if method == HTTPREQUEST_GET_IMAGE then
        method = HTTPREQUEST_GET
        request.type = HTTPREQUEST_GET_IMAGE
    end
    request:setRequestType(method)

    if reqData ~= nil then
        request:setRequestData(reqData)
    end

    if headers ~= nil then
        request:setHeaders(headers)
    end

    request:setResponseCallback(function(response) network.onResponse(response, callback) end);
    
    return request
end

function network.getHttpClient()
    return runaway.HttpClientNative:getInstance()
end

function network.enableCookies(cookieFileName)
    local httpclient = network.getHttpClient()
    local cookiePath = string.format("%s%s",device.writablePath, cookieFileName)
    echoInfo("enableCookies:%s",cookiePath)
    httpclient:enableCookies(cookiePath)
end

function network.clearCookieFile(cookieFileName)
    local cookiePath = string.format("%s%s",device.writablePath, cookieFileName)
    echoInfo("Deleting cookie:\n%s", cookiePath)
    if io.exists(cookiePath) then
        os.remove(cookiePath)
    end
end

function network.send(request, immediate)
    immediate = immediate or true
    local httpclient = network.getHttpClient()
    httpclient:send(request)
end

function network.onResponse(response, callback)
    local result = {}

    if response == nil then
        result.message = "No response"
        result.code = HTTPREQUEST_ERROR_NO_RESPONSE
        callback(result)
        return
    end

    local req = response:getHttpRequest()
    local tag = req:getTag()
    local responseCode = response:getResponseCode()
    
    local responseIsSucceed = response:isSucceed()
    if not responseIsSucceed then
        if (responseCode == 304) then
            result.message = "not modified";
            result.code = HTTPREQUEST_SUCCESS_NOT_MODIFIED;
        elseif (responseCode == 401) then
            result.message = string.format( "Unauthorised: %s", response:getErrorBuffer() )
            result.code = HTTPREQUEST_ERROR_UNAUTHORIZED
        else
            result.message = string.format( "response failed: %s", response:getErrorBuffer() )
            result.code = HTTPREQUEST_ERROR_NOT_SUCCEED
        end
        result.httpCode = responseCode
        callback(result, nil, req, response)
        return
    end

    result.message = "network success"
    result.code = HTTPREQUEST_SUCCESS

    -- BTH split out and store the response headers
    result.headers = {}
    local responseHeaders = response:getResponseHeader()
    -- echoInfo(responseHeaders)
    if responseHeaders then
        for headerItem in responseHeaders:gmatch("[^\r\n]+") do
            local separator = string.find(headerItem, " : ", 0, true);
            if (separator) then
                local headerKey = string.sub(headerItem, 0, separator - 1);
                local headerData = string.sub(headerItem, separator + 2);
                if headerKey == "Content-Type" and headerData == " image/gif" then
                    result = {}
                    result.message = string.format( "response failed: %s", response:getErrorBuffer() )
                    result.code = HTTPREQUEST_ERROR_NOT_SUCCEED
                    result.httpCode = responseCode
                    callback(result, nil, req, response)
                    return
                end
                result.headers[headerKey]=headerData
            else
                table.insert(result.headers, headerItem)
            end
        end
    end

    if req.type == HTTPREQUEST_GET_IMAGE then
        local responseSprite = response:getResponseSprite()
        callback(result, responseSprite, req, response)
    else
        local responseData = response:getResponseData()
        callback(result, responseData, req, response)
    end
end

function network.getHttpClientFile()
    return runaway.HttpClientNative:getInstance()
end

function network.createHTTPFileRequest(progress, callback, url, output, tag)
    local request = runaway.HttpRequestNative:create()

    request:setUrl(url)

    request:setOutputFile(output)

    if tag ~= nil then
        request:setTag(tag)
    end 

    request:setRequestType(HTTPREQUEST_GET)

    request:setProgressCallback(function(request) network.onProgressFile(request, progress) end)
    request:setResponseCallback(function(response) network.onResponseFile(response, callback) end)
    
    return request
end

function network.getFile(request)
    local httpclient = network.getHttpClientFile()
    httpclient:getFile(request)
end

function network.onProgressFile(request, callback)
    local result = {}

    if request == nil then
        result.message = "No request"
        result.code = HTTPREQUEST_ERROR_NO_REQUEST
        callback(result)
        return
    end

    -- local tag = req:getTag()
    -- echoInfo("onRequest tag:%s %d/%d", tag, req:getNowDownloaded(), req:getTotalToDownload())
    result.message = "network file progress success"
    result.code = HTTPREQUEST_SUCCESS
    callback(result, request:getNowDownloaded(), request:getTotalToDownload())
end

function network.onResponseFile(response, callback)
    local result = {}

    if response == nil then
        result.message = "No response"
        result.code = HTTPREQUEST_ERROR_NO_RESPONSE
        callback(result)
        return
    end

    local req = response:getHttpRequest()

    local tag = req:getTag()
    local responseCode = response:getResponseCode()
    local responseIsSucceed = response:isSucceed()
    if not responseIsSucceed then
        if (responseCode == 304) then
            result.message = "not modified";
            result.code = HTTPREQUEST_SUCCESS_NOT_MODIFIED;
        elseif responseCode == 403 then
            result.message = string.format( "Forbidden: %s", response:getErrorBuffer() )
            result.code = HTTPREQUEST_ERROR_FORBIDDEN
        elseif responseCode == 404 then
            result.message = string.format( "Not Found: %s", response:getErrorBuffer() )
            result.code = HTTPREQUEST_ERROR_NOT_FOUND
        else
            result.message = string.format( "response failed: %s", response:getErrorBuffer() )
            result.code = HTTPREQUEST_ERROR_NOT_SUCCEED
        end
        result.httpCode = responseCode
        callback(result)
        return
    end

    result.message = "network file success"
    result.code = HTTPREQUEST_SUCCESS

    -- BTH split out and store the response headers
    result.headers = {}
    local responseHeaders = response:getResponseHeader()
    -- echoInfo(responseHeaders)
    if responseHeaders then
        for headerItem in responseHeaders:gmatch("[^\r\n]+") do
            local separator = string.find(headerItem, " : ", 0, true);
            if (separator) then
                local headerKey = string.sub(headerItem, 0, separator - 1);
                local headerData = string.sub(headerItem, separator + 2);
                result.headers[headerKey]=headerData
            else
                table.insert(result.headers, headerItem)
            end
        end
    end


    callback(result, req)
end

function network.setTimeoutForRead(timeout)
    local httpclient = network.getHttpClient()
    httpclient:setTimeoutForRead(timeout)
end

return network
