local fyber = {}
local eventDispatcher = cc.Director:getInstance():getEventDispatcher()

function fyber.init(devInfo, pluginManager, onOffersAvailable, onStatusChange, onVCResponse, onVCError)

    fyber.fyberPlugin = pluginManager:loadPlugin("AdsFyber")
    if fyber.fyberPlugin ~= nil then
        fyber.fyberPlugin:configDeveloperInfo(devInfo)
        fyber.fyberPlugin:setDebugMode(true);

        fyber.onStatusChange = onStatusChange
        fyber.onOffersAvailable = onOffersAvailable
        fyber.onVCResponse = onVCResponse
        fyber.onVCError = onVCError

        if fyber.fyberPluginBrandEngageClientAreOffersAvailable == nil then
            fyber.fyberPluginBrandEngageClientAreOffersAvailable = cc.EventListenerCustom:create(FYBER_BRANDENGAGECLIENT_AREOFFERSAVAILABLE, fyber.BrandEngageAreOffersAvailable)
            eventDispatcher:addEventListenerWithFixedPriority(fyber.fyberPluginBrandEngageClientAreOffersAvailable, 1)
        end

        if fyber.fyberPluginBrandEngageClientStatusChange == nil then
            fyber.fyberPluginBrandEngageClientStatusChange = cc.EventListenerCustom:create(FYBER_BRANDENGAGECLIENT_STATUSCHANGE, fyber.BrandEngageStatusChange)
            eventDispatcher:addEventListenerWithFixedPriority(fyber.fyberPluginBrandEngageClientStatusChange, 1)
        end

        if fyber.fyberPluginVCDeltaOfCoinsResponse == nil then
            fyber.fyberPluginVCDeltaOfCoinsResponse = cc.EventListenerCustom:create(FYBER_VC_DELTAOFCOINSRESPONSE, fyber.VCDeltaOfCoinsResponse)
            eventDispatcher:addEventListenerWithFixedPriority(fyber.fyberPluginVCDeltaOfCoinsResponse, 1)
        end

        if fyber.fyberPluginBVCFailedWithError == nil then
            fyber.fyberPluginBVCFailedWithError = cc.EventListenerCustom:create(FYBER_VC_FAILEDWITHERROR, fyber.VCFailedWithError)
            eventDispatcher:addEventListenerWithFixedPriority(fyber.fyberPluginBVCFailedWithError, 1)
        end

        if fyber.fyberPluginOfferWallDismissed == nil then
            fyber.fyberPluginOfferWallDismissed = cc.EventListenerCustom:create(FYBER_OFFERWALL_CLOSED, fyber.OfferWallDismissed)
            eventDispatcher:addEventListenerWithFixedPriority(fyber.fyberPluginOfferWallDismissed, 1)
        end
    end
end

function fyber.requestVideo(placementId, currencyId)
    if fyber.fyberPlugin ~= nil then
        echoInfo("requestVideo:%s %s", placementId, currencyId)
        fyber.fyberPlugin:callFuncWithParam("requestVideo", {placementId, currencyId})
    end
end

function fyber.showVideo(placementId)
    if fyber.fyberPlugin ~= nil then
        fyber.oldMuteState = audioManager.getMute()
        audioManager.setMute(AUDIO.VOLUMES.MASTER.NAME, true)
        fyber.fyberPlugin:callFuncWithParam("showVideo", {placementId})
    end
end

function fyber.requestRewards(currencyId)
    if fyber.fyberPlugin ~= nil then
        fyber.fyberPlugin:callFuncWithParam("requestRewards", {currencyId})
    end
end

function fyber.showOfferWall(showCloseButtonOnLoad, shouldDismissOnRedirect)
    if fyber.fyberPlugin ~= nil then
        fyber.fyberPlugin:callFuncWithParam("showOfferWall", {showCloseButtonOnLoad, shouldDismissOnRedirect})
    end
end

function fyber.getAreOffersAvailable(placementId)
    local available = false;
    if fyber.fyberPlugin ~= nil then
        available = fyber.fyberPlugin:callBoolFuncWithParam("areOffersAvailable", {placementId})
    end
    echoInfo("offersavailable:%s %s", placementId, tostring(available))
    return available
end

function fyber.BrandEngageAreOffersAvailable(event)
    if event.getUserDataString then
        local eventDataString = event:getUserDataString()
        local eventData = json.decode(eventDataString)
        dump(eventData, "eventData")
        local areOffersAvailable = eventData["areOffersAvailable"]
        local placementId = eventData["placementId"]
        local isFromShowingVideo = eventData["isFromShowingVideo"]
        fyber.onOffersAvailable(areOffersAvailable, placementId, isFromShowingVideo)
    end
end

function fyber.BrandEngageStatusChange(event)
    if event.getUserDataString then
        eventDataString = event:getUserDataString()
        eventData = json.decode(eventDataString)
        dump(eventData, "eventData")
        local status = eventData["status"]
        local placementId = eventData["placementId"]
        if status == FyberBrandEngageStatus.STARTED then
            echoInfo("FyberBrandEngageClientStatusChange STARTED %s", placementId)
        elseif status == FyberBrandEngageStatus.CLOSE_FINISHED then
            echoInfo("FyberBrandEngageClientStatusChange CLOSE_FINISHED %s", placementId)
            audioManager.setMute(AUDIO.VOLUMES.MASTER.NAME, fyber.oldMuteState or false)
        elseif status == FyberBrandEngageStatus.CLOSE_ABORTED then
            echoInfo("FyberBrandEngageClientStatusChange CLOSE_ABORTED %s", placementId)
            audioManager.setMute(AUDIO.VOLUMES.MASTER.NAME, fyber.oldMuteState or false)
        elseif status == FyberBrandEngageStatus.ERROR then
            echoInfo("FyberBrandEngageClientStatusChange ERROR %s", placementId)
            audioManager.setMute(AUDIO.VOLUMES.MASTER.NAME, fyber.oldMuteState or false)
        end

        fyber.onStatusChange(status, placementId, FYBER_TYPE.REWARD_VIDEO)
    end
end

function fyber.VCDeltaOfCoinsResponse(event)
    if event.getUserDataString then
        eventDataString = event:getUserDataString()
        eventData = json.decode(eventDataString)
        dump(eventData, "eventData")
        local deltaOfCoins = eventData["deltaOfCoins"]
        local currencyId = eventData["currencyId"]
        local currencyName = eventData["currencyName"]
        local latestTransactionId = eventData["latestTransactionId"]

        fyber.onVCResponse(deltaOfCoins, currencyId)
    end
end

function fyber.VCFailedWithError(event)
    if event.getUserDataString then
        -- Error Reporting changed in c
        --
        -- eventDataString = event:getUserDataString()
        -- eventData = json.decode(eventDataString)
        -- dump(eventData, "eventData")
        -- local errorMsg = eventData["errorMessage"]
        -- echoInfo("Fyber VC Error:%s", errorMsg)
        fyber.onVCError(errorType, errorCode)--, errorMsg)
    end
end

function fyber.OfferWallDismissed(event)
    if event.getUserDataString then
        eventDataString = event:getUserDataString()
        eventData = json.decode(eventDataString)
        dump(eventData, "eventData")
        
        if eventData.status ~= nil then
            fyber.onStatusChange(eventData.status, nil, FYBER_TYPE.AD_WALL)
        end
    end
end

return fyber