local M = {}

HELPSHIFT_NOTIFICATION = "helpshiftNotification"

M.userId = nil
M.usernmae = nil
M.email = nil
M.onNotificationCallback = nil

-- https://developers.helpshift.com/ios/sdk-configuration/

function M.init(devInfo, pluginManager)
	echoInfo("Helpshift Plugin init")

	M.supportHelpshift = pluginManager:loadPlugin("SupportHelpshift")
	if M.supportHelpshift ~= nil then
		callbackManager.listenForCallback(HELPSHIFT_NOTIFICATION, M.onNotification)
	end
end

function M.getSharedOptions(options)
	options = options or {}
	if persistenceUser and persistenceUser.data and persistenceUser.data.disableSupport then
		options.enableContactUs = "NEVER"
	end
	options.showSearchOnNewConversation = "YES"
	options.showConversationResolutionQuestion = "NO"
	return options
end

function M.onNotificationHandler(callback)
	M.onNotificationCallback = callback
end

function M.onNotification(event)
    local resultString = event:getUserDataString()
    dump(resultString, "resultString")
    if M.onNotificationCallback ~= nil then
    	M.onNotificationCallback(event)
    end
end

function M.login()
	if M.supportHelpshift ~= nil then
		local data = {}
		data["userId"] = M.userId
		data["usernmae"] = M.usernmae
		data["email"] = M.email
		M.supportHelpshift:login(data)
	end
end

function M.logout()
	if M.supportHelpshift ~= nil then
		local data = {}
		M.supportHelpshift:login(data)
	end
end

function M.leaveBreadCrumb(msg)
	if M.supportHelpshift ~= nil then
		M.supportHelpshift:leaveBreadCrumb(msg)
	end
end

function M.showFAQs(options)
	if M.supportHelpshift ~= nil then
		options = M.getSharedOptions(options)
		M.supportHelpshift:showFAQs(options)
	end
end

function M.showFAQSection(sectionPublishId, options)
	if M.supportHelpshift ~= nil then
		options = M.getSharedOptions(options)
		M.supportHelpshift:showFAQSection(sectionPublishId, options)
	end
end

function M.showSingleFAQ(publishId, options)
	if M.supportHelpshift ~= nil then
		options = M.getSharedOptions(options)
		M.supportHelpshift:showSingleFAQ(publishId, options)
	end
end

function M.showConversation(options)
	if M.supportHelpshift ~= nil then
		options = M.getSharedOptions(options)
		M.supportHelpshift:showConversation(options)
	end
end


-- @param flowsData      A list of NSDictionary objects.
-- Following are the key/value pairs required in the dictionary
-- 1. "type" - this key should contain the type of flow. value should be one of the following :
--  1. Support.FAQS_FLOW
--  2. Support.CONVERSATION_FLOW
--  3. Support.FAQ_SECTION_FLOW
--  4. Support.SINGLE_FAQ_FLOW
-- 2. "title" - resource name of the title shown for the flow
-- 3. "data" - data required for the flow. its value should be as described below :
--     for FAQ_SECTION_FLOW, "data" should contain string section id
--     for SINGLE_FAQ_FLOW, "data" shoudl contain string faq id
--     for FAQsFlow and ConversationFlow, no "data" is required
-- 4. "config" - it should be a HashMap which is used to pass as config to Support APIs.
--     for DYNAMIC_FORM_FLOW, no "config" is required
function M.showDynamicForm(title, arr, options)
	if M.supportHelpshift ~= nil then
		M.supportHelpshift:showDynamicForm(title, arr, options)
	end
end

function M.getNotificationCount()
	if M.supportHelpshift ~= nil then
		return M.supportHelpshift:getNotificationCount()
	end

	return 0
end

function M.getNotificationCountFromRemote(callback)

	--Callback
	if M.supportHelpshift ~= nil then
		local listenerName = "getNotificationCountFromRemote"

		M.supportHelpshift:registerEventHandler(function(resultStr, dataStr)
			M.supportHelpshift:unregisterEventHandler(listenerName)
			local result = json.decode(resultStr)

			if result and result["type"] ~= nil then
				echoInfo("code:%s, msg:%s",result["code"], result["msg"])
				local data = json.decode(dataStr)
				if callback ~= nil and data ~= nil then
					callback(result, data["count"])
				end
			else
				echoInfo("Result does not contain a type")
			end
		end, listenerName)

		M.supportHelpshift:getNotificationCountFromRemote()
	end

	return 0
end


return M