
local scheduler = {}
local sharedScheduler = cc.Director:getInstance():getScheduler()

function scheduler.scheduleUpdateGlobal(callback)
   return sharedScheduler:scheduleScriptFunc(callback, 0, false)
end

function scheduler.scheduleGlobal(callback, interval)
   return sharedScheduler:scheduleScriptFunc(callback, interval, false)
end

function scheduler.unscheduleGlobal(callback)
	if not callback then return end
   sharedScheduler:unscheduleScriptEntry(callback)
end

function scheduler.performWithDelayGlobal(callback, time)
   local newCallback
   newCallback = sharedScheduler:scheduleScriptFunc(function()
      scheduler.unscheduleGlobal(newCallback)
      callback() 
   end, time, false)
   return newCallback
end
------------------------------

function scheduler.attachLocalScheduler(node)
	if not node.scheduledFunctions then node.scheduledFunctions = {} end

	-- To save having to manage scheduler 'handles', here you can supply an optional key for scheduling a function.
	-- Scheduled functions are all unscheduled automatically when the entity exits the display list.
	-- An interval of 0 schedules the function for every frame.
	-- (If you don't supply a key, the function reference itself is used as a key.)
	function node.scheduleFunction(func, interval, key)
		interval = interval or 0
		key = key or func
		if node.scheduledFunctions[key] then
			node.unscheduleFunction(key)
		end
		local handle = sharedScheduler:scheduleScriptFunc(func, interval, false)
		node.scheduledFunctions[key] = handle
		return key
	end

	function node.unscheduleFunction(key)
		local handle = node.scheduledFunctions[key]
		if not handle then return end
		sharedScheduler:unscheduleScriptEntry(handle)
		node.scheduledFunctions[key] = nil
	end

	function node.scheduleOnce(func, delay, key)
		delay = delay or 0
		key = key or func
		local selfRemoving
		selfRemoving = function()
			func()
			if (node.unscheduleFunction) then
				node.unscheduleFunction(key)
			end
		end
		node.scheduleFunction(selfRemoving, delay, key)
		return key
	end

	function node.unscheduleAllFunctions()
		for _,handle in pairs(node.scheduledFunctions) do
			sharedScheduler:unscheduleScriptEntry(handle)
		end
		node.scheduledFunctions = {}
	end

	function node.cleanupLocalScheduler()
		node.unscheduleAllFunctions()
	end
end

return scheduler
