local Server = require(cc.PACKAGE_NAME .. ".server.server")
local OAuth2 = class("OAuth2", Server)

function OAuth2:ctor(consumer_key, consumer_secret, params)
	echoInfo("OAuth:ctor()")
	OAuth2.super.ctor(self)

	params = params or {}

	self.m_supportsAuthHeader = true
	
	if type(params.UseAuthHeaders) == "boolean" then
		self.m_supportsAuthHeader = params.UseAuthHeaders
	end
end

function OAuth2:PerformRequest(method, url, arguments, headers, callback)	
	if not callback then
		error("PerformRequest needs a callback")
	end

	local function onLocalResponse(result, txt, request, response)
        if result.code == HTTPREQUEST_SUCCESS then
        	echoInfo("OAuth2 PerformRequest success from %q", url)

            if txt ~= nil and self.m_cipher ~= nil and self.m_key ~= nil and self.m_iv ~= nil then
            	txt = self:decrypt(txt)
        	end

	        local values = self:getValues(result.headers, txt)
	        if (values.errors ~= nil) and #values.errors then
	        	for _, errorNum in pairs(values.errors) do
	        		echoInfo("found error #%d", errorNum)
	        		local errorDetail = ERROR_CODES_SERVER[errorNum]
	        		if errorDetail ~= nil and errorDetail.showDialog == true then
		        		echoInfo("OAuth2 response error #%d:\n%s\n%s", errorNum, errorDetail.name, errorDetail.message)
	        			dialogManager.showDialog( 
                            errorDetail.name, 
                            errorDetail.message,
                            true,
                            { ui.SYSTEM_DIALOG_BUTTON_OK }, 
                            function()
                            	if errorDetail.needRestart then
                            		gameManager.reset()
                            	end
                            end,
                            {hsUrl = errorDetail.hsUrl});
	        			return
	        		else
	        			if errorDetail == nil then
			        		echoInfo("OAuth2 response error #%d: UNKNOWN ERROR NUMBER\nUNKNOWN", errorNum)
		        			dialogManager.showDialog( 
	                            "Unknown server error", 
	                            "An unknown server error #"..errorNum.." occured.",
	                            true,
	                            { ui.SYSTEM_DIALOG_BUTTON_OK }, 
	                            function()
	                        		gameManager.reset()
	                            end);
		        			return
		        		end
	        		end
		        end
	        end
	        -- dump(values, "values")
            callback(result, values)
        else
        	echoInfo("OAuth2 PerformRequest failure from %q", url)
        	if DEBUG == 2 then
        		-- dump(response, "response")
        		-- dump(response:getResponseCode(), "response:getResponseCode()")
        		-- dump(result.headers, "result.headers")
	        	dump(result, "result")
	        	dump(txt, "txt")
	        end
        	callback(result, nil)
        end
    end

    -- Build the 'Authorization' header if the provider supports it
	if self.m_supportsAuthHeader then
		local oauth_headers
		local useAuthHeader = false

		if self.m_oauth_access_token ~= nil then
			useAuthHeader = true
			oauth_headers = string.format("Bearer %s", self.m_oauth_access_token)
		else

			oauth_headers = { "OAuth" }

			if (type(arguments) == "table") then
				for k,v in pairs(arguments) do
					if k:match("^oauth_") then
						useAuthHeader = true
						table.insert(oauth_headers, k .. "=\"" .. self:oauth_encode(v) .. "\"")
						arguments[k] = nil
					end
				end
			end

			oauth_headers = table.concat(oauth_headers, ", ")
		end

		if useAuthHeader then
			headers["Authorization"] = oauth_headers
		end
	end

    local reqHeaders = {}
	for k, header in pairs(headers) do
	 	table.insert(reqHeaders, tostring(k)..":"..tostring(header))
	end

	if method == HTTPREQUEST_GET then
        url= string.format("%s%s", url, self:convertParamsToUrl(arguments))
        arguments = ""
    end

	if(self:checkIsJson(headers)) then
		arguments = json.encode(arguments)
	end

	if self.m_cipher ~= nil and self.m_key ~= nil and self.m_iv ~= nil then
		if arguments and (string.len(arguments) ~= 0) then
        	arguments = self:encrypt(arguments)
        end
        table.insert(reqHeaders, "encrypted:true")
    end
	
	-- dump(url, "url")
	-- dump(method, "method")
	-- dump(arguments,"arguments")
	-- dump(reqHeaders,"reqHeaders")
	local req = network.createHTTPRequest(onLocalResponse, url, method, "OAuth2", arguments, reqHeaders)
    network.send(req)
end


return OAuth2