local lcdFreeManager = {}
local eventDispatcher = cc.Director:getInstance():getEventDispatcher()
local guid = require(cc.PACKAGE_NAME..".libraries.guid")
local jwt = require(cc.PACKAGE_NAME..".libraries.luajwt")

local StateContainerFactory = require(cc.PACKAGE_NAME .. ".api.StateContainer")
local authState     = StateContainerFactory.createContainer()

lcdFreeManager.LoginEndpoint = "LoginNative"
lcdFreeManager.skus = IAP_SKUS 	--	should be defined in game's config.lua
lcdFreeManager.SKU_BATCH_SIZE = 10

function lcdFreeManager:init()
	echoInfo("lcdFreeManager:init()")
	lcdFreeManager.waitLayer = nil
	self.isPaused = true
	self.currentUser = nil
	self.currencyCode = nil
	self.currencyRate = 0
	self.onGetVCBundlesCallbacks = {}

    local function authStateListener(event)
        return event.values
    end
    authState.init(authStateListener, "authentication", "key")		---	TODO: shouldn't need this with LCD gone
    self:reload()

	local pluginManager = runaway.PluginManager:getInstance()
	self.iapPlugin = pluginManager:loadPlugin("RunawayIAP")

	if self:isAvailable() then
		self.iapPlugin:configDeveloperInfo(devInfo)
		self.iapPlugin:setDebugMode(false)
	end

	-- wire up the return from purchasing a product
	local function onConfirmTransactionRequested(result, data)
		self:confirmTransaction(result, data)
	end
	self:registerCallback(self.iapPlugin, "confirmTransaction", onConfirmTransactionRequested, true)	

	local function onCloseTransactionCompleted(result, data)
		self:onCloseTransactionCompleted(result, data)
	end
	self:registerCallback(self.iapPlugin, "closeTransaction", onCloseTransactionCompleted, true)	

end

function lcdFreeManager:reload()
    self.authData = authState.load()
end

function lcdFreeManager:cleanup()
end


function lcdFreeManager:isAvailable()
	return true
end


--	getAccessToken is simply a sample stub at this stage, until we get this all working
--	is this even relevant anymore?
function lcdFreeManager:getAccessToken()
	echoInfo("lcdFreeManager:getAccessToken called.  THIS ISNT IMPLEMENTED")
	local function t2s(o)
        if type(o) == 'table' then
            local s = '{ '
            for k,v in pairs(o) do
                if type(k) ~= 'number' then k = '"'..k..'"' end
                s = s .. '['..k..'] = ' .. t2s(v) .. ','
            end

            return s .. '} '
        else
            return tostring(o)
        end
	end

	local key = "example_key"

	local claim = {
		iss = "12345678",
		nbf = os.time(),
		exp = os.time() + 3600,
	}

	local alg = "HS256" -- default alg
	local token, err = jwt.encode(claim, key, alg)

	echoInfo("getAccessToken -> Token: %s", token)

	--	test it with the following:
	-- local validate = true -- validate exp and nbf (default: true)
	-- local decoded, err = jwt.decode(token, key, validate)

	-- echoInfo("getAccessToken -> Claim: %s", t2s(decoded) )

	return token
end


function lcdFreeManager:getSDKVersion()
	return 1.0
end

function lcdFreeManager:isSandbox()
	--	TODO: need to wire up.
	return not runaway.GameConfig:isProduction() 
end


function lcdFreeManager:pause()
    echoInfo("LCDFree Pausing")
	if not(self.isPaused) then
		self.isPaused = true
	end
end

function lcdFreeManager:resume()
    echoInfo("LCDFree resuming")
	self.isPaused = false
	self:setStarted(true)

	self:onLcdSessionUpdate()
end


function lcdFreeManager:getIsPaused()
	return self.isPaused
end

function lcdFreeManager:setStarted(started)
	lcdFreeManager.isStarted = started
	if (not started) then
		self:pause()
	end
end

function lcdFreeManager:getCurrentUser()
	if not self.authData then
	    self.authData = authState.load()
	end
	return self.authData or {}
end

function lcdFreeManager:registerCallback(plugin, functionName, callback, doNotUnregister)
	plugin:registerEventHandler(function(resultStr, dataStr)
		if not doNotUnregister then
			echoInfo("lcdFreeManager unwiring %s function", functionName)
			plugin:unregisterEventHandler(functionName)
		end
		-- echoInfo("LCDFree result")
		local result = json.decode(resultStr)
		-- dump(result, "self.lcdPlugin(functionName=".. functionName..")")
		if result and result["type"] ~= nil then
			-- echoInfo("code:%s, msg:%s dataStr:%s",result["code"], result["msg"], dataStr)
			local resultData = json.decode(dataStr)
			if type(dataStr) == "string" and type(resultData) == "nil" then	--	handle the case where the returned data is not JSON, but a raw string
				resultData = dataStr
			end
			callback(result, resultData)
		else
			echoInfo("Result does not contain a type")
		end
	end, functionName)
end

function lcdFreeManager:callFuncWithParam(plugin, functionName, data, callback)
	callback = callback or function() end
end

function lcdFreeManager:linkAccount(callback)
	echoInfo("lcdFreeManager:linkAccount called and ignored")
end

function lcdFreeManager:loadAccount(callback)
	echoInfo("lcdFreeManager:loadAccount called and ignored")
end

function lcdFreeManager.attachSkuValues(bundles)
	-- dump(bundles,"attachSkuValues")
	-- dump(lcdFreeManager.skus,"attachSkuValues.skus")
	for sku, value in pairs(lcdFreeManager.skus) do
		for _, bundle in pairs(bundles) do
			if (bundle.sku == sku) or string.endsWith(bundle.sku, "."..sku) then
				bundle.value = value
				bundle.shortSku = sku
			end
		end
	end
	-- dump(bundles,"attachSkuValues- post")
	return bundles
end


function lcdFreeManager:getVCBundleBatches(remainingSkus)
	-- dump(remainingSkus, "getVCBundleBatches -> remainingSkus")

	local function getNextBatch()
		local skuCount = 0
		local skuBatch = {}	
		local newRemainingSkus = {}		

		-- generate two sku lists - the current batch, and the remaining skus.
		-- dump(remainingSkus, "getNextBatch -> remainingSkus (pre split)")
		for _, sku in pairs(remainingSkus) do
			skuCount = skuCount + 1
			if skuCount > lcdFreeManager.SKU_BATCH_SIZE then
				table.insert(newRemainingSkus, sku)
			else
				table.insert(skuBatch, sku)
			end
		end	
		remainingSkus = newRemainingSkus
		-- dump(skuBatch, "getNextBatch -> skuBatch")
		-- dump(remainingSkus, "getNextBatch -> remainingSkus (post split)")
		return skuBatch
	end


	local function onCompletedFetchVCBundles(result)
		-- dump(result, "onCompletedFetchVCBundles result")
		bundleData = self:fetchVCBundles()
		-- dump(bundleData, "onCompletedFetchVCBundles bundleData")
		echoInfo("Sending EVENT_VC_PACKAGES_RECEIVED")
		callbackManager.sendCallback(EVENT_VC_PACKAGES_RECEIVED, {result = result, data = bundleData})
		for k, callbackFunc in pairs(self.onGetVCBundlesCallbacks) do
			callbackFunc(bundleData)
		end
		self.onGetVCBundlesCallbacks = {}
		self.fetchingVCBundles = false
	end

	local function mergeBundles(existingList, newList)
		existingList = existingList or {}
		-- dump(existingList, "mergeBundles existingList")
		-- dump(newList, "mergeBundles newList")
		local newCount = #newList
		local oldCount = #existingList

		for i = 1, newCount do
			local newBundle = newList[i]
			local found = false

			for j = 1, oldCount do
				if newBundle.sku == existingList[j].sku then
					found = true
					break
				end
			end
			if not found then
				table.insert(existingList, newBundle)
			end
		end

		-- dump(existingList, "mergeBundles existingList completed")
		return existingList
	end

	local runProcess = nil
	local function onGotProducts(result, data)
		local bundleData = nil
		if (result.code == runaway.PluginResult.kSucceed) and data then
			-- merge the new list into the existing list
			self.bundles = mergeBundles(self.bundles, lcdFreeManager.attachSkuValues(data.bundles))
			-- dump(self.bundles, "onGotProducts partial result.")
			runProcess()
		else
			onCompletedFetchVCBundles(result)
		end
	end
	self:registerCallback(self.iapPlugin, "getProducts", onGotProducts, true)	--	true = doNotUnregister

	runProcess = function()
		-- echoInfo("lcdFreeManager:getVCBundleBatches -> runProcess")
		local skuBatch = getNextBatch()
		if #skuBatch == 0 then
			onCompletedFetchVCBundles({code=runaway.PluginResult.kSucceed})
		else			
			local skuList = table.concat(skuBatch, "|")
			self.iapPlugin:getProducts({items=skuList})
		end
	end

	runProcess()
	-- self.iapPlugin:getProducts({items=""})
end


--	this method handles both callback and EVENT_VC_PACKAGES_RECEIVED event
function lcdFreeManager:getVCBundles(optionalCallback)
	echoInfo("lcdFreeManager:getVCBundles()")
	if type(optionalCallback) == "function" then
		table.insert(self.onGetVCBundlesCallbacks, optionalCallback)
	end
	if not self.fetchingVCBundles then
		if self.bundles == nil then
			self.fetchingVCBundles = true

			-- get the full list of skus
			local products = table.keys(lcdFreeManager.skus)
			for i=1,#products do
				local product = products[i]
				products[i] = string.format("%s%s%s", IAP_PREFIX, product, IAP_SUFFIX)
			end

			self:getVCBundleBatches(products)
		end
	end
end

function lcdFreeManager:fetchVCBundles()
	echoInfo("lcdFreeManager:fetchVCBundles()")
	--returns null if we haven't done getVCBundles yet
	return self.bundles and {bundles = self.bundles}
end



function lcdFreeManager:getSkuItem(sku)
   if self.bundles then
      -- dump(self.bundles, "getting sku: " .. (sku or "noSKU") )
      for i, item in pairs(self.bundles) do
         -- find the item in the bundles
         if (item.sku == sku) or (string.endsWith(item.sku, "."..sku)) then
            return item
         end
      end
   end
end

function lcdFreeManager:getVCBalance(callback)
	echoInfo("lcdFreeManager:getVCBalance called and ignored")
	return persistenceData.userData and persistenceData.userData.hardC or 0
end

function lcdFreeManager:purchaseVCBundle(sku)
	echoInfo("lcdFreeManager:purchaseVCBundle called for sku:%s", sku)

	-- the confirmTransaction callback is wired up in init()
	self.iapPlugin:payForProduct({sku = sku})
end

---	also has the side effect of pre-loading the sku bundles
function lcdFreeManager:processUnConsumedPurchases()
	-- the confirmTransaction callback is wired up in init()
	local function onBundlesLoaded()
		echoInfo("triggering runawayPlugin.processUnConsumedPurchases")
		self.iapPlugin:processUnConsumedPurchases()
	end

	if self.bundles then
		onBundlesLoaded()
	else
		-- we don't have the sku data yet, so grab that first.
		lcdFreeManager:getVCBundles(onBundlesLoaded)
	end
end


function lcdFreeManager:setOnProductPurchasedFunc(callbackFunc)
	echoInfo("lcdFreeManager wiring up 'onProductPurchasedFunc' callbackFunc")
	self.onProductPurchasedFunc = callbackFunc

	local function onProductPaidFor(result, transData)
		-- dump(result, "lcdFreeManager internal onProductPaidFor result")
		-- dump(transData, "lcdFreeManager onPayForProduct transData")
		if type(self.onProductPurchasedFunc) == "function" then
			--TODO Check, this only gets called when purchase fails
			local data = {}
			if device.client_platform == "android" or device.client_platform == "amazon" then
				data.transactionId = transData.orderId
				data.sku = transData.productId
			else
			end

			self.onProductPurchasedFunc(result, data, transData)
		else
			echoInfo("purchased %s but there was no onProductPurchasedFunc", sku or "undefined")
		end		
	end

	self:registerCallback(self.iapPlugin, "payForProduct", onProductPaidFor, true)
end


function lcdFreeManager:confirmTransaction(result, transData)
	echoInfo("lcdFreeManager:confirmPurchase()")
	-- dump(result, "confirmTransaction.result")
	-- dump(transData, "confirmTransaction.transData")

	local function onConfirmedTransaction(result, data)
		-- dump(result, "onConfirmedTransaction.result", 5)
		-- dump(data, "onConfirmedTransaction.data", 5)
		self:onTransactionConfirmed(result, data, transData)
	end

	local params = {}
	
	params["android"] = false
	if device.client_platform == "android" then
		params["android"] = true
			--{"INAPP_PURCHASE_DATA":"{\"orderId\":\"GPA.1382-0531-3465-93510\",\"packageName\":\"com.runawayplay.Splash.dev\",\"productId\":\"pt_ios_hc_2\",\"purchaseTime\":1446693085939,\"purchaseState\":0,\"developerPayload\":\"payForProduct\",\"purchaseToken\":\"ibagglgiibaiifnlpffkdgob.AO-J1OwhUqmPKLfMSXUJY7SLkmVnBoG2xU0TNo_VE-677DVJHjgzGDdkMMIBYovkescHkyE3ArUVrp-oFonCIPxQm23SC_UX2qwuog1v7EcvPsLJMEBlgHGGBv8ULL5Dwog_vO93jxgs\"}","INAPP_DATA_SIGNATURE":"RxkA+nZiXc1qwOclW8T+5Hiy9IYSyufDVwxUjtQNIJAGYJl9OIaoHhht\/YifXwEVhV1FI8ObKiwso\/F5IJ9YSkbLakNCvQjj7pHRBGFW6EbWz3Z1qCxs4Hn1m7jR\/Q720Q\/POGC4GepYTxkJWbMbs7264XK6ugmZL25fbJz5iMjYYaLNXE\/iHU4lQIeVgq97D2ZivEqGiMHg3KRkylKh6qeBIimnYXJii3Mo5j3mUaCb7Fn2h4vWang+xJFDn\/8fL3GGa4VH90954j7USbpsrGg2g\/TchKy8YoGdr1M8DMPbjacRLlgfMba2p9hMPgWWvZgzi3CHTsgljtZLPGC2IQ==","RESPONSE_CODE":0}
		params["amazon"] = false
		params["receipt"] = transData.receiptData
		local iapData = json.decode(transData.INAPP_PURCHASE_DATA)
		dump(iapData,"iapData")
		params["transactionId"] = iapData.orderId
		params["packageName"] = iapData.packageName
		params["sku"] = iapData.productId 	--	sku
		params["purchaseToken"] = iapData.purchaseToken
	elseif device.client_platform == "amazon" then
		params["android"] = true
		params["amazon"] = true
		local receiptData = transData.receipt
		params["transactionId"] = receiptData.receiptId
		params["sku"] = receiptData.sku
		params["userId"] = transData.userId
	else
		params["receipt"] = transData.receiptData --Needed?
		params["transactionId"] = transData.transactionId

		-- for ios subscription testing
		if not runaway.GameConfig:isProduction()  then
			params["password"] = STAGING_SHARED_SECRET
		end
		-- params["productId"] = transData.productId 	--	can we get sku on ios?
	end
	params["isSandbox"] = self:isSandbox()		--	TODO: do this properly
	params["timestamp"] = timeManager.getTime()
	dump(params, "sending params to ConfirmTransaction")
	-- onConfirmedTransaction({}, params)
	serverManager.performRequest("ConfirmTransaction", params, nil, onConfirmedTransaction, true)
end


function lcdFreeManager:onTransactionConfirmed(result, data, transData)
	-- Hack to make it always work
	-- result.code = runaway.PluginResult.kSucceed
	-- Amazon

	-- Android
	-- local iapData = json.decode(transData.INAPP_PURCHASE_DATA)
	-- data = {}
	-- data.transactionId = iapData.orderId
	-- data.sku = iapData.productId

	-- dump(result, "onTransactionConfirmed.result")
	-- dump(data, "onTransactionConfirmed.receiptData")
	if result.code == runaway.PluginResult.kSucceed
		and type(data) == "table"
		and data.transactionId 
		and data.sku then
		
		local transactionId = data.transactionId
		local sku = data.sku


		--	if we didn't get a sku returned, then the confirm failed somehow
		if sku then
			--	if this transaction has already been consumed, then just close the transaction
			if serverManager.hasSpecificError(data, 39) then
				self:closeTransaction(data)
			else
				if type(self.onProductPurchasedFunc) == "function" then
					echoInfo("purchased sku: %s. sending to game code", sku)
					self.onProductPurchasedFunc(result, data, transData)
					echoInfo("game has completed purchase of sku: %s.  closing transaction", sku)
					self:closeTransaction(data)
				else
					echoInfo("purchased sku: %s but there was no onProductPurchasedFunc", sku)
					--	not actually handled, but will likely be handled correctly later via processUnConsumedPurchases
					--	this is usually because there was a crash during the previous purchase process, and this will be called on startup.
				end	
			end
			return --	transaction is handled, so bail out here
		end

		--	if we get here, then the transaction isn't confirmed.
		echoInfo("onTransactionConfirmed failed for transactionId: %s", transactionId)
	else
		echoInfo("onTransactionConfirmed called with no transactionId or Transaction was not found")
		result.code = runaway.PluginResult.kFailed
		if type(self.onProductPurchasedFunc) == "function" then
			self.onProductPurchasedFunc(result, data, transData)
		end	
	end
end


function lcdFreeManager:closeTransaction(data)
	data.errors = nil	--	remove any errors from the data (converting table to java doesn't like non string values).
	-- dump(data, "lcdFreeManager:closeTransaction")
	self.iapPlugin:closeTransaction(data)
end
 

function lcdFreeManager:onCloseTransactionCompleted(result, data)
	if result.code == runaway.PluginResult.kSucceed and data.transactionId then
		local transactionId = data.transactionId
		--	the OS has closed the transaction, now update our server
		echoInfo("lcdFreeManager:onCloseTransactionCompleted %s", transactionId)
		local params = {}
		params["transactionId"] = transactionId
		params["timestamp"] = timeManager.getTime()
		local function ignoredCallback() end
		serverManager.performRequest("CloseTransaction", params, nil, ignoredCallback, true)
	end
end



function lcdFreeManager:cancelTransaction(transactionId)
	echoInfo("lcdFreeManager:cancelTransaction %s", transactionId)
	self.iapPlugin:cancelTransaction({transactionId = tostring(transactionId)})		-- TODO:	nothing yet wired up in C++.
end


function lcdFreeManager:spend(item_id, value, quantity, callback)
	echoInfo("lcdFreeManager:spend()")
	callback = callback or function() end
	if LOCAL_MODE then
		callback({code = runaway.PluginResult.kSucceed})
	else
	    if value < 0 then   --  ensure that the spent amount is sent as a negative
	        value = -value
	    end
		local params = {}
	    params["item_id"] = item_id or "undefined"
	    params["value"] = value
	    params["quantity"] = quantity
	    params["current_hc"] = gameManager.getHardCFunc()
	    serverManager.performRequest("Spend", params, nil, callback, true)
	end
end

function lcdFreeManager:gift(amount, source, callback)
	echoInfo("lcdFreeManager:gift()")
	callback = callback or function() end

	if LOCAL_MODE then
		callback({code = runaway.PluginResult.kSucceed})
	else
		local params = {}
	    params["amount"] = amount
	    params["source"] = source or "undefined"
	    params["timestamp"] = timeManager.getTime()
		params["current_hc"] = gameManager.getHardCFunc()
	    serverManager.performRequest("Gift", params, nil, callback, true)
	end
end

--********************************
--**   Event handling section   **
--********************************

--	TODO: figure out if this is actually used.
function lcdFreeManager:onLcdSessionUpdate(newUser)
	local forceUpdate = (self.authData == nil) or (newUser == nil) 
	local currentUser = self:getCurrentUser()

	if forceUpdate or (newUser.userId ~= currentUser.userId) then
    	lcdFreeManager.accessToken = self:getAccessToken()
    	if type(lcdFreeManager.onNewSessionFunc) == "function" then
    		lcdFreeManager.onNewSessionFunc({user = currentUser})
    	end
    end
end

function lcdFreeManager.isDeveloper()
	return lcdFreeManager.developer and (lcdFreeManager.developer == 1)
end

function lcdFreeManager.setOnNewSessionFunc(func)
	lcdFreeManager.onNewSessionFunc = func
end

function lcdFreeManager.onGameEnterBackground(event)
	lcdFreeManager:pause()
end

function lcdFreeManager.onGameEnterForeground(event)
	if lcdFreeManager.isStarted then
		echoInfo("LCD Resuming on enter foreground")
		lcdFreeManager:resume()
	else
		echoInfo("LCD Skipping resume on enter foreground because it's paused")
	end
end

function lcdFreeManager:getCurrencyRate(currencyCode, callback)
	echoInfo("lcdFreeManager:getCurrencyRate %s", currencyCode)
	local function returnResult(rate)
		if type(callback) == "function" then
			callback(self.currencyRate or 0)
		end
	end

	if LOCAL_MODE or (self.currencyCode == currencyCode) then
		returnResult(self.currencyRate or 0)
	else
		local function onGetCurrencyResult(result, data)
			if result["code"] == runaway.PluginResult.kSucceed then
				self.currencyCode = currencyCode
				self.currencyRate = data.rate or 0
				echoInfo ("found currencyRate for %s = %d", currencyCode or "NONE", self.currencyRate)
				returnResult(self.currencyRate)
			else
				returnResult(0)
			end
		end

		local params = {}
		params["currency_code"] = currencyCode
		serverManager.performRequest("CurrencyRate", params, nil, onGetCurrencyResult)
	end
end

function lcdFreeManager:getAppStoreReceipt()
	echoInfo("lcdFreeManager:getAppStoreReceipt")

	return self.iapPlugin:callStringFuncWithParam("getAppStoreReceipt", {})
end

return lcdFreeManager
