local SocialService = require(cc.PACKAGE_NAME..".social.socialService")

local AmazonSocialService = class("AmazonSocialService", SocialService)

function AmazonSocialService:ctor()
	echoInfo("AmazonSocialService:ctor()")
	AmazonSocialService.super.ctor(self)
	local pluginManager = runaway.PluginManager:getInstance()
	self.userAmazon = pluginManager:loadPlugin("UserAmazon") -- login, profile
	-- self.shareAmazon = pluginManager:loadPlugin("ShareAmazon") -- friends
	self.socialAmazon = pluginManager:loadPlugin("SocialAmazon") -- cheevo's
end

function AmazonSocialService:init(devInfo)
	echoInfo("AmazonSocialService:init()")
	self.friendStash = {}

	self.userAmazon:configDeveloperInfo(devInfo)
	self.userAmazon:setDebugMode(true)

	-- self.shareAmazon:configDeveloperInfo(devInfo)
	-- self.shareAmazon:setDebugMode(true)

	self.socialAmazon:configDeveloperInfo(devInfo)
	self.socialAmazon:setDebugMode(true)
end

function AmazonSocialService:isAvailable()

	if self.userAmazon == nil then
		echoInfo("userAmazon not available on this platform")
		return false
	end

	return true
end

function AmazonSocialService:isLogined()
	return self.userAmazon:isLogined()
end

function AmazonSocialService:login(params, callback)
	if self:isLogined() then
		--	gamecenter is never actually logged out. 
		--	if we log out and back in again, just return success.
		echoInfo("Attempting to log in Amazon (already logged in")
		callback({
		         ["type"] = "login",
		         ["code"] = runaway.PluginResult.kSucceed,
		         ["msg"] = "Amazon already logged in."
		         })
	else
		echoInfo("AmazonSocialService:login()")

		local function onAmazonLogin(resultStr, dataStr)
			self.userAmazon:unregisterEventHandler("login")
			echoInfo("Amazon result login")
			local result = json.decode(resultStr)
			local data = dataStr and json.decode(dataStr) or {}

			if result and result["type"] ~= nil then
				echoInfo("code:%s, msg:%s", result["code"], result["msg"])
			else
				echoInfo("Result does not contain a type")
			end
			callback(result, data)
		end

		self.userAmazon:registerEventHandler(onAmazonLogin, "login")
		self.userAmazon:login(params)
	end
end

function AmazonSocialService:logout(callback)
	-- self.userAmazon:registerEventHandler(function(resultStr, dataStr)
	-- 	self.userAmazon:unregisterEventHandler()
	-- 	echoInfo("Amazon result logout")
	-- 	local result = json.decode(resultStr)
	-- 	if result and result["type"] ~= nil then
	-- 		echoInfo("code:%s, msg:%s", result["code"], result["msg"])

	-- 		if result["type"] == "logout"then
	-- 			callback(result)
	-- 		end
	-- 	else
	-- 		echoInfo("Result does not contain a type")
	-- 	end
	-- end)
	-- self.userAmazon:logout()
	echoInfo("You cannot logout of Amazon yet.")
	callback({
	         ["type"] = "logout",
	         ["code"] = runaway.PluginResult.kSucceed,
	         ["msg"] = "Amazon logout not yet implemented"
	         })
end

function AmazonSocialService:getProfile(callback)
	self.userAmazon:registerEventHandler(function(resultStr, dataStr)
		self.userAmazon:unregisterEventHandler("profile")
		echoInfo("Amazon result getProfile")
		local result = json.decode(resultStr)
		if result and result["type"] ~= nil then
			echoInfo("code:%s, type:%s, msg:%s",result["code"],result["type"], result["msg"])
			if result["type"] == "profile" then
				local data = json.decode(dataStr)
				callback(result, data)
			end
		else
			echoInfo("Result does not contain a type")
		end
	end, "profile")
	self.userAmazon:getProfile()
end


function AmazonSocialService:friendsAvail()
	if self.shareAmazon == nil then
		echoInfo("shareAmazon not available on this platform")
		return false
	end

	return true
end

function AmazonSocialService:getImageForFriend(friend, callback)
	echoInfo("getImageForFriend not avilable with Amazon")
end

function AmazonSocialService:applyFriendData(friendAccount)
	echoInfo("applyFriendData not avilable with Amazon")
	return false
end

function AmazonSocialService:getFriends(callback)
	echoInfo("getFriends not avilable with Amazon")
end

function AmazonSocialService:updateRemote(params, callback) -- ANOTHER STUB FUNCTION
	iprint(params.data)
	local result = {code=0,type='updateRemote',msg='Update remote data success'}
	callback(result)
end

function AmazonSocialService:requestAvail()
	if self.shareAmazon == nil then
		echoInfo("shareAmazon not available on this platform")
		return false
	end

	return true
end

function AmazonSocialService:achievementsAvail()
	if self.socialAmazon == nil then
		echoInfo("socialAmazon not available on this platform")
		return false
	end

	return true
end

function AmazonSocialService:showAchievements()
	echoInfo("AmazonSocialService:showAchievements()")
	self.socialAmazon:showAchievements()
end

function AmazonSocialService:submitAchievement(achievementInfo, callback)
	self.socialAmazon:registerEventHandler(function(resultStr, dataStr)
		self.socialAmazon:unregisterEventHandler()
		-- echoInfo("Amazon submitAchievement result")
		local result = json.decode(resultStr)
		if result and result["type"] ~= nil then
			-- echoInfo("code:%s, type:%s, msg:%s",result["code"],result["type"], result["msg"])
			if result["type"] == "achievement" then
				callback(result)
			end
		else
			echoInfo("Result does not contain a type")
		end
	end)
	self.socialAmazon:submitAchievement(achievementInfo)
end

function AmazonSocialService:resetAchievements(callback)
	echoInfo("AmazonSocialService:resetAchievements()")
	self.socialAmazon:resetAchievements()
end

function AmazonSocialService:createAchievementInfo(goal)
	local goalId = runaway.GameConfig:isProduction()
		and	goal.release.googlePlayID
		or 	goal.staging.googlePlayID
	return {
		identifier = goalId,
		perComplete = 100
	}
end

return AmazonSocialService