local SocialService = require(cc.PACKAGE_NAME..".social.socialService")

local FacebookSocialService = class("FacebookSocialService", SocialService)

local GETPROFILE_CACHE_TIME = 20

EVENT_FB_APPLINK_DATA = "FacebookAppLinkData"

function FacebookSocialService:ctor()
	echoInfo("FacebookSocialService:ctor()")
	FacebookSocialService.super.ctor(self)
	local pluginManager = runaway.PluginManager:getInstance()
	self.facebook = pluginManager:loadPlugin("UserFacebook") -- login, profile
	self.shareFacebook = pluginManager:loadPlugin("ShareFacebook") -- share, friends, messages, requests
	self.isInited = false

	if self.facebook ~= nil then
		local appLinkData = self.facebook:callStringFuncWithParam("getAppLinkData", {})
		dump(appLinkData,"appLinkData")
		if appLinkData ~= "" then 
		    onAppLinkData(appLinkData)
		end

		callbackManager.listenForCallback(EVENT_FB_APPLINK_DATA, onAppLinkData)
	end
	    
end

function onAppLinkData(event)
	echoInfo("FacebookSocialService:onAppLinkData()")

    local url = nil
    if type(event) == "string" then
        url = event
    else
        local resultString = event:getUserDataString()
        url = resultString
    end

    dump(url, "url") --https://fb.me/1180885788606245?label=730596
    -- the "label" param is the sponsor userGameId
	local urlParams = utils.getQuerystringParams(url)
	if urlParams.label then
		if platformUserGameData.getSponsorId() == nil then
			platformUserGameData.setSponsorId(urlParams.label)
		end
	end
    
    platformUserGameData.setData("fb_attribution", url)

	local trackAdvertisedIdData = {}
	if url ~= nil and omniataPlugin ~= nil then
		trackAdvertisedIdData.facebook = url
		omniataPlugin.trackAdvertiserID(trackAdvertisedIdData)
	end
end

function FacebookSocialService:init(devInfo, serverManager)
	echoInfo("FacebookSocialService:init()")

	if self.isInited then
		echoInfo("Facebook social service already inited")
		return
	end

	self.isInited = true
	self.friendStash = {}
	self.grantedPermissions = {}
	self.declinedPermissions = {}
	
	self.facebook:configDeveloperInfo(devInfo)
	self.facebook:setDebugMode(true)

	self.shareFacebook:configDeveloperInfo(devInfo)
	self.shareFacebook:setDebugMode(true)

	self.serverManager = serverManager

	self.profile = nil
end

function FacebookSocialService:isAvailable()

	if self.facebook == nil then
		echoInfo("Facebook not available on this platform")
		return false
	end

	return true
end

function FacebookSocialService:getPermissions()
	echoInfo("FacebookSocialService:getPermissions")
	local dataStr = self.facebook:callStringFuncWithParam("getPermissions",{})
	local data = json.decode(dataStr)
	dump(data,"data")
	self.grantedPermissions = data.granted
	self.declinedPermissions = data.declined
end

function FacebookSocialService:checkPermission(permission)
	echoInfo("FacebookSocialService:checkPermission:%s", permission)
	for i,granted in ipairs(self.grantedPermissions) do
		if granted == permission then
			return true
		end
	end

	return false
end

function FacebookSocialService:checkPermissions(permissions)
	echoInfo("FacebookSocialService:checkPermissions")
	for i,permission in ipairs(permissions) do
		local result = self:checkPermission(permission)
		if not result then
			return false
		end 
	end
	
	return true
end

function FacebookSocialService:requestPermissions(permissions, callback)
	echoInfo("FacebookSocialService:requestPermissions")
	self:callFuncWithParam("requestPermissions", permissions, function(result, data)
		if result["code"] == runaway.PluginResult.kSucceed then
			echoInfo("permission success...")
		else
			echoInfo("permission failed, things may break")
		end
		callback(result, data)
	end)
	return true
end

function FacebookSocialService:checkAndRequestPermissions(permissions, callback)
	self:getPermissions()

	if self:checkPermissions(permissions) then
		echoInfo("permissions exist")
		callback(runaway.PluginResult.kSucceed)
	else
		echoInfo("permission don't exist")
		self:requestPermissions(permissions, callback)
	end
end

function FacebookSocialService:login(params, callback)
	echoInfo("FacebookSocialService:login()")
	local socialAccessToken = nil
	self.grantedPermissions = {}
	self.declinedPermissions = {}

	local onFacebookLogin
	local checkAndRequestFriendPermission
	local onRequestPermission
	local onGetProfile

	onFacebookLogin = function(resultStr, dataStr)
		-- dump(resultStr, "onFacebookLogin result")
		-- dump(dataStr, "onFacebookLogin data (this should be the facebook accessToken)")
		self.facebook:unregisterEventHandler("login")
		local result = json.decode(resultStr)

		if result and result["type"] ~= nil then
			echoInfo("code:%s, msg:%s", result["code"], result["msg"])
			if (dataStr) then
				socialAccessToken = dataStr
			end
			if result["code"] == runaway.PluginResult.kSucceed then 
				checkAndRequestFriendPermission()
				return
			end
		else
			echoInfo("Result does not contain a type")
		end
		callback(result, data)
	end

	checkAndRequestFriendPermission =function()
		echoInfo("checkAndRequestFriendPermission")
		self:checkAndRequestPermissions(params.permissions, onRequestPermission)
	end

	onRequestPermission = function(result, data)
		echoInfo("onRequestPermission")
		self:getProfile(onGetProfile)
	end

	onGetProfile = function(result, data)
		if data then
			if data.email then
				persistenceUser.username = data.name or ''
				persistenceUser.email = data.email
			end

			--	switch out the token_for_business and the gameSpecificId
			if USE_FACEBOOK_TOKEN_FOR_BUSINESS then
				data.gameSpecificId = data.id
				data.id = data.token_for_business
				data.tokenForBusiness = data.token_for_business
			end

			FacebookSocialService.saveProfile(data)
			callbackManager.sendCallback(EVENT_FACEBOOK_CONNECTED, {data = data, result = result})
		else
			data = {}
		end
		echoInfo("setting socialAccessToken = %s", socialAccessToken)
		data.socialAccessToken = socialAccessToken

		callback(result, data)
	end

	self.facebook:registerEventHandler(onFacebookLogin, "login")
	local permissionStr = json.encode(params.permissions)
	local paramsToSend = {}
	paramsToSend.permissions = permissionStr
	self.facebook:login(paramsToSend)
end

function FacebookSocialService.saveProfile(data)
	local platformData = {}			
	local keysToIgnore = {"username","email","id","token_for_business", "tokenForBusiness","gameSpecificId","first_name","last_name","name","socialAccessToken"}
	for k,v in pairs(data) do
		if not table.contains(keysToIgnore, k) then
         -- echoInfo("%s %s",k,v)
         platformData[k] = v
      end
	end

	platformUserGameData.setData("fb_profile", platformData)
end

function FacebookSocialService:isLogined()
	return self.facebook:isLogined()
end

function FacebookSocialService:getAccessToken(callback)
	local token = self.facebook:callStringFuncWithParam("getAccessToken", {})
	echoInfo("Facebook serviceToken = %s", token or "NIL")
	return token
end

function FacebookSocialService:logout(callback)
	if self.facebook:isLogined() then
		self.facebook:registerEventHandler(function(resultStr, dataStr)
			self.facebook:unregisterEventHandler("logout")
			echoInfo("Facebook result logout")
			local result = json.decode(resultStr)
			if result and result["type"] ~= nil then
				echoInfo("code:%s, msg:%s", result["code"], result["msg"])

				if result["type"] == "logout"then
					callback(result)
				end
			else
				echoInfo("Result does not contain a type")
			end
		end, "logout")
		self.profile = nil
		self.facebook:logout()
	else
		callback({code = runaway.PluginResult.kSucceed})
	end
end

function FacebookSocialService:getProfile(callback)
	echoInfo("Facebook getProfile")

	-- dump(self.profile ,"self.profile")
	if self.profile ~= nil and self.profile.time ~= nil and (os.time() - self.profile.time < GETPROFILE_CACHE_TIME) then
		echoInfo("Returning cached profile")
		local result = {}
		result["code"] = runaway.PluginResult.kSucceed
		callback(result, self.profile.data)
	else
		echoInfo("Querying profile from Facebook")
		self:callFuncWithParam("runQuery", {"/me?fields=token_for_business,name,last_name,first_name,gender"}, function(result, data) 
			-- dump(result, "facebook.getProfile() result")
			-- dump(data, "facebook.getProfile() data", 10)
			self.profile = {}
			self.profile.data = data
			self.profile.time = os.time() -- Not using timeManager.getTime() since it is not set when first require the profile
			callback(result, data)
		end, "getProfile")
	end

end

function FacebookSocialService:getImage(callback) --TODO this
	local result = {code=2,type='picture',msg='not yet implemented'}
	callback(result)
end

function FacebookSocialService:friendsAvail()
	if self.shareFacebook == nil then
		echoInfo("ShareFacebook not available on this platform")
		return false
	end

	return true
end

function FacebookSocialService:applyFriendData(friendAccount)
	--	find the facebook account info
	local facebookAccount = nil
	if type(friendAccount.accounts) == "table" then
		for k = 1,#friendAccount.accounts do
        	local account = friendAccount.accounts[k]
			if account.account_type == SOCIAL_FACEBOOK then
				facebookAccount = account
				break
			end
		end
	end

	if facebookAccount then
		--	now find the appropriate facebook friend information from the friendStash
		if type(self.friendStash) == "table" then
			for k = 1,#self.friendStash do
	        	local friend = self.friendStash[k]
				if (facebookAccount.account_id == friend.id) or (facebookAccount.game_specific_id == friend.id) then
					--	facebook has the highest priority, so just stomp over any existing data
					friendAccount.name = friend.name
					friendAccount.picUrl =  (friend.picture and friend.picture.data.url) or nil
					return true
				end
			end
		end
	end
	return false
end


function FacebookSocialService:getFriends(callback)
	echoInfo("FacebookSocialService:getFriends")
	self.shareFacebook:registerEventHandler(function (resultStr, dataStr)
		self.shareFacebook:unregisterEventHandler("friends")
		echoInfo("Facebook result friends")
		local result = json.decode(resultStr)
		if result and result["type"] ~= nil then
			echoInfo("code:%s, type:%s, msg:%s",result["code"],result["type"], result["msg"])
			if result["type"] == "friends" then
				local data = json.decode(dataStr)

				--	reformat the list to be consistent across all social types 
				if type(data) ~= "table" then
					echoInfo("data was nil!")
				else
					-- dump(data, "Facebook friends", 10)
					for k = 1,#data do
	        			local friend = data[k]
						friend.accountType = SOCIAL_FACEBOOK
					end
				end				
				
				-- store the friends list for later reference
				self.friendStash = data

				callback(result, data)
			end
		else
			echoInfo("Result does not contain a type")
		end
	end, "friends")
	self.shareFacebook:getFriends()
end

function FacebookSocialService:updateRemote(params, callback) -- ANOTHER STUB FUNCTION
	iprint(params.data)
	local result = {code=0,type='updateRemote',msg='Update remote data success'}
	callback(result)
end

function FacebookSocialService:requestAvail()
	if self.shareFacebook == nil then
		echoInfo("ShareFacebook not available on this platform")
		return false
	end

	return true
end

function FacebookSocialService:sendInvite(params, callback)
	self.shareFacebook:registerEventHandler(function (resultStr, dataStr)
		self.shareFacebook:unregisterEventHandler()
		echoInfo("Facebook result")
		local result = json.decode(resultStr)
		if result and result["type"] ~= nil then
			echoInfo("code:%s, type:%s, msg:%s",result["code"],result["type"], result["msg"])
			if result["type"] == "invite" then
				local data = json.decode(dataStr)
				callback(result, data)
			end
		else
			echoInfo("Result does not contain a type")
		end
	end)
	self.shareFacebook:sendInvite(params)
end

-- Fields:
--title [string]
-- message [string]
-- objectID [string]
-- data [string], 
-- actionType [enum] 0 = no action, 1 = user sending object, 2 = user is asking for object, 3 = turn action
-- filters [enum] 0 = none, 1 = App Users, 2 = Non App users
-- recipients [csv]
-- recipientSuggestions [csv]
function FacebookSocialService:sendRequest(params, callback)
	self.shareFacebook:registerEventHandler(function (resultStr, dataStr)
		self.shareFacebook:unregisterEventHandler()
		echoInfo("Facebook result")
		local result = json.decode(resultStr)
		if result and result["type"] ~= nil then
			echoInfo("code:%s, type:%s, msg:%s",result["code"],result["type"], result["msg"])
			if result["type"] == "request" then
				local data = json.decode(dataStr)
				callback(result, data)
			end
		else
			echoInfo("Result does not contain a type")
		end
	end)
	self.shareFacebook:sendRequest(params)
end

function FacebookSocialService:postsAvail()
	if self.shareFacebook == nil then
		echoInfo("ShareFacebook not available on this platform")
		return false
	end

	return true
end

function FacebookSocialService:post(params, callback)
	self.shareFacebook:registerEventHandler(function (resultStr, dataStr)
		self.shareFacebook:unregisterEventHandler()
		echoInfo("Facebook result")
		local result = json.decode(resultStr)
		if result and result["type"] ~= nil then
			echoInfo("code:%s, type:%s, msg:%s",result["code"],result["type"], result["msg"])
			if result["type"] == "share" then
				local data = json.decode(dataStr)
				callback(result, data)
			end
		else
			echoInfo("Result does not contain a type")
		end
	end)

	self.shareFacebook:share(params)
end

function FacebookSocialService:postPhoto(imagePath, message, callback)

	local function onGotPublishPermissions(result)
		if result == runaway.PluginResult.kSucceed then

			self.shareFacebook:registerEventHandler(function (resultStr, dataStr)
				self.shareFacebook:unregisterEventHandler("uploadPhoto")
				echoInfo("Facebook uploadPhoto result")
				dump(resultStr, "resultStr")
				dump(dataStr, "dataStr")
				local result = json.decode(resultStr)
				if result and result["type"] ~= nil then
					local data = json.decode(dataStr)
					callback(result, data)
				else
					echoInfo("Result does not contain a type")
				end
			end, "uploadPhoto")

			local data = {imagePath, message}
			--data = {json.encode(data)}
			self.shareFacebook:callFuncWithParam("uploadPhoto", data)
		else
			callback({code=result})
		end
	end

	local permissions = {}
	--	ensure that we have the correct permissions for publishing photos first
	self:checkAndRequestPermissions(permissions, onGotPublishPermissions)
end


function FacebookSocialService:callFuncWithParam(functionName, data, callback, listenerName)
	if listenerName == nil then
		listenerName = functionName
	else
		table.insert(data, listenerName)
	end

	self.facebook:registerEventHandler(function(resultStr, dataStr)
		self.facebook:unregisterEventHandler(listenerName)
		echoInfo("Facebook result from function %s", listenerName)
		local result = json.decode(resultStr)

		if result and result["type"] ~= nil then
			echoInfo("code:%s, msg:%s",result["code"], result["msg"])

			local data = json.decode(dataStr)
			if callback ~= nil then
				callback(result, data)
			end
		else
			echoInfo("Result does not contain a type")
		end
	end, listenerName)

	self.facebook:callFuncWithParam(functionName, data)
end

return FacebookSocialService