local SocialService = require(cc.PACKAGE_NAME..".social.socialService")

local GameCenterSocialService = class("GameCenterSocialService", SocialService)

function GameCenterSocialService:ctor()
	echoInfo("GameCenterSocialService:ctor()")
	GameCenterSocialService.super.ctor(self)
	local pluginManager = runaway.PluginManager:getInstance()
	self.userGameCenter = pluginManager:loadPlugin("UserGameCenter") -- login, profile
	self.shareGameCenter = pluginManager:loadPlugin("ShareGameCenter") -- friends
	self.socialGameCenter = pluginManager:loadPlugin("SocialGameCenter") -- cheevo's
end

function GameCenterSocialService:init(devInfo)
	echoInfo("GameCenterSocialService:init()")
	self.friendStash = {}

	self.userGameCenter:configDeveloperInfo(devInfo)
	self.userGameCenter:setDebugMode(true)

	self.shareGameCenter:configDeveloperInfo(devInfo)
	self.shareGameCenter:setDebugMode(true)

	self.socialGameCenter:configDeveloperInfo(devInfo)
	self.socialGameCenter:setDebugMode(true)
end

function GameCenterSocialService:isAvailable()

	if self.userGameCenter == nil then
		echoInfo("UserGameCenter not available on this platform")
		return false
	end

	return true
end

function GameCenterSocialService:isLogined()
	return self.userGameCenter:isLogined()
end

function GameCenterSocialService:login(params, callback)
	if self:isLogined() then
		--	gamecenter is never actually logged out. 
		--	if we log out and back in again, just return success.
		echoInfo("Attempting to log in GameCenter (already logged in")
		callback({
		         ["type"] = "login",
		         ["code"] = runaway.PluginResult.kSucceed,
		         ["msg"] = "GameCenter already logged in."
		         })
	else
		echoInfo("GameCenterSocialService:login()")

		local function onGameCenterLogin(resultStr, dataStr)
			self.userGameCenter:unregisterEventHandler("login")
			echoInfo("GameCenter result login")
			local result = json.decode(resultStr)
			local data = dataStr and json.decode(dataStr) or {}

			if result and result["type"] ~= nil then
				echoInfo("code:%s, msg:%s", result["code"], result["msg"])
			else
				echoInfo("Result does not contain a type")
			end
			callback(result, data)
		end

		self.userGameCenter:registerEventHandler(onGameCenterLogin, "login")
		self.userGameCenter:login(params)
	end
end


function GameCenterSocialService:getAccessToken()
	return nil
end


function GameCenterSocialService:logout(callback)
	-- self.userGameCenter:registerEventHandler(function(resultStr, dataStr)
	-- 	self.userGameCenter:unregisterEventHandler()
	-- 	echoInfo("GameCenter result logout")
	-- 	local result = json.decode(resultStr)
	-- 	if result and result["type"] ~= nil then
	-- 		echoInfo("code:%s, msg:%s", result["code"], result["msg"])

	-- 		if result["type"] == "logout"then
	-- 			callback(result)
	-- 		end
	-- 	else
	-- 		echoInfo("Result does not contain a type")
	-- 	end
	-- end)
	-- self.userGameCenter:logout()
	echoInfo("You cannot logout of GameCenter.")
	callback({
	         ["type"] = "logout",
	         ["code"] = runaway.PluginResult.kSucceed,
	         ["msg"] = "You cannot logout of GameCenter... this is by design of Apple."
	         })
end

function GameCenterSocialService:getProfile(callback)
	self.userGameCenter:registerEventHandler(function(resultStr, dataStr)
		self.userGameCenter:unregisterEventHandler("profile")
		echoInfo("GameCenter result getProfile")
		local result = json.decode(resultStr)
		if result and result["type"] ~= nil then
			echoInfo("code:%s, type:%s, msg:%s",result["code"],result["type"], result["msg"])
			if result["type"] == "profile" then
				local data = json.decode(dataStr)
				callback(result, data)
			end
		else
			echoInfo("Result does not contain a type")
		end
	end, "profile")
	self.userGameCenter:getProfile()
end


function GameCenterSocialService:friendsAvail()
	if self.shareGameCenter == nil then
		echoInfo("ShareGameCenter not available on this platform")
		return false
	end

	return true
end

function GameCenterSocialService:getImageForFriend(friend, callback)
	echoInfo("GameCenter getImageForFriend %s", friend.name or "unnamed")
	local GCAccount
	--	check if this friend is a gamecenter account
	if type(friend.accounts) == "table" then
		for k = 1,#friend.accounts do
	        local account = friend.accounts[k]
			if account.account_type == SOCIAL_GAMECENTER then
				GCAccount = account
				break
			end
		end
	end
	if GCAccount then
		echoInfo("this friend IS a gamecenter friend")
		local function getSpriteFromUIImage(UIImageRef)
			if UIImageRef == nil then
				return display.newSprite(DEFAULT_PROFILE_PICTURE or "#Avatar")				
			end
			
			local sprite = cc.Sprite:create(UIImageRef)
			if sprite then
				echoInfo('Successful conversion')
				return sprite
			else
				echoInfo('Unsuccessful conversion')
				return nil
			end
		end
		self.shareGameCenter:registerEventHandler(function (resultStr, data)
			self.shareGameCenter:unregisterEventHandler()
			echoInfo("GameCenter result getProfilePicture")
			local result = json.decode(resultStr)
			if result and result["type"] ~= nil then
				if result["type"] == "picture" then
					local sprite = getSpriteFromUIImage(data)
					callback(result, {picture = sprite})
				end
			else
				echoInfo("Result does not contain a type")
			end
		end)
		self.shareGameCenter:callFuncWithParam('getProfilePicture', {GCAccount.account_id})
	else
		result = {code=2,type='picture',msg='No profile picture info'}
		callback(result)
	end
end



function GameCenterSocialService:applyFriendData(friendAccount)
	--	find the facebook account info
	local gamecenterAccount = nil
	if type(friendAccount.accounts) == "table" then
		for k = 1,#friendAccount.accounts do
	        local account = friendAccount.accounts[k]
			if account.account_type == SOCIAL_GAMECENTER then
				gamecenterAccount = account
				break
			end
		end
	end

	if gamecenterAccount then
		--	now find the appropriate gamecenter friend information from the friendStash
		if type(self.friendStash) == "table" then
			for k = 1,#self.friendStash do
	        	local friend = self.friendStash[k]
				if gamecenterAccount.account_id == friend.id then
					-- don't overwrite any existing values
					if friendAccount.name == nil then
						friendAccount.name = friend.name
						
						--	doesn't have picture url in this blob
						--friendAccount.picUrl =  (friend.picture and friend.picture.data.url) or nil
					end
					return true
				end
			end
		end
	end
	return false 	--	 return true if the friend is using this social type
end

function GameCenterSocialService:getFriends(callback)
	echoInfo("GameCenter getFriends")
	self.shareGameCenter:registerEventHandler(function (resultStr, dataStr)
		self.shareGameCenter:unregisterEventHandler("friends")
		echoInfo("GameCenter result getFriends")
		local result = json.decode(resultStr)
		if result and result["type"] ~= nil then
			echoInfo("code:%s, type:%s, msg:%s",result["code"],result["type"], result["msg"])
			if result["type"] == "friends" then
				local data = json.decode(dataStr)

				--	reformat the list to be consistent across all social types 
				if type(data) ~= "table" then
					echoInfo("data was nil!")
				else
					for k = 1,#data do
	        			local friend = data[k]
						friend.accountType = SOCIAL_GAMECENTER
					end
				end

				-- store the friends list for later reference
				self.friendStash = data or {}

				callback(result, data)
			end
		else
			echoInfo("Result does not contain a type")
		end
	end, "friends")
	self.shareGameCenter:getFriends()
end

function GameCenterSocialService:updateRemote(params, callback) -- ANOTHER STUB FUNCTION
	iprint(params.data)
	local result = {code=0,type='updateRemote',msg='Update remote data success'}
	callback(result)
end

function GameCenterSocialService:requestAvail()
	if self.shareGameCenter == nil then
		echoInfo("ShareGameCenter not available on this platform")
		return false
	end

	return true
end

function GameCenterSocialService:achievementsAvail()
	if self.socialGameCenter == nil then
		echoInfo("SocialGameCenter not available on this platform")
		return false
	end

	return true
end

function GameCenterSocialService:showAchievements()
	echoInfo("GameCenterSocialService:showAchievements()")
	self.socialGameCenter:showAchievements()
end

function GameCenterSocialService:submitAchievement(achievementInfo, callback)
	self.socialGameCenter:registerEventHandler(function(resultStr, dataStr)
		self.socialGameCenter:unregisterEventHandler()
		-- echoInfo("GameCenter submitAchievement result")
		local result = json.decode(resultStr)
		if result and result["type"] ~= nil then
			-- echoInfo("code:%s, type:%s, msg:%s",result["code"],result["type"], result["msg"])
			if result["type"] == "achievement" then
				callback(result)
			end
		else
			echoInfo("Result does not contain a type")
		end
	end)
	self.socialGameCenter:submitAchievement(achievementInfo)
end

function GameCenterSocialService:resetAchievements(callback)
	echoInfo("GameCenterSocialService:resetAchievements()")
	self.socialGameCenter:resetAchievements()
end

function GameCenterSocialService:createAchievementInfo(goal)
	local gameCenterGoalId = runaway.GameConfig:isProduction()
		and	goal.release.gameCenterID
		or 	goal.staging.gameCenterID
	return {
		identifier = gameCenterGoalId,
		percentComplete = 100
	}
end

return GameCenterSocialService