local SocialService = require(cc.PACKAGE_NAME..".social.socialService")

local GooglePlaySocialService = class("GooglePlaySocialService", SocialService)

function GooglePlaySocialService:ctor()
	echoInfo("GooglePlaySocialService:ctor()")
	GooglePlaySocialService.super.ctor(self)
	local pluginManager = runaway.PluginManager:getInstance()
	self.userGooglePlay = pluginManager:loadPlugin("UserGooglePlay") -- login, profile
	-- self.shareGooglePlay = pluginManager:loadPlugin("ShareGooglePlay") -- friends
	self.socialGooglePlay = pluginManager:loadPlugin("SocialGooglePlay") -- cheevo's
end

function GooglePlaySocialService:init(devInfo)
	echoInfo("GooglePlaySocialService:init()")
	self.friendStash = {}

	self.userGooglePlay:configDeveloperInfo(devInfo)
	self.userGooglePlay:setDebugMode(true)

	-- self.shareGooglePlay:configDeveloperInfo(devInfo)
	-- self.shareGooglePlay:setDebugMode(true)

	self.socialGooglePlay:configDeveloperInfo(devInfo)
	self.socialGooglePlay:setDebugMode(true)
end

function GooglePlaySocialService:isAvailable()

	if self.userGooglePlay == nil then
		echoInfo("userGooglePlay not available on this platform")
		return false
	end

	return true
end

function GooglePlaySocialService:isLogined()
	return self.userGooglePlay:isLogined()
end

function GooglePlaySocialService:login(params, callback)
	if self:isLogined() then
		--	gamecenter is never actually logged out. 
		--	if we log out and back in again, just return success.
		echoInfo("Attempting to log in Google Play (already logged in")
		callback({
		         ["type"] = "login",
		         ["code"] = runaway.PluginResult.kSucceed,
		         ["msg"] = "Google Play already logged in."
		         })
	else
		echoInfo("GooglePlaySocialService:login()")

		local function onGooglePlayLogin(resultStr, dataStr)
			self.userGooglePlay:unregisterEventHandler("login")
			echoInfo("GooglePlay result login")
			local result = json.decode(resultStr)
			local data = dataStr and json.decode(dataStr) or {}

			if result and result["type"] ~= nil then
				echoInfo("code:%s, msg:%s", result["code"], result["msg"])
			else
				echoInfo("Result does not contain a type")
			end
			callback(result, data)
		end

		self.userGooglePlay:registerEventHandler(onGooglePlayLogin, "login")
		self.userGooglePlay:login(params)
	end
end

function GooglePlaySocialService:logout(callback)
	-- self.userGooglePlay:registerEventHandler(function(resultStr, dataStr)
	-- 	self.userGooglePlay:unregisterEventHandler()
	-- 	echoInfo("GooglePlay result logout")
	-- 	local result = json.decode(resultStr)
	-- 	if result and result["type"] ~= nil then
	-- 		echoInfo("code:%s, msg:%s", result["code"], result["msg"])

	-- 		if result["type"] == "logout"then
	-- 			callback(result)
	-- 		end
	-- 	else
	-- 		echoInfo("Result does not contain a type")
	-- 	end
	-- end)
	-- self.userGooglePlay:logout()
	echoInfo("You cannot logout of Google Play yet.")
	callback({
	         ["type"] = "logout",
	         ["code"] = runaway.PluginResult.kSucceed,
	         ["msg"] = "Google play logout not yet implemented"
	         })
end

function GooglePlaySocialService:getProfile(callback)
	self.userGooglePlay:registerEventHandler(function(resultStr, dataStr)
		self.userGooglePlay:unregisterEventHandler("profile")
		echoInfo("GooglePlay result getProfile")
		local result = json.decode(resultStr)
		if result and result["type"] ~= nil then
			echoInfo("code:%s, type:%s, msg:%s",result["code"],result["type"], result["msg"])
			if result["type"] == "profile" then
				local data = json.decode(dataStr)
				callback(result, data)
			end
		else
			echoInfo("Result does not contain a type")
		end
	end, "profile")
	self.userGooglePlay:getProfile()
end


function GooglePlaySocialService:friendsAvail()
	if self.shareGooglePlay == nil then
		echoInfo("shareGooglePlay not available on this platform")
		return false
	end

	return true
end

function GooglePlaySocialService:getImageForFriend(friend, callback)
	echoInfo("GooglePlay getImageForFriend %s", friend.name or "unnamed")
	local GCAccount
	--	check if this friend is a GooglePlay account
	if type(friend.accounts) == "table" then
		for k = 1,#friend.accounts do
	        local account = friend.accounts[k]
			if account.account_type == SOCIAL_GOOGLEPLAY then
				GCAccount = account
				break
			end
		end
	end
	if GCAccount then
		echoInfo("this friend IS a GooglePlay friend")
		local function getSpriteFromUIImage(UIImageRef)
			if UIImageRef == nil then
				return display.newSprite(DEFAULT_PROFILE_PICTURE or "#Avatar")				
			end
			
			local sprite = cc.Sprite:create(UIImageRef)
			if sprite then
				echoInfo('Successful conversion')
				return sprite
			else
				echoInfo('Unsuccessful conversion')
				return nil
			end
		end
		self.shareGooglePlay:registerEventHandler(function (resultStr, data)
			self.shareGooglePlay:unregisterEventHandler()
			echoInfo("GooglePlay result getProfilePicture")
			local result = json.decode(resultStr)
			if result and result["type"] ~= nil then
				if result["type"] == "picture" then
					local sprite = getSpriteFromUIImage(data)
					callback(result, {picture = sprite})
				end
			else
				echoInfo("Result does not contain a type")
			end
		end)
		self.shareGooglePlay:callFuncWithParam('getProfilePicture', {GCAccount.account_id})
	else
		result = {code=2,type='picture',msg='No profile picture info'}
		callback(result)
	end
end



function GooglePlaySocialService:applyFriendData(friendAccount)
	--	find the GP account info
	local googlePlayAccount = nil
	if type(friendAccount.accounts) == "table" then
		for k = 1,#friendAccount.accounts do
	        local account = friendAccount.accounts[k]
			if account.account_type == SOCIAL_GOOGLEPLAY then
				googlePlayAccount = account
				break
			end
		end
	end

	if googlePlayAccount then
		--	now find the appropriate GP friend information from the friendStash
		if type(self.friendStash) == "table" then
			for k = 1,#self.friendStash do
	        	local friend = self.friendStash[k]
				if googlePlayAccount.account_id == friend.id then
					-- don't overwrite any existing values
					if friendAccount.name == nil then
						friendAccount.name = friend.name
						
						--	doesn't have picture url in this blob
						--friendAccount.picUrl =  (friend.picture and friend.picture.data.url) or nil
					end
					return true
				end
			end
		end
	end
	return false 	--	 return true if the friend is using this social type
end

function GooglePlaySocialService:getFriends(callback)
	if self:friendsAvail() then
		echoInfo("GooglePlay getFriends")
		self.shareGooglePlay:registerEventHandler(function (resultStr, dataStr)
			self.shareGooglePlay:unregisterEventHandler("friends")
			echoInfo("GooglePlay result getFriends")
			local result = json.decode(resultStr)
			if result and result["type"] ~= nil then
				echoInfo("code:%s, type:%s, msg:%s",result["code"],result["type"], result["msg"])
				if result["type"] == "friends" then
					local data = json.decode(dataStr)

					--	reformat the list to be consistent across all social types 
					if type(data) ~= "table" then
						echoInfo("data was nil!")
					else
						for k = 1,#data do
	        				local friend = data[k]
							friend.accountType = SOCIAL_GOOGLEPLAY
						end
					end

					-- store the friends list for later reference
					self.friendStash = data or {}

					callback(result, data)
				end
			else
				echoInfo("Result does not contain a type")
			end
		end, "friends")
		self.shareGooglePlay:getFriends()
	else
		echoInfo("Skipping friend loading with google play")
		callback({code = runaway.PluginResult.kSucceed}, {})
	end
end

function GooglePlaySocialService:updateRemote(params, callback) -- ANOTHER STUB FUNCTION
	iprint(params.data)
	local result = {code=0,type='updateRemote',msg='Update remote data success'}
	callback(result)
end

function GooglePlaySocialService:requestAvail()
	if self.shareGooglePlay == nil then
		echoInfo("shareGooglePlay not available on this platform")
		return false
	end

	return true
end

function GooglePlaySocialService:achievementsAvail()
	if self.socialGooglePlay == nil then
		echoInfo("socialGooglePlay not available on this platform")
		return false
	end

	return true
end

function GooglePlaySocialService:showAchievements()
	echoInfo("GooglePlaySocialService:showAchievements()")
	self.socialGooglePlay:showAchievements()
end

function GooglePlaySocialService:submitAchievement(achievementInfo, callback)
	self.socialGooglePlay:registerEventHandler(function(resultStr, dataStr)
		self.socialGooglePlay:unregisterEventHandler()
		-- echoInfo("GooglePlay submitAchievement result")
		local result = json.decode(resultStr)
		if result and result["type"] ~= nil then
			-- echoInfo("code:%s, type:%s, msg:%s",result["code"],result["type"], result["msg"])
			if result["type"] == "achievement" then
				callback(result)
			end
		else
			echoInfo("Result does not contain a type")
		end
	end)
	self.socialGooglePlay:submitAchievement(achievementInfo)
end

function GooglePlaySocialService:resetAchievements(callback)
	echoInfo("GooglePlaySocialService:resetAchievements()")
	self.socialGooglePlay:resetAchievements()
end

function GooglePlaySocialService:createAchievementInfo(goal)
	local goalId = runaway.GameConfig:isProduction()
		and	goal.release.googlePlayID
		or 	goal.staging.googlePlayID
	return {
		identifier = goalId,
		perComplete = 100
	}
end

return GooglePlaySocialService