local SocialService = require(cc.PACKAGE_NAME..".social.socialService")

local TwitterSocialService = class("TwitterSocialService", SocialService)

local GETPROFILE_CACHE_TIME = 20

function TwitterSocialService:ctor()
	echoInfo("TwitterSocialService:ctor()")
	TwitterSocialService.super.ctor(self)
	local pluginManager = runaway.PluginManager:getInstance()
	self.userTwitter = pluginManager:loadPlugin("UserTwitter") -- login, profile
	self.shareTwitter = pluginManager:loadPlugin("ShareTwitter") -- share, friends, messages, requests
	self.isInited = false
end

function TwitterSocialService:init(devInfo)
	echoInfo("TwitterSocialService:init()")

	if self.isInited then
		echoInfo("Twitter social service already inited")
		return
	end

	-- self.userTwitter:configDeveloperInfo(devInfo)
	self.userTwitter:setDebugMode(true)

	-- self.shareTwitter:configDeveloperInfo(devInfo)
	self.shareTwitter:setDebugMode(true)

	self.cachedProfile = {}
	self.cachedProfile.data = nil
	self.cachedProfile.time = nil

	self.isInited = true
end

function TwitterSocialService:isAvailable()
	return self.userTwitter ~= nil
end

function TwitterSocialService:login(params, callback)
	print("Twitter login!")

	self.userTwitter:registerEventHandler(function(resultStr, dataStr)
		dump(resultStr, "onTwitterLogin result")
		self.userTwitter:unregisterEventHandler("login")
		local result = json.decode(resultStr)

		if result and result["type"] ~= nil then
			echoInfo("code:%s, msg:%s", result["code"], result["msg"])
		else
			echoInfo("Result does not contain a type")
		end

		local data = dataStr and json.decode(dataStr) or {}
		callback(result, data)
	end, "login")
	self.userTwitter:login(params)
end

function TwitterSocialService:isLogined()
	return self.userTwitter:isLogined()
end

function TwitterSocialService:logout(callback)
	if self.userTwitter:isLogined() then
		self.userTwitter:registerEventHandler(function(resultStr, dataStr)
			self.userTwitter:unregisterEventHandler("logout")
			echoInfo("Twitter result logout")
			local result = json.decode(resultStr)
			if result and result["type"] ~= nil then
				echoInfo("code:%s, msg:%s", result["code"], result["msg"])

				if result["type"] == "logout"then
					callback(result)
				end
			else
				echoInfo("Result does not contain a type")
			end
		end, "logout")
		self.cachedProfile = {}
		self.userTwitter:logout()
	else
		callback({code = runaway.PluginResult.kSucceed})
	end
end

function TwitterSocialService:getProfile(callback)
	if self.cachedProfile.data and (self.cachedProfile.time and (os.time() - self.cachedProfile.time < GETPROFILE_CACHE_TIME)) then
		local result = {}
		result["code"] = runaway.PluginResult.kSucceed
		callback(result, {
			self.cachedProfile.data.userID,
			self.cachedProfile.data.screenName
		})
		return
	end
	self.userTwitter:registerEventHandler(function(resultStr, dataStr)
		self.userTwitter:unregisterEventHandler("profile")
		echoInfo("Twitter result getProfile")
		local result = json.decode(resultStr)
		if result and result["type"] ~= nil then
			echoInfo("code:%s, type:%s, msg:%s",result["code"],result["type"], result["msg"])
			if result["type"] == "profile" then
				local data = json.decode(dataStr)
				self.cachedProfile.data = data
				self.cachedProfile.time = os.time()

				callback(result, {
					id = data.userID,
					name = data.name
				})
			else
				callback({code = runaway.PluginResult.kFailed})
			end
		else
			echoInfo("Result does not contain a type")
		end
	end, "profile")
	self.userTwitter:getProfile()
end

function TwitterSocialService:friendsAvail()
	return false
end

function TwitterSocialService:requestAvail()
	return false
end

function TwitterSocialService:postsAvail()
	if not self.shareTwitter then
		echoInfo("Twitter sharing not available on this platform")
		return false
	end
	return true
end

function TwitterSocialService:post(params, callback)
	self.shareTwitter:registerEventHandler(function(resultStr, dataStr)
		self.shareTwitter:unregisterEventHandler("share")
		echoInfo("Twitter result share")
		local result = json.decode(resultStr)
		if result and result["type"] ~= nil then
			echoInfo("code:%s, type:%s, msg:%s",result["code"],result["type"], result["msg"])
			if result["type"] == "share" then
				local data = json.decode(dataStr)
				callback(result, data)
			end
		else
			echoInfo("Result does not contain a type")
		end
	end, "share")
	if device.platform == 'android' then
		self.shareTwitter:callFuncWithParam("shareFromJSON", params)
	else
		self.shareTwitter:callFuncWithParam("share", params)
	end
end

function TwitterSocialService:callFuncWithParam(functionName, data, callback, listenerName)
	if listenerName == nil then
		listenerName = functionName
	else
		table.insert(data, listenerName)
	end

	self.userTwitter:registerEventHandler(function(resultStr, dataStr)
		self.userTwitter:unregisterEventHandler(listenerName)
		echoInfo("Twitter result from function %s", listenerName)
		local result = json.decode(resultStr)

		if result and result["type"] ~= nil then
			echoInfo("code:%s, msg:%s",result["code"], result["msg"])

			local data = json.decode(dataStr)
			if callback ~= nil then
				callback(result, data)
			end
		else
			echoInfo("Result does not contain a type")
		end
	end, listenerName)

	self.userTwitter:callFuncWithParam(functionName, data)
end

function TwitterSocialService:callBoolFuncWithParam(functionName, params)
	return self.userTwitter:callBoolFuncWithParam(functionName, params)
end

return TwitterSocialService