local ui = {}

ui.DEFAULT_TTF_FONT      = "1x/hud/fonts/Arial.ttf"
ui.DEFAULT_TTF_FONT_SIZE = 24

ui.TEXT_ALIGN_LEFT    = cc.TEXT_ALIGNMENT_LEFT
ui.TEXT_ALIGN_CENTER  = cc.TEXT_ALIGNMENT_CENTER
ui.TEXT_ALIGN_RIGHT   = cc.TEXT_ALIGNMENT_RIGHT
ui.TEXT_VALIGN_TOP    = cc.VERTICAL_TEXT_ALIGNMENT_TOP
ui.TEXT_VALIGN_CENTER = cc.VERTICAL_TEXT_ALIGNMENT_CENTER
ui.TEXT_VALIGN_BOTTOM = cc.VERTICAL_TEXT_ALIGNMENT_BOTTOM

ui.SYSTEM_DIALOG_BUTTON_OK          = "OK"
ui.SYSTEM_DIALOG_BUTTON_CANCEL      = "CANCEL"
ui.SYSTEM_DIALOG_BUTTON_YES         = "YES"
ui.SYSTEM_DIALOG_BUTTON_HELP        = "HELP"
ui.SYSTEM_DIALOG_BUTTON_LONG        = "LONG"

function ui.newTTFLabel(params)
    assert(type(params) == "table",
           "[framework.ui] newTTFLabel() invalid params")

    local text       = tostring(params.text)
    local font       = params.font or ui.DEFAULT_TTF_FONT
    local size       = params.size or ui.DEFAULT_TTF_FONT_SIZE
    local color      = params.color or display.COLOR_WHITE
    local textAlign  = params.align or ui.TEXT_ALIGN_LEFT
    local textValign = params.valign or ui.TEXT_VALIGN_CENTER
    local x, y       = params.x, params.y
    local dimensions = params.dimensions

    assert(type(size) == "number",
           "[framework.ui] newTTFLabel() invalid params.size")

    local label
    if dimensions then
        label = cc.LabelTTF:create(text, font, size, dimensions, textAlign, textValign)
    else
        label = cc.LabelTTF:create(text, font, size)
    end

    if label then
        -- CCNodeExtend.extend(label)
        label:setColor(color)

        function label:realign(x, y)
            if textAlign == ui.TEXT_ALIGN_LEFT then
                label:setPosition(math.round(x + label:getContentSize().width / 2), y)
            elseif textAlign == ui.TEXT_ALIGN_RIGHT then
                label:setPosition(x - math.round(label:getContentSize().width / 2), y)
            else
                label:setPosition(x, y)
            end
        end

        if x and y then label:realign(x, y) end
    end

    return label
end

function ui.newTTFLabelWithShadow(params)
    assert(type(params) == "table",
           "[framework.ui] newTTFLabelWithShadow() invalid params")

    local color       = params.color or display.COLOR_WHITE
    local shadowColor = params.shadowColor or display.COLOR_BLACK
    local x, y        = params.x, params.y

    local g = display.newNode()
    params.size = params.size
    params.color = shadowColor
    params.x, params.y = 0, 0
    g.shadow1 = ui.newTTFLabel(params)
    local offset = 1 / (display.widthInPixels / display.width)
    g.shadow1:realign(offset, -offset)
    g:addChild(g.shadow1)

    params.color = color
    g.label = ui.newTTFLabel(params)
    g.label:realign(0, 0)
    g:addChild(g.label)

    function g:setString(text)
        g.shadow1:setString(text)
        g.label:setString(text)
    end

    function g:realign(x, y)
        g:setPosition(x, y)
    end

    function g:getContentSize()
        return g.label:getContentSize()
    end

    function g:setColor(...)
        g.label:setColor(...)
    end

    function g:setShadowColor(...)
        g.shadow1:setColor(...)
    end

    function g:setOpacity(opacity)
        g.label:setOpacity(opacity)
        g.shadow1:setOpacity(opacity)
    end

    if x and y then
        g:setPosition(x, y)
    end

    return g
end

function ui.newTTFLabelWithOutline(params)
    assert(type(params) == "table",
           "[framework.ui] newTTFLabelWithShadow() invalid params")

    local color        = params.color or display.COLOR_WHITE
    local outlineColor = params.outlineColor or display.COLOR_BLACK
    local x, y         = params.x, params.y

    local g = display.newNode()
    params.size  = params.size
    params.color = outlineColor
    params.x, params.y = 0, 0
    g.shadow1 = ui.newTTFLabel(params)
    g.shadow1:realign(1, 0)
    g:addChild(g.shadow1)
    g.shadow2 = ui.newTTFLabel(params)
    g.shadow2:realign(-1, 0)
    g:addChild(g.shadow2)
    g.shadow3 = ui.newTTFLabel(params)
    g.shadow3:realign(0, -1)
    g:addChild(g.shadow3)
    g.shadow4 = ui.newTTFLabel(params)
    g.shadow4:realign(0, 1)
    g:addChild(g.shadow4)

    params.color = color
    g.label = ui.newTTFLabel(params)
    g.label:realign(0, 0)
    g:addChild(g.label)

    function g:setString(text)
        g.shadow1:setString(text)
        g.shadow2:setString(text)
        g.shadow3:setString(text)
        g.shadow4:setString(text)
        g.label:setString(text)
    end

    function g:getContentSize()
        return g.label:getContentSize()
    end

    function g:setColor(...)
        g.label:setColor(...)
    end

    function g:setOutlineColor(...)
        g.shadow1:setColor(...)
        g.shadow2:setColor(...)
        g.shadow3:setColor(...)
        g.shadow4:setColor(...)
    end

    function g:setOpacity(opacity)
        g.label:setOpacity(opacity)
        g.shadow1:setOpacity(opacity)
        g.shadow2:setOpacity(opacity)
        g.shadow3:setOpacity(opacity)
        g.shadow4:setOpacity(opacity)
    end

    if x and y then
        g:setPosition(x, y)
    end

    return g
end

function ui.newCompositeButton(params)
    assert(type(params) == "table",
           "[framework.ui] newCompositeButton() invalid params")    


    -- params
    local title = params.title or ""
    local buttonSize = params.buttonSize or cc.size(44, 42)
    local clickDepth = params.clickDepth or cc.p(0, -5)
    local clickDuration = params.clickDuration or 0.10
    local titleColor = params.titleColor or display.COLOR_WHITE
    local titleFontSize = params.titleFontSize or 10
    local titleFontName = params.titleFontName or FONT_ARIAL_ROUNDED
    --  requires the following sprite sheet loaded for default sprites:
    --    display.loadSpriteFrames("hud/buttons.plist", "hud/buttons.pvr.ccz")

    local buttonSpritePath = params.buttonSpritePath or '#ButtonRoundBlue'
    local shadowSpritePath = params.shadowSpritePath or nil-- '#ButtonRoundBlue'
    -- local frameSpritePath = params.frameSpritePath or '#ButtonFrameRound'
    -- local buttonSpritePath = params.buttonSpritePath or '#ButtonWideBlue'
    -- local shadowSpritePath = params.shadowSpritePath or  '#ButtonWideBlue_downState'

    local shadowNudge = params.shadowNudge or cc.p(0,0)
    local inlaySpritePath = params.inlaySpritePath
    local showInlay = inlaySpritePath and true
    local backgroundColor = params.backgroundColor
    local buttonOpacity = params.opacity or 255

    -- sound stuff
    local pressDownSound = params.pressDownSound or AUDIO_EVENT_SFX_BUTTONON
    local pressUpSound = params.pressUpSound or AUDIO_EVENT_SFX_BUTTONOFF
    local shouldPlaySound = params.shouldPlaySound or true

    --  load the sprites
    local buttonSprite = display.newSprite(buttonSpritePath)
    local shadowSprite = display.newSprite(shadowSpritePath)
    -- local frameSprite  = display.newSprite(frameSpritePath)

    --  calculations
    local buttonSpriteSize = buttonSprite:getContentSize()
    local shadowSize = shadowSprite:getContentSize()
    local halfClickDepth = cc.p(clickDepth.x / 2, clickDepth.y / 2)
    local buttonCenter = cc.p(buttonSize.width / 2, buttonSize.height / 2)
    local buttonSpriteCenter = cc.p(buttonCenter.x - halfClickDepth.x, buttonCenter.y - halfClickDepth.y)
    local shadowSpriteCenter = cc.p(buttonCenter.x + halfClickDepth.x, buttonCenter.y + halfClickDepth.y)
    local contentScaleX = buttonSize.width / (buttonSpriteSize.width + math.abs(clickDepth.x))
    local contentScaleY = buttonSize.height / (buttonSpriteSize.height + math.abs(clickDepth.y))


    --  layout and scale the sprites
    buttonSprite:setAnchorPoint(cc.p(0.5, 0.5))
    buttonSprite:setPosition(buttonSpriteCenter)
    buttonSprite:setScaleX(contentScaleX)
    buttonSprite:setScaleY(contentScaleY)
    buttonSprite:setOpacity(buttonOpacity)
    buttonSprite:setCascadeOpacityEnabled(true)

    shadowSprite:setAnchorPoint(cc.p(0.5, 0.5))
    shadowSprite:setPosition(cc.p(shadowSpriteCenter.x + shadowNudge.x, shadowSpriteCenter.y + shadowNudge.y))
    shadowSprite:setScaleX(contentScaleX)
    shadowSprite:setScaleY(contentScaleY)
    shadowSprite:setOpacity(buttonOpacity)
    shadowSprite:setCascadeOpacityEnabled(true)

    -- frameSprite:setAnchorPoint(cc.p(0.5, 0.5))
    -- frameSprite:setPosition(buttonSpriteCenter)
    -- frameSprite:setScaleX(contentScaleX)
    -- frameSprite:setScaleY(contentScaleY)
    -- frameSprite:setOpacity(buttonOpacity)

    --Creating movable layer
    local fixedLayer = ccui.Layout:create()
    fixedLayer:setContentSize(buttonSize)
    fixedLayer:addChild(shadowSprite);
    fixedLayer:setCascadeOpacityEnabled(true)
    -- fixedLayer:addChild(frameSprite);
    
    if backgroundColor then
        fixedLayer:setBackGroundColorType(ccui.LayoutBackGroundColorType.solid)
        fixedLayer:setBackGroundColor(backgroundColor)
        if backgroundColor.a then
            fixedLayer:setOpacity(backgroundColor.a)
        end
    end

    --Creating movable layer
    local movableLayer = ccui.Layout:create()
    movableLayer:setContentSize(buttonSize)
    movableLayer:addChild(buttonSprite)
    movableLayer.buttonSprite = buttonSprite
    movableLayer:setCascadeOpacityEnabled(true)

    local textLabel = nil;
    if showInlay then
        --  add the inlay glyph if specified
        local inlaySprite
        if type(inlaySpritePath) == "function" then
            inlaySprite = inlaySpritePath()
            inlaySprite:setAnchorPoint(cc.p(0.5, 0.5))
            inlaySprite:setPosition(buttonSpriteCenter)
            inlaySprite:setCascadeOpacityEnabled(true)
        else
            inlaySprite = display.newSprite(inlaySpritePath)
            inlaySprite:setAnchorPoint(cc.p(0.5, 0.5))
            inlaySprite:setPosition(buttonSpriteCenter)
            inlaySprite:setScaleX(contentScaleX)
            inlaySprite:setScaleY(contentScaleY)
            inlaySprite:setCascadeOpacityEnabled(true)
        end

        movableLayer:addChild(inlaySprite);
        movableLayer.inlaySprite = inlaySprite
    else
        --  otherwise add the button text
        local textLabel = ui.newTTFLabel({
                text = title,
                font = titleFontName,
                size = titleFontSize,
                color = titleColor,
                align = ui.TEXT_ALIGNMENT_CENTER,
            });
        textLabel:setPosition(buttonSpriteCenter)
        textLabel:setCascadeOpacityEnabled(true)
        movableLayer:addChild(textLabel);
    end
 
    --Create actual button, and attach the layers
    local button = runaway.CompositeButton:create(fixedLayer, movableLayer, clickDuration, clickDepth)
    button.fixedLayer = fixedLayer
    button.movableLayer = movableLayer
    button:setCascadeOpacityEnabled(true)

    button:setAnchorPoint(cc.p(0.0, 0.0))
    button:setContentSize(buttonSize)


    -- add a method to change the title
    button.titleNode = textLabel;
    function button:setTitle(text)
        if (button.titleNode) then
            button.titleNode:setString(text);
        end
    end

    local function makeSound(sender, eventType)
        if eventType == ccui.TouchEventType.began then
            if playSound and button.pressDownSound then
                audioManager.playRepeatable(button.pressDownSound)
            end
        elseif eventType == ccui.TouchEventType.ended then
            if playSound and button.pressUpSound then
                audioManager.playRepeatable(button.pressUpSound)
            end
        end
    end
    if shouldPlaySound then
        button:addTouchEventListener(makeSound)
    end

    return button
end


function ui.showSystemDialog(title, description, isError, buttonsList, callback, args)
    local ignoreConfigUIBuild = (args and args.ignoreConfigUIBuild) or false
    if CONFIG_UI_BUILD and not ignoreConfigUIBuild then
        return CONFIG_UI_BUILD(title, description, isError, buttonsList, callback, args)
    end

    local customTable = SYS_DIALOG_SETTINGS
    if customTable == nil then
        echoInfo("ERROR: SYS_DIALOG_SETTINGS not set in config.lua. Go sort that out!!!")
        return
    end

    -- Pretty sure the only thing in all three games 
    -- that uses this function is the GDPR dialog

    local dialogBase = nil
    local parentLayer = SYSTEMLAYER
  
    -- constants and calculations for later
    local buttonCount = #buttonsList
    local shortestSize = ((display.width > display.height) and display.height or display.width);
    local dialogWidthAdjust = 10
    local dialogHeightAdjust = 33
    local buttonAdjustHeight = 21
    local buttonAdjustWidth = 20
    local dialogZOrder = 100
    local dialogSize = customTable.dialogSize or cc.size((shortestSize / 1.4) + dialogWidthAdjust, (shortestSize / 1.6) + dialogHeightAdjust)
    local GDPRAcceptanceCallback

    -- Using this to generate labels in lieu of there being a custom one
    local function systemLabelMaker(args)
        local label
        if customTable.labelMaker then
            label = customTable.labelMaker(args)
            return label
        end

        local font = nil
        local string = ""
        local fontSize = 10
        local dimensions = cc.size(0,0)
        local hAlign = cc.TEXT_ALIGNMENT_CENTER
        local vAlign = cc.VERTICAL_TEXT_ALIGNMENT_CENTER

        if args.font          ~= nil then font         = args.font        end 
        if args.string        ~= nil then string       = args.string      end
        if args.fontSize      ~= nil then fontSize     = args.fontSize    end
        if args.dimensions    ~= nil then dimensions   = args.dimensions  end 
        if args.hAlign        ~= nil then hAlign       = args.hAlign      end 
        if args.vAlign        ~= nil then vAlign       = args.vAlign      end 

        label = cc.Label:createWithSystemFont(string, font, fontSize, dimensions, hAlign, vAlign)
        return label
    end

    local function backButtonEvent(title)
        -- Should the dialog be closed with android back button?
        if not args.blockBackButton then
            parentLayer:removeChild(dialogBase)
            local dialogCallback = dialogManager.removeCurrentDialog()
            if dialogCallback then dialogCallback(title) end
        end
    end

    local function buttonEvent(sender,eventType)
        if eventType == ccui.TouchEventType.ended then
            if sender.enabled then
                local title = sender.event.title
                parentLayer:removeChild(dialogBase)
                if GDPRAcceptanceCallback then GDPRAcceptanceCallback() end
                local dialogCallback = dialogManager.removeCurrentDialog()
                if dialogCallback then dialogCallback(title) end
            end
        end
    end

    local function makeButton(title, position)
        local inlaySpritePath, buttonSpritePath, buttonSize, shadowSpritePath

        if title == ui.SYSTEM_DIALOG_BUTTON_CANCEL then
            inlaySpritePath = customTable.assets.buttonIconCancel
            buttonSpritePath = customTable.assets.buttonImage
            buttonSize = customTable.buttonSize
            shadowSpritePath = customTable.assets.buttonBacking
        elseif title == ui.SYSTEM_DIALOG_BUTTON_LONG then
            inlaySpritePath = function()
                local labelArgs = {
                    string = SKELETON_MESSAGES.Accept, 
                    font = customTable.titleFont, 
                    fontSize = 10, 
                    dimensions = cc.size(0,0), 
                    hAlign = cc.TEXT_ALIGNMENT_CENTER, 
                    vAlign = cc.VERTICAL_TEXT_ALIGNMENT_CENTER
                }
                local text = systemLabelMaker(labelArgs)
                -- Button text color defaults to black
                local buttonTextColor = customTable.buttonTextColor or cc.c3b(0,0,0)
                text:setColor(buttonTextColor)
                return text
            end
            buttonSpritePath = customTable.assets.buttonLong
            shadowSpritePath = customTable.assets.buttonLong
            buttonSize = cc.size(customTable.buttonSize.width * 2, customTable.buttonSize.height)
        else
            inlaySpritePath = customTable.assets.buttonIconTick
            buttonSpritePath = customTable.assets.buttonImage
            buttonSize = customTable.buttonSize
            shadowSpritePath = customTable.assets.buttonBacking
        end

        local button = ui.newCompositeButton({ 
            buttonSize = buttonSize,
            clickDepth = cc.p(0.2, -0.2),
            clickDuration = 0.05,
            buttonSpritePath = buttonSpritePath,
            inlaySpritePath = inlaySpritePath,
            shadowSpritePath = shadowSpritePath 
        })
        if (customTable.buttonOffset) then
            button:setPosition(position.x + customTable.buttonOffset.x, position.y + customTable.buttonOffset.y)
        else
            button:setPosition(position)
        end
        button:setAnchorPoint(cc.p(0.5,0.5))
        button:addTouchEventListener(buttonEvent)
        button.event = {}
        button.event.title = title
        button.enabled = true
        return button
    end

    local function makeHyperlink(URL, text, maxWidth)
        local blue = cc.c3b(6,69,173)
        local labelArgs = {
            string = text, 
            font = customTable.contentFont, 
            fontSize = 8, 
            dimensions = cc.size(maxWidth,0), 
            hAlign = cc.TEXT_ALIGNMENT_LEFT, 
            vAlign = cc.VERTICAL_TEXT_ALIGNMENT_CENTER
        }
        local label = systemLabelMaker(labelArgs)
        label:enableUnderline()
        label:setColor(blue)

        local labelSize = label:getContentSize()
        local buttonBounds = ccui.Layout:create()
        buttonBounds:setContentSize(labelSize)
        buttonBounds:setTouchEnabled(true)
        buttonBounds:setSwallowTouches(true)
        buttonBounds:addTouchEventListener(function(sender, eventType) 
            if eventType == ccui.TouchEventType.ended then 
                device.openURL(URL)
            end
        end)

        label:addChild(buttonBounds)
        return label
    end

    dialogBase = ccui.Layout:create()
    dialogBase:setPosition(cc.p(0,0))

    local opaqueLayer  = ccui.Layout:create()
    opaqueLayer:setBackGroundColorType(ccui.LayoutBackGroundColorType.solid)
    opaqueLayer:setBackGroundColor(cc.c4b(0,0,0,128))
    opaqueLayer:setOpacity(128)
    opaqueLayer:setTouchEnabled(true) --  to swallow any touches
    opaqueLayer:setContentSize(cc.size(display.width, display.height))
    opaqueLayer:setPosition(cc.p(0,0))
    dialogBase:addChild(opaqueLayer)
    dialogBase.opaqueLayer = opaqueLayer

    local mainBacking = display.newScale9Sprite(customTable.assets.panel9Slice)
    mainBacking:setContentSize(dialogSize)

    dialogBase:addChild(mainBacking)
    dialogBase.mainBacking = mainBacking

    if customTable.assets.titleBacking then
        local titleBacking = display.newScale9Sprite(customTable.assets.titleBacking)
        titleBacking:setAnchorPoint(0.5, 0.5)
        titleBacking:setPosition(dialogSize.width * 0.5, dialogSize.height)
        titleBacking:setScale(1.5, 1.5)
        mainBacking:addChild(titleBacking) 
    end

    local textNode = display.newNode()
    dialogBase:addChild(textNode)
    dialogBase.textNode = textNode
    local labelArgs = {
        string = title, 
        font = customTable.titleFont, 
        fontSize = 12, 
        dimensions = cc.size(0,0), 
        hAlign = cc.TEXT_ALIGNMENT_CENTER, 
        vAlign = cc.VERTICAL_TEXT_ALIGNMENT_TOP
    }
    local titleText = systemLabelMaker(labelArgs)
    titleText:setAnchorPoint(cc.p(0.5, 1))
    titleText:enableBold()
    if isError then
        titleText:setColor(cc.c4b(255, 0, 0, 128))
    else
        titleText:setColor(customTable.titleColor or cc.c3b(76,89,84))
    end

    textNode:addChild(titleText)

    local labelArgs = {
        string = description, 
        font = customTable.contentFont, 
        fontSize = 10, 
        dimensions = cc.size(200,0), 
        hAlign = SYS_DIALOG_SETTINGS.contentHorizontalAlignment or cc.TEXT_ALIGNMENT_LEFT,
        vAlign = cc.VERTICAL_TEXT_ALIGNMENT_TOP
    }
    local contentText = systemLabelMaker(labelArgs)
    contentText:setAnchorPoint(cc.p(0.5, 1))
    if isError then
        contentText:setColor(cc.c4b(64, 64, 64, 128))
    else
        contentText:setColor(cc.c3b(64,101,125))
    end
    textNode:addChild(contentText)

    local topShim = 10
    local bottomShim = 50
    local titleHeight = titleText:getContentSize().height
    local contentsHeight = contentText:getContentSize().height
    local gapBetweenTitleAndContents = customTable.gapBetweenTitleAndContents or 5
    local totalHeight = titleHeight + contentsHeight + gapBetweenTitleAndContents + topShim + bottomShim

    textNode:setPosition(display.cx, display.cy)
    mainBacking:setPosition(display.cx, display.cy)
    if not customTable.dialogSize then
        mainBacking:setContentSize(dialogSize.width, totalHeight)
    end
   
    if customTable.assets.titleBacking then
        titleText:setPositionY(dialogSize.height * 0.5 + (titleHeight * 0.5))
    else
        titleText:setPositionY((totalHeight * 0.5) - topShim)
    end
    
    contentText:setPositionY(titleText:getPositionY() - titleHeight - gapBetweenTitleAndContents)

    local buttonPosition = 0
    local buttonSpacing = 0
    local dialogButtonAreaWidth = dialogSize.width + buttonAdjustWidth
    local decorSprite = nil

    --Temp turn off the postivie dialog
    if isError then
        if customTable.assets.errorDecor then
            decorSprite = cc.Sprite:createWithSpriteFrameName(customTable.assets.errorDecor)
            decorSprite:setPosition(dialogSize.width/2, buttonAdjustHeight - 2)
        end
    else
        if customTable.assets.Decor then
            decorSprite = display.newSprite(customTable.assets.Decor)
            decorSprite:setScale(mainBacking:getContentSize().width / decorSprite:getContentSize().width)
            decorSprite:setPosition((decorSprite:getContentSize().width * decorSprite:getScaleX())/2 - 1, 
                                    (decorSprite:getContentSize().height * decorSprite:getScaleY())/2 + 2)
        end
    end
    if decorSprite then
        mainBacking:addChild(decorSprite)
        dialogBase.decorSprite = decorSprite
    end

    -- Make Buttons
    local buttons = {}
    buttonPosition = cc.p(0, buttonAdjustHeight);
    if (buttonCount == 1) then
        -- Single button
    else
        local buttonSpreadWidth = (dialogButtonAreaWidth) - (customTable.buttonSize.width * 2);
        buttonSpacing = buttonSpreadWidth / (buttonCount - 1)
    end

    for k,buttonName in pairs(buttonsList) do
        local uiButton = makeButton(buttonName, buttonPosition)
        local buttonWidth = customTable.buttonSize.width
        if buttonName == ui.SYSTEM_DIALOG_BUTTON_LONG then 
            buttonWidth = buttonWidth * 1.5
        end
        mainBacking:addChild(uiButton)
        table.insert(buttons, uiButton)
        if buttonCount == 1 then
            uiButton:setPosition(dialogSize.width/2 + ((dialogButtonAreaWidth - buttonWidth * 2)/2), uiButton:getPositionY())
        else
            uiButton:setPosition(dialogSize.width/2 - ((dialogButtonAreaWidth - buttonWidth * 2)/2) + ((k-1) * buttonSpacing), uiButton:getPositionY())
        end
    end

    -- checkbox for GDPR:
    local checkBox
    if args and args.GDPR then
        if customTable.assets.buttonCheckbox then
            local okButton = buttons[1]
            okButton:setOpacity(50)
            okButton.enabled = false
            local okButtonPos = cc.p(okButton:getPosition())
            local checkCallback = function(enabled)
                okButton.enabled = enabled
                if enabled then
                    okButton:setOpacity(255) else
                    okButton:setOpacity(50)
                end
            end

            local checkboxBacking = display.newSprite(customTable.assets.buttonCheckbox)
            local checkboxTick = display.newSprite(customTable.assets.buttonCheckTick)
            local newScale = utils.resizeSprite(checkboxBacking, customTable.buttonSize)
            checkboxTick:setScale(newScale)
            checkboxTick:setVisible(false)
            GDPRAcceptanceCallback = function()
                if persistenceUserGame then
                    persistenceUserGame.gdpr_acceptance = true
                    -- persistence.save()
                end
            end

            -- Layout depends whether checkbox is on the left or right
            if customTable.checkBoxLeft then
                local labelArgs = {
                    string = SKELETON_MESSAGES.PrivacyPolicy, 
                    font = customTable.contentFont, 
                    fontSize = 8, 
                    dimensions = cc.size(100,0), 
                    hAlign = customTable.checkBoxLeftHAlign or cc.TEXT_ALIGNMENT_CENTER, 
                    vAlign = cc.VERTICAL_TEXT_ALIGNMENT_CENTER
                }
                local agreeText = systemLabelMaker(labelArgs)
                local agreeTextSize = agreeText:getContentSize()
                agreeText:setColor(customTable.agreeTextColor or cc.c3b(0,0,0))
                agreeText:setAnchorPoint(0.5,1)
                local gapBetweenCOntentsAndAgreeText = 20
                local agreeTextYPosition = contentText:getPositionY() - (contentText:getContentSize().height * 0.5) - (agreeTextSize.height * 0.5) - gapBetweenCOntentsAndAgreeText
                local agreeTextXPosition = 15 -- Offset to agreeText and checkBox appear centered together
                agreeText:setPositionY(agreeTextYPosition)
                agreeText:setPositionX(agreeTextXPosition)

                if (customTable.agreeTextOffset ~= nil) then
                    agreeText:setPosition(agreeText:getPositionX() + customTable.agreeTextOffset.x, agreeText:getPositionY() + customTable.agreeTextOffset.y)
                end
                textNode:addChild(agreeText)

                checkBox = ui.createCheckBox(checkboxBacking, checkboxTick, checkCallback)
                checkBox:setAnchorPoint(1, 1)
                checkBox:setPosition((-agreeTextSize.width * 0.5) - (checkBox:getContentSize().width * 0.5), agreeTextYPosition - (checkBox:getContentSize().height * 0.5) - 15)
                checkBox.enabled = false
                textNode:addChild(checkBox)
            else
                checkBox = ui.createCheckBox(checkboxBacking, checkboxTick, checkCallback)
                checkBox:setPosition(okButtonPos.x - (customTable.buttonSize.width * 1.5), okButtonPos.y)
                checkBox.enabled = false
                mainBacking:addChild(checkBox)
                
                local labelArgs = {
                    string = SKELETON_MESSAGES.PrivacyPolicy, 
                    font = customTable.contentFont, 
                    fontSize = 8, 
                    dimensions = cc.size(100,0), 
                    hAlign = cc.TEXT_ALIGNMENT_RIGHT, 
                    vAlign = cc.VERTICAL_TEXT_ALIGNMENT_CENTER
                }
                local agreeText = systemLabelMaker(labelArgs)
                local agreeTextSize = agreeText:getContentSize()
                agreeText:setColor(cc.c3b(0,0,0))
                agreeText:setAnchorPoint(1,0.5)
                agreeText:setPosition(checkBox:getPositionX() - (customTable.buttonSize.width * 0.5) - 5, okButtonPos.y)
                mainBacking:addChild(agreeText)
            end

            local privacyPolicy = makeHyperlink(RUNAWAY_PRIVACY_POLICY, SKELETON_MESSAGES.PrivacyPolicyTitle, 0)
            local privacyPolicySize = privacyPolicy:getContentSize()
            privacyPolicy:setAnchorPoint(0,0)
            -- Offset from dialog bottom left
            if customTable.privacyPolicyOffset then
                privacyPolicy:setPosition((dialogSize.width * -0.5) + customTable.privacyPolicyOffset.x, (dialogSize.height * -0.5) + customTable.privacyPolicyOffset.y)
            else
                privacyPolicy:setPosition((dialogSize.width * -0.5) + 5, (totalHeight * -0.5) + 8)
            end
            textNode:addChild(privacyPolicy)
        end
    end

    dialogBase.buttons = buttons

    --  display it.
    parentLayer:addChild(dialogBase, dialogZOrder)
    return function(title) backButtonEvent(title) end
end


function ui.createCheckBox(backing, icon, callback)
    local result = display.newNode()
    result:addChild(backing)
    result:addChild(icon)
    result.enabled = true

    function result.setEnabled(bool)
        result.enabled = bool
        if bool then
            icon:setVisible(true)
        else
            icon:setVisible(false)
        end
    end

    function result.toggle()
        result.setEnabled(not result.enabled)
        callback(result.enabled)
    end

    local touchSize = backing:getContentSize()
    local buttonBounds = ccui.Layout:create()
    buttonBounds:setContentSize(touchSize)
    buttonBounds:setPosition(-touchSize.width/2, -touchSize.height/2)
    buttonBounds:setTouchEnabled(true)
    buttonBounds:setSwallowTouches(true)
    buttonBounds:addTouchEventListener(function(sender, eventType) 
        if eventType == ccui.TouchEventType.ended then 
            result.toggle()
        end
    end)
    result:addChild(buttonBounds)
    return result
end


return ui
