local M = {}

M.data = {}

function M.init()
end

function M.setDeviceInfo()
	M.setData("lastDeviceId", device.getDeviceIdentifier())
	M.setData("vendorId", device.getVendorIdentifier())
	M.setData("deviceId", device.getDeviceModel())

	-- Android 7.0 has a strange issue which causes it to take ages to come back with
	-- the device's advertiser id, and it comes back as an empty string when it does.
	-- This was causing a black screen for a good 20 seconds on game launch.
	-- This fix is a bit dirty, but it just assumes the advertiser id is an empty string 
	-- if the device is running 7.0 or above, instead of calling away for it.
	local advertiserId
	local useEmptyAdvertiserId = false
	if device.client_platform == 'android' then
		local systemVersion = device.getSystemVersion()
		if systemVersion then
			local versionSplit = utils.splitString(tostring(systemVersion), "%.")
			local versionNumber = tonumber(versionSplit[1])
			if versionNumber and versionNumber > 6 then
				useEmptyAdvertiserId = true
			end
		end
	end
	if useEmptyAdvertiserId then
		advertiserId = ""
	else
		advertiserId = device.getAdvertiserIdentifier()
	end
	M.setData("advertiserId", advertiserId)
end

function M.getDataForKey(key)
	return M.data[key]
end

function M.setData(key, value)
	M.data[key] = value
end

function M.restoreData(newData)
	if newData ~= nil then
		for key, value in pairs(newData) do
			M.data[key] = value
		end
	end

	M.setDeviceInfo()
end

function M.getData()
	return M.data
end

function M.clearData()
	M.data = {}
end

function M.setSponsorCheckFunc(func)
	M.sponsorCheckFunc = func
end

function M.setSponsorId(sponsorUserId)
	if persistenceUserGame.sponsor_parent_id == nil then
		local canSetSponsor = true
		if type(M.sponsorCheckFunc) == "function" then
			canSetSponsor = M.sponsorCheckFunc()
		end

		if canSetSponsor then 
			persistenceUserGame.sponsor_parent_id = sponsorUserId

			local contents = {
				eventName = 'inviteAccepted',
				senderID = gameManager.userId,
				recipientID = sponsorUserId
			}
			mailboxManager.sendMail(sponsorUserId, {mailType = MAIL_SOCIAL, data = contents})
		else
			echoError("cannot set sponsorParent, sponsorCheckFunc failed")
		end
	else
		echoError("cannot set sponsorParent, persistenceUserGame.sponsor_parent_id ~= nil")
	end
end

function M.getSponsorId()
	return persistenceUserGame.sponsor_parent_id
end

function M.checkSponsorId()
    if persistenceUser.sponsor_parent_id == nil then
        if M.data.fb_attribution then
        	-- check facebook 
			local fbUrlParams = utils.getQuerystringParams(M.data.fb_attribution)
			if fbUrlParams and fbUrlParams.label then
				M.setSponsorId(fbUrlParams.label)
			end
		elseif M.data.attribution and 
			M.data.attribution.clickLabel and 
			M.data.attribution.clickLabel ~= "null" and
			M.data.attribution.clickLabel ~= "(null)" then
			-- check adjust
			M.setSponsorId(M.data.attribution.clickLabel)
        end
    end
end

return M